// ASP.cpp : Implementation of CASP
#include "stdafx.h"
#include "Generic_ToolAdaptor_ActiveX.h"
#include "ASP.h"
#include "TA_XSLT_Support.h"

#include <sstream>

extern void dummy(void);
extern void LoadBstrSafeArray(CStringList& dest, SAFEARRAY* psa);
extern bool __find_pstruct(list<PARADIGM_STRUCT> &lps1, list<PARADIGM_STRUCT> &lps2);

bool initialized = false;

struct ASP_ARG_LIST {
	CASP *casp;
	string name;
	string version;
	string type;
};

/////////////////////////////////////////////////////////////////////////////
// CASP

STDMETHODIMP CASP::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IASP,
	};
	for (int i=0;i<sizeof(arr)/sizeof(arr[0]);i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CASP::OnStartPage (IUnknown* pUnk)  
{
	if(!pUnk)
		return E_POINTER;

	CComPtr<IScriptingContext> spContext;
	HRESULT hr;

	// Get the IScriptingContext Interface
	hr = pUnk->QueryInterface(IID_IScriptingContext, (void **)&spContext);
	if(FAILED(hr))
		return hr;

	// Get Request Object Pointer
	hr = spContext->get_Request(&m_piRequest);
	if(FAILED(hr))
	{
		spContext.Release();
		return hr;
	}

	// Get Response Object Pointer
	hr = spContext->get_Response(&m_piResponse);
	if(FAILED(hr))
	{
		m_piRequest.Release();
		return hr;
	}
	
	// Get Server Object Pointer
	hr = spContext->get_Server(&m_piServer);
	if(FAILED(hr))
	{
		m_piRequest.Release();
		m_piResponse.Release();
		return hr;
	}
	
	// Get Session Object Pointer
	hr = spContext->get_Session(&m_piSession);
	if(FAILED(hr))
	{
		m_piRequest.Release();
		m_piResponse.Release();
		m_piServer.Release();
		return hr;
	}

	// Get Application Object Pointer
	hr = spContext->get_Application(&m_piApplication);
	if(FAILED(hr))
	{
		m_piRequest.Release();
		m_piResponse.Release();
		m_piServer.Release();
		m_piSession.Release();
		return hr;
	}
	m_bOnStartPageCalled = TRUE;

	return S_OK;
}

STDMETHODIMP CASP::OnEndPage ()  
{
	m_bOnStartPageCalled = FALSE;
	// Release all interfaces
	m_piRequest.Release();
	m_piResponse.Release();
	m_piServer.Release();
	m_piSession.Release();
	m_piApplication.Release();

	return S_OK;
}


STDMETHODIMP CASP::getExistingDocuments(BSTR *ret) 
{

	CString documents = "";

  bool exc = false;
  std::string errtxt;
	try {

		this->existing_documents.clear();
		this->existing_documents = this->ta->getDocuments();

		int offset = 0;
		bool corresponding_document = false;
		for(vector<DOCUMENT_STRUCT>::iterator i=this->existing_documents.begin(); i!=this->existing_documents.end(); i++) {
			list<PARADIGM_STRUCT> _document_paradigms = (*i).paradigms;

			bool founded = false;
			list<PARADIGM_STRUCT> __paradigms;
			for(list<CONFIG>::iterator qq=this->config.begin(); qq!=this->config.end(); qq++) {
				__paradigms = (*qq).paradigms;		
				if (__find_pstruct(__paradigms, _document_paradigms)) founded = true;
			}
			
			if (founded) {

				string temp_paradigms;
				for (list<PARADIGM_STRUCT>::iterator q=_document_paradigms.begin(); q!=_document_paradigms.end(); q++) {
					if (++q==_document_paradigms.end()) {
						--q;
						temp_paradigms += (*q).name + " v" + (*q).version;
					} else {
						--q;
						temp_paradigms += (*q).name + " v" + (*q).version + ",";
					}
				}	

				string temp = (*i).name + " v" + (*i).version + " (" + (*i).remarks + ")" + " (" + temp_paradigms + ")";
				corresponding_document = true;

				char _temp[10];
				ltoa((*i).docID, _temp, 10);
				documents += (CString)_temp + (CString)"\n" + (CString)temp.c_str() + (CString)"\n\n";
			}
		}

	}
  catch (udmcorba_exception & e)
  {
     errtxt = std::string(e.what());
     exc = true;
	}
  catch (udm_exception & e)
  {
    errtxt = std::string(e.what());
    exc = true;
	}
	
  if (exc)
  {
    documents = "hiba";
    documents += CString(errtxt.c_str());
  }

	*ret = documents.AllocSysString();

	return S_OK;

}


// Physically download the document on the server side
STDMETHODIMP CASP::requestDocument(BSTR extension, BSTR mga_par, BSTR directory, long doc_id, BSTR xslt_filename, BSTR *err) 
{

  bool exc = false;
  std::string documents;
  std::string errtxt;

try 
{
  for(map<long, ASP_SUBSCRIPTION>::iterator i= subscriptions.begin(); i!=subscriptions.end(); i++) 
  {

	  string ff_name = "";
	  string xsd_file = "";

	  CString filetitle;
	  CString temp_filename;
	  CString xsl_file = "";

	  ASP_my_ToolAdaptor *_ta = (*i).second.fetch_ta;

	  map<long, DOCUMENTS>::iterator j = _ta->documents.find(doc_id);
	  if (j == _ta->documents.end())
    {
		  errtxt  = "Unable to find the corresponding document!";
      exc = true;
      break;
	  }

	  // Get the rootobject
	  Uml::Class rootobject = _ta->getDocumentRootobjectMeta(doc_id, _ta->meta_diagram);
	  if (!rootobject)
    {
		  errtxt = "Could not find the meta of the rootobject of the document! Could not save it.";
      exc = true;
      break;
	  }

	  _chdir((LPCSTR)(CString)directory);

	  // create unique filename
	  time_t ltime;
	  time( &ltime );
	  char _uniq_temp[50];
	  ltoa(ltime, _uniq_temp, 10);

	  CString filename =  (CString)(*j).second.name.c_str() + "_v" + (CString)(*j).second.version.c_str() + 
		  "_temp" + (CString)_uniq_temp + "." + (CString)extension;

	  if (filename.Right(4)==".xml")
    {
		  // xml layer specific metalocator
		  ff_name = _ta->dn->getParadigmName() + "_v" + _ta->dn->getParadigmVersion();

		  // Create temporary xsd for the paradigm			
		  xsd_file = ff_name + ".xsd";
		  OTIF_XSLT::GenerateXMLSchemaFile(_ta->meta_diagram, xsd_file.c_str());

		  if (xsl_file.Compare("")) {
			  temp_filename = filename;
			  filename = "__temp_" + filetitle + "_xslt.xml";
		  }

	  }
    else if (filename.Right(4)==".mga")
    {
		  // mga layer specific metalocator
		  ff_name = (CString)mga_par;
	  }

	  // Create the new datanetwork
	  _ta->dn->CreateNew((LPCSTR)(filename), ff_name.c_str(), rootobject, Udm::CHANGES_PERSIST_ALWAYS);

	  _ta->dn->fetchDocument(doc_id);
	  _ta->dn->SaveAs((LPCSTR)(filename));
	  _ta->dn->CloseNoUpdate();
	  _ta->dn->release();


	  // XSLT translation
	  CString xslt_filename_c = xslt_filename;
	  xslt_filename_c.Replace("\\", "\\\\");

	  if (xslt_filename_c.Compare("") && filename.Right(4)==".xml") 
    {

		  // do the transform
		  int result = 0;
		  try
      {
			  OTIF_XSLT::XSLTransformer xslt;
			  result = xslt.executeTransform((LPCSTR)filename, (LPCSTR)xslt_filename_c, (LPCSTR)(filename + "_xslt.xml"), false, "");

		  }
      catch (OTIF_XSLT::xslt_exception &e)
      {
        errtxt = e.what();
        exc = true;
        break;
		  }

		  if (result!=0)
      {
			  errtxt = "Could not transform the document with the XSLT script";
        exc = true;
        break;
		  }

		  DeleteFile(filename);
		  filename = filename + "_xslt.xml";
		  xsd_file = "";
	  }


	  (*j).second.filename = filename; 
	  (*j).second.xsd_filename = xsd_file;

    documents += filename + "\n";
    documents += xsd_file;
  }
}
catch (udmcorba_exception &e) 
{
	errtxt  = e.what();
	exc = true;
} 
catch (udm_exception &e) 
{
	errtxt  = e.what();
  exc = true;
}
catch (...)
{
	errtxt = "Fatal error!";
	exc = true;
}

  if (exc)
  {
    documents = "hiba";
    documents += CString(errtxt.c_str());
  }
  CString ret(documents.c_str());
	*err = ret.AllocSysString();

	return S_OK;
}


STDMETHODIMP CASP::availableDocuments(BSTR *ret) {

	CString documents = "";

	for(map<long, ASP_SUBSCRIPTION>::iterator i=this->subscriptions.begin(); i!=this->subscriptions.end(); i++) {
		for(map<long, DOCUMENTS>::iterator j=(*i).second.fetch_ta->documents.begin(); j!=(*i).second.fetch_ta->documents.end(); j++) {

			time_t ltime;
			time( &ltime );
			long ka = (*j).second.keepalive - (ltime - (*j).second.notified_time);
			if (ka < 0) continue;
			
			char _temp[10];
			ltoa(ka, _temp, 10);
			char _temp2[10];
			ltoa((*j).first, _temp2, 10);
			CString item = (CString)_temp2 + "\n:\n" + (CString)(*j).second.name.c_str() + "\n:\n" + (CString)(*j).second.version.c_str() 
				+ "\n:\n" + _temp + "\n:\n" + (CString)(*j).second.remarks.c_str() + "\n:\n" + (CString)(*j).second.filename.c_str()
				+ "\n:\n" + (CString)(*j).second.xsd_filename.c_str() + "\n:\n:\n";
			documents += item;
		}
	}

	*ret = documents.AllocSysString();

	return S_OK;
}


STDMETHODIMP CASP::availableGMEParadigms(BSTR *ret) {

	CString gme_paradigms = "";

	CComPtr<IMgaRegistrar> registrar;
	::CoInitialize(NULL);
	registrar.CoCreateInstance(OLESTR("Mga.MgaRegistrar"));
	CComVariant vpars;
	registrar->get_Paradigms(REGACCESS_BOTH, &vpars);

	CStringList paradigms;
	ASSERT( ((vpars.vt & VT_ARRAY) == VT_ARRAY) || (vpars.vt == VT_EMPTY));
	LoadBstrSafeArray(paradigms, vpars.parray);

	POSITION pos = paradigms.GetHeadPosition();
	while (pos) {
		gme_paradigms += paradigms.GetNext(pos) + "\n";
	}

	*ret = gme_paradigms.AllocSysString();

	return S_OK;
}


// *************************************************************************************************************************
// fetching Thread
// *************************************************************************************************************************
UINT AspMyFetchingThread( LPVOID pParam ) {

	::CoInitialize(NULL);

	try {

		ASP_ARG_LIST *arg = (ASP_ARG_LIST*) pParam;
		CASP *casp = arg->casp;
		string name = (string) arg->name;
		string version = (string) arg->version;
		string type = (string) arg->type;

		Uml::Diagram paradigm_diagram = casp->uml_ta->remoteFetchMyParadigm(name.c_str(), version.c_str());

		Udm::UdmDiagram udmDataDiagram;
		udmDataDiagram.dgr = &paradigm_diagram;
		udmDataDiagram.init = dummy;

		ASP_my_ToolAdaptor *ta = new ASP_my_ToolAdaptor(type.c_str(), 0, NULL, paradigm_diagram, casp->subscriptions_id);

		ASP_SUBSCRIPTION subscr;
		subscr.paradigm.name = name;
		subscr.paradigm.version = version;
		subscr.fetch_ta = ta;
		casp->subscriptions.insert(pair<const long, ASP_SUBSCRIPTION>(casp->subscriptions_id++, subscr));

		delete arg;
		
		ta->subscribe(udmDataDiagram, name.c_str(), version.c_str());
		ta->run();

		ta->logout();
		delete ta;

	} catch (udm_exception &) {
		return -1;
	} catch (udmcorba_exception &) {
		return -1;
	}

	return 0;
}

// *************************************************************************************************************************
CWinThread *asp_fetching_thread;

STDMETHODIMP CASP::subscribeParadigm(BSTR par_name, BSTR par_version, BSTR *err) {

	try {
		string par_name_str = (CString)par_name;
		string par_version_str = (CString)par_version;

		for(map<long, PARADIGM_STRUCT>::iterator i=this->remote_paradigms.begin(); i!=this->remote_paradigms.end(); i++) {
			if (!par_name_str.compare((*i).second.name) && !par_version_str.compare((*i).second.version)) {

				ASP_ARG_LIST *arg = new ASP_ARG_LIST();
				arg->casp = this;
				arg->name = (*i).second.name;
				arg->version = (*i).second.version;		
				arg->type = this->ta->getToolAdaptorType();

				asp_fetching_thread = AfxBeginThread(AspMyFetchingThread, arg);

				map<long, bool>::iterator l=this->subscribed.find( (*i). first );
				if (l==this->subscribed.end()) {
					this->subscribed.insert( pair<long, bool>((*i).first, true) );
				} else {
					(*l).second = true;
				}
			}
		}

	} catch (udmcorba_exception &e) {
		CString error = e.what();
		*err = error.AllocSysString();
		return S_FALSE;
	} catch (udm_exception &e) {
		CString error = e.what();
		*err = error.AllocSysString();
		return S_FALSE;
	} catch (...) {
		CString error = "Fatal error!";
		*err = error.AllocSysString();
		return S_FALSE;
	}

	return S_OK;
};


STDMETHODIMP CASP::unsubscribeParadigm(BSTR par_name, BSTR par_version, BSTR *err) {

	try {
		string par_name_str = (CString)par_name;
		string par_version_str = (CString)par_version;

		for(map<long, ASP_SUBSCRIPTION>::iterator j=this->subscriptions.begin(); j!=this->subscriptions.end(); j++) {
			if (!par_name_str.compare(j->second.paradigm.name.c_str()) && !par_version_str.compare(j->second.paradigm.version.c_str())) {
				for(map<long, DOCUMENTS>::iterator i=j->second.fetch_ta->documents.begin(); i!=j->second.fetch_ta->documents.end(); i++) {
					if (!(*i).second.filename.empty()) DeleteFile( (*i).second.filename.c_str() );
					if (!(*i).second.xsd_filename.empty()) DeleteFile( (*i).second.xsd_filename.c_str() );
				}
				j->second.fetch_ta->documents.clear();
				j->second.fetch_ta->terminate();
				subscriptions.erase(j);
			}
		}

		for(map<long, PARADIGM_STRUCT>::iterator k=this->remote_paradigms.begin(); k!=this->remote_paradigms.end(); k++) {
			if (!par_name_str.compare((*k).second.name) && !par_version_str.compare((*k).second.version)) {
				map<long, bool>::iterator l=this->subscribed.find( (*k). first );
				if (l!=this->subscribed.end()) (*l).second = false;
			}
		}

	} catch (udmcorba_exception &e) {
		CString error = e.what();
		*err = error.AllocSysString();
		return S_FALSE;
	} catch (udm_exception &e) {
		CString error = e.what();
		*err = error.AllocSysString();
		return S_FALSE;
	} catch (...) {
		CString error = "Fatal error!";
		*err = error.AllocSysString();
		return S_FALSE;
	}

	return S_OK;
};


struct _DOCUMENTS {
	string par_name;
	string par_version;
	string filename;
};


STDMETHODIMP CASP::publishDocument(BSTR name, BSTR version, long keepAlive, BSTR remarks, BSTR filename, BSTR xslt_filename, BSTR attachments, long update_id, BSTR *err)
 {
    

//	try {
		bool publishing_multiple_documents = false;


		CString filename_c = filename;
		CString xslt_filename_c = xslt_filename;
		CString attachments_c = attachments;
		attachments_c.Replace("\\", "\\\\");

		// get the publishable documents into a list
		list<_DOCUMENTS> _docs;
		int offset = filename_c.Find("\t\t");
		while (offset != -1) 
    {
			CString item = filename_c.Left(offset+1);
			_DOCUMENTS _doc;



			int offset2 = item.Find("\t");
			_doc.par_name = item.Left(offset2);
			item.Delete(0, offset2+1);

			offset2 = item.Find("\t");
			_doc.par_version = item.Left(offset2);
			item.Delete(0, offset2+1);

			offset2 = item.Find("\t");
			_doc.filename = item.Left(offset2);


			_docs.push_back( _doc );

			filename_c.Delete(0, offset+2);
			offset = filename_c.Find("\t\t");
		}

		// get the xslt files into a list
		list<_DOCUMENTS> _xslts;
		offset = xslt_filename_c.Find("\t\t");
		while (offset != -1) {
			CString item = xslt_filename_c.Left(offset+1);
			_DOCUMENTS _xslt;
			
			int offset2 = item.Find("\t");
			_xslt.par_name = item.Left(offset2);
			item.Delete(0, offset2+1);

			offset2 = item.Find("\t");
			_xslt.par_version = item.Left(offset2);
			item.Delete(0, offset2+1);

			offset2 = item.Find("\t");
			_xslt.filename = item.Left(offset2);

			_xslts.push_back( _xslt );

			xslt_filename_c.Delete(0, offset+2);
			offset = xslt_filename_c.Find("\t\t");
		}

		// get the attachment files into a map
		map<string, string> _attachments; // filename, directory
		offset = attachments_c.Find("\t");
		while (offset != -1)
    {
			CString attachment_file = attachments_c.Left(offset);
			
			ifstream additional_file( attachment_file );
			if (additional_file.fail()) {
					CString error = "Could not find attachment file: " + attachment_file;
					*err = error.AllocSysString();
					return S_FALSE;
			}

			int offset2 = attachment_file.ReverseFind('\\');
			_attachments.insert( pair<string, string>( (string)attachment_file.Right(attachment_file.GetLength()-(offset2+1)), (string)attachment_file.Left(offset2+1) ) );

			attachments_c.Delete(0, offset+1);
			offset = attachments_c.Find("\t");
		}

		if (_docs.size() != 1)
      publishing_multiple_documents = true;


		// Publishing simple document
		if (!publishing_multiple_documents) 
    {
		
			CString filename_c = _docs.front().filename.c_str();
			filename_c.Replace("\\", "\\\\");
			CString xslt_filename_c = _xslts.front().filename.c_str();
			xslt_filename_c.Replace("\\", "\\\\");

			int offset = filename_c.ReverseFind('\\');
			CString _file = filename_c.Mid(offset+1);
			CString fileext = _file.Right(3);

			string par_name_str = _docs.front().par_name;
			string par_version_str = _docs.front().par_version;

			// if it is fetched remotely we will use that Diagram and paradigm
			for(list<REGISTERED_PARADIGM>::iterator j=this->registered_paradigms.begin(); j!=this->registered_paradigms.end(); j++) {
				if (!par_name_str.compare((*j).name) && !par_version_str.compare((*j).version)) {
					this->udmDataDiagram = (*j).udmDataDiagram;
					this->paradigm_diagram = (*j).paradigm_diagram;
				}
			}

			// if it is not fetched yet, we fetch it and put the paradigm into our store to use it now and later
			if (!this->ta->isRegisteredDatanetwork(par_name_str.c_str(), par_version_str.c_str())) {
				this->paradigm_diagram = this->uml_ta->remoteFetchMyParadigm(par_name_str.c_str(), par_version_str.c_str());
				this->udmDataDiagram.dgr = &this->paradigm_diagram;
				this->udmDataDiagram.init = dummy;

				REGISTERED_PARADIGM temp_reg_par;
				temp_reg_par.name = par_name_str;
				temp_reg_par.version = par_version_str;
				temp_reg_par.paradigm_diagram = this->paradigm_diagram;
				temp_reg_par.udmDataDiagram = this->udmDataDiagram;
				this->registered_paradigms.push_front(temp_reg_par);
			}

			// XSLT translation
			if (xslt_filename_c.Compare("") && !fileext.Compare("xml")) 
      {

				// do the trasnform
				int result = 0;
				try {
					OTIF_XSLT::XSLTransformer xslt;
					result = xslt.executeTransform((LPCSTR)filename_c, (LPCSTR)xslt_filename_c, \
						(LPCSTR)(filename_c + "_xslt.xml"), false, "");

				} catch (OTIF_XSLT::xslt_exception &e) {
					CString error = e.what();
					*err = error.AllocSysString();
					return S_FALSE;
				}

				if (result!=0) {
					CString error = "Could not transform the document with the XSLT script";
					*err = error.AllocSysString();
					return S_FALSE;
				}

				filename_c = filename_c + "_xslt.xml";
			}


			UdmCORBA::CORBADataNetwork *dn;
			try {
				// create DN with the remote paradigm
				dn = this->ta->getDatanetwork(this->udmDataDiagram, par_name_str.c_str(), par_version_str.c_str());
				// open the publishable document
				dn->OpenExisting((LPCSTR)filename_c, "", Udm::CHANGES_LOST_DEFAULT);
			} catch (udm_exception &e) {
				CString error = e.what();
				if (error.Compare("Not found") == 0) error = "Invalid input file!";
				*err = error.AllocSysString();
				return S_FALSE;
			}

			// attachment files
			for(map<string, string>::iterator addfile = _attachments.begin(); addfile!=_attachments.end(); addfile++) {
				dn->addAdditionalFile( (*addfile).first.c_str(), (*addfile).second.c_str() );
			}


			// publish the document
			dn->publishDocument((CString)name, (CString)version, (CString)remarks, keepAlive, update_id);
			// close the document
			dn->CloseNoUpdate();

			// delete temporary docs after XSLT transform
			if (xslt_filename_c.Compare("")) DeleteFile((LPCSTR)filename_c);


		// Publishing multiple document
		} 
    else 
    { 

			list<UdmCORBA::CORBADataNetwork*> docs;
			list<CString> temp_xsl_files;

			for(list<_DOCUMENTS>::iterator i=_docs.begin(); i!=_docs.end(); i++) 
      {

				CString filename_c = (*i).filename.c_str();
				filename_c.Replace("\\", "\\\\");



				CString xslt_filename_c;
				for(list<_DOCUMENTS>::iterator j=_xslts.begin(); j!=_xslts.end(); j++) {
					if (!(*i).par_name.compare( (*j).par_name ) && !(*i).par_version.compare( (*j).par_version )) xslt_filename_c = (*j).filename.c_str();
				}
				xslt_filename_c.Replace("\\", "\\\\");

				int offset = filename_c.ReverseFind('\\');
				CString _file = filename_c.Mid(offset+1);
				CString fileext = _file.Right(3);

				string par_name_str = (*i).par_name;
				string par_version_str = (*i).par_version;
			
				Uml::Diagram paradigm_diagram = uml_ta->remoteFetchMyParadigm(par_name_str.c_str(), par_version_str.c_str());
				Udm::UdmDiagram udmDataDiagram;
				udmDataDiagram.dgr = &paradigm_diagram;
				udmDataDiagram.init = dummy;


				// XSLT translation
				if (xslt_filename_c.Compare("") && !fileext.Compare("xml")) {

					// do the trasnform
					int result = 0;
					try {
						OTIF_XSLT::XSLTransformer xslt;
						result = xslt.executeTransform((LPCSTR)filename_c, (LPCSTR)xslt_filename_c, \
							(LPCSTR)(filename_c + "_xslt.xml"), false, "");

					} catch (OTIF_XSLT::xslt_exception &e) {
						CString error = e.what();
						*err = error.AllocSysString();
						return S_FALSE;
					}

					if (result!=0) {
						CString error = "Could not transform the document with the XSLT script";
						*err = error.AllocSysString();
						return S_FALSE;
					}

					filename_c = filename_c + "_xslt.xml";
					temp_xsl_files.push_back(filename_c);
				}

				UdmCORBA::CORBADataNetwork *dn;
				try {
					// create DN with the remote paradigm
					dn = ta->getDatanetwork(udmDataDiagram, par_name_str.c_str(), par_version_str.c_str());
					// open the publishable document
					dn->OpenExisting((LPCSTR)filename_c, "", Udm::CHANGES_LOST_DEFAULT);
				} catch (udm_exception &e) {
					CString error = e.what();
					if (error.Compare("Not found") == 0) error = "Invalid input file!";
					*err = error.AllocSysString();
					return S_FALSE;
				}

				docs.push_back(dn);
			}

			// attachment files
			for(map<string, string>::iterator addfile = _attachments.begin(); addfile!=_attachments.end(); addfile++) {
				(*docs.begin())->addAdditionalFile( (*addfile).first.c_str(), (*addfile).second.c_str() );
			}

			ta->publishMultipleDocuments(docs, (CString)name, (CString)version, (CString)remarks, keepAlive, update_id);

			for(list<UdmCORBA::CORBADataNetwork*>::iterator q=docs.begin(); q!=docs.end(); q++) (*q)->CloseNoUpdate();

			// delete temporary docs after XSLT transform
			for(list<CString>::iterator txf=temp_xsl_files.begin(); txf!=temp_xsl_files.end(); txf++) DeleteFile((LPCSTR)(*txf));
		}
/*
	} catch (udmcorba_exception &e) {
		CString error = e.what();
		*err = error.AllocSysString();
		return S_FALSE;
	} catch (udm_exception &e) {
		CString error = e.what();
		*err = error.AllocSysString();
		return S_FALSE;
	} catch (...) {
		CString error = "Fatal error!";
		*err = error.AllocSysString();
		return S_FALSE;
	}
*/
	return S_OK;
}



STDMETHODIMP CASP::loginToolAdaptor(BSTR type, BSTR *err) {

	try {
		if (this->ta) delete this->ta;
		if (this->uml_ta) delete this->uml_ta;

		CString ta_type = type;

		this->ta = new ASP_my_ToolAdaptor((LPCSTR)ta_type, 0, NULL);

		this->uml_ta = new ASP_my_UML_ToolAdaptor(0, NULL);
		this->uml_ta->getDatanetwork(Uml::diagram);

		remote_paradigms.clear();
		this->ta->getWorkingParadigms(this->remote_paradigms);

		this->config.clear();
		this->ta->getTAConfig(this->config);

		this->subscribable.clear();
		this->ta->getTASubscribable(this->subscribable);

	} catch (udmcorba_exception &e) {
		CString error = e.what();
		*err = error.AllocSysString();
		return S_FALSE;
	} catch (udm_exception &e) {
		CString error = e.what();
		*err = error.AllocSysString();
		return S_FALSE;
	}

	return S_OK;
}

STDMETHODIMP CASP::logout(BSTR *ret) {

	for(map<long, ASP_SUBSCRIPTION>::iterator j=this->subscriptions.begin(); j!=this->subscriptions.end(); j++) {
		for(map<long, DOCUMENTS>::iterator i=(*j).second.fetch_ta->documents.begin(); i!=(*j).second.fetch_ta->documents.end(); i++) {
			if (!(*i).second.filename.empty()) DeleteFile( (*i).second.filename.c_str() );
			if (!(*i).second.xsd_filename.empty()) DeleteFile( (*i).second.xsd_filename.c_str() );
		}
		j->second.fetch_ta->documents.clear();
		j->second.fetch_ta->terminate();
	}

	this->remote_paradigms.clear();
	this->config.clear();
	this->subscribable.clear();
	this->registered_paradigms.clear();
	this->subscriptions.clear();
	this->subscribed.clear();

	return S_OK;
}

STDMETHODIMP CASP::initialize(BSTR parameters, BSTR *err) {

	// TODO: Add your implementation code here
	if (initialized) return S_OK;
	
	vector<string> params;
	params.push_back((LPCSTR)parameters);

	try {
		UdmCORBA::corba_startup(params);
	} catch (udmcorba_exception &e) {
		CString error = e.what();
		*err = error.AllocSysString();
		return S_FALSE;
	} catch (udm_exception &e) {
		CString error = e.what();
		*err = error.AllocSysString();
		return S_FALSE;
	}

	initialized = true;

	return S_OK;
}
//==========================================
struct ActionDescr
{
  std::string m_paradigm;
  std::string m_version;
  bool m_publish;
  bool m_subscribe;
  bool m_subscribeMode;
  ActionDescr() : m_publish(false), m_subscribe(false) {}

  std::string toStr() const
  {
    std::string out;
    out += "paradigm:";
    out += m_paradigm;
    out += "version:";
    out += m_version;
    if (m_publish)
      out += "action:publish";

    if (m_subscribe)
    {
      if (m_subscribeMode)
        out += "action:subscribe:true";
      else
        out += "action:subscribe:false";
    }
    return out;
  }
      
};

typedef std::map<std::string, ActionDescr> StrActionDescrMap;
STDMETHODIMP CASP::getActions(BSTR *ret) 
{

	StrActionDescrMap map;

  list<CONFIG>::iterator i = subscribable.begin();
 	for(; i!= subscribable.end(); i++)
  {
		list<PARADIGM_STRUCT> pars = (*i).paradigms;
    list<PARADIGM_STRUCT>::iterator j=pars.begin();
		for(; j!=pars.end(); j++) 
    {

	    bool subscr = false;
      std::map<long, PARADIGM_STRUCT>::iterator k = remote_paradigms.begin();
	    for(; k!=remote_paradigms.end(); k++)
      {
		    if (!(*j).name.compare((*k).second.name) && 
            !(*j).version.compare((*k).second.version))
        {
          std::map<long, bool>::iterator l = subscribed.find( (*k). first );
			    if (l!=subscribed.end()) 
            subscr = (*l).second;
		    }
	    }
      std::string pd = j->name + j->version;

      map[pd].m_paradigm = j->name;
      map[pd].m_version = j->version;
      map[pd].m_subscribe = true;
      map[pd].m_subscribeMode = ((subscr) ? true : false);

  	}
	}

  list<CONFIG>::iterator p=config.begin();
	for(; p!=config.end(); p++)
  {
		list<PARADIGM_STRUCT> pars = (*p).paradigms;
    list<PARADIGM_STRUCT>::iterator j=pars.begin();
		for(; j!=pars.end(); j++)
    {
      const std::string pd = j->name + j->version;

      map[pd].m_paradigm = j->name;
      map[pd].m_version = j->version;
      map[pd].m_publish = true;
		}
	}

  CString sret;
  StrActionDescrMap::const_iterator it = map.begin();
  for (; it != map.end(); ++it)
  {
    sret += it->second.toStr().c_str();
  }
  
	*ret = sret.AllocSysString();

  return S_OK;
}

STDMETHODIMP CASP::getSubscribable(BSTR *ret) {

	CString paradigms = "";

	for(list<CONFIG>::iterator i=this->subscribable.begin(); i!=this->subscribable.end(); i++) {
		list<PARADIGM_STRUCT> pars = (*i).paradigms;
		for(list<PARADIGM_STRUCT>::iterator j=pars.begin(); j!=pars.end(); j++) {

			bool subscr = false;
			for(map<long, PARADIGM_STRUCT>::iterator k=this->remote_paradigms.begin(); k!=this->remote_paradigms.end(); k++) {
				if (!(*j).name.compare((*k).second.name) && !(*j).version.compare((*k).second.version)) {
					map<long, bool>::iterator l=this->subscribed.find( (*k). first );
					if (l!=this->subscribed.end()) subscr = (*l).second;
				}
			}
			
			CString item = (CString)(*j).name.c_str() + "\n" + (CString)(*j).version.c_str() + "\n" + ((subscr) ? "true" : "false") + "\n\n";
			paradigms += item;
		}
	}

	*ret = paradigms.AllocSysString();

	return S_OK;
}

STDMETHODIMP CASP::getPublishable(BSTR *ret) {

	CString paradigms = "";

	for(list<CONFIG>::iterator i=config.begin(); i!=config.end(); i++) {
		list<PARADIGM_STRUCT> pars = (*i).paradigms;
		for(list<PARADIGM_STRUCT>::iterator j=pars.begin(); j!=pars.end(); j++) {
			CString item = (CString)(*j).name.c_str() + "\n" + (CString)(*j).version.c_str() + "\n\n";
			paradigms += item;
		}
	}

	*ret = paradigms.AllocSysString();

	return S_OK;
}

STDMETHODIMP CASP::remoteParadigms(BSTR *ret) {

	CString paradigms = "";

	for(map<long, PARADIGM_STRUCT>::iterator i=this->remote_paradigms.begin(); i!=this->remote_paradigms.end(); i++) {
		CString item = (CString)(*i).second.name.c_str() + "\n" + (CString)(*i).second.version.c_str() + "\n\n";
		paradigms += item;
	}
	
	*ret = paradigms.AllocSysString();

	return S_OK;

}
STDMETHODIMP CASP::remoteTypes(BSTR *ret) {

	list<string> types;
	CString ta_types = "";
	
  bool exc = false;
  std::string errtxt;
	try 
  {
		ToolAdaptor::getBackplaneToolAdaptorTypes(types, 0, NULL);
	} 
  catch (udmcorba_exception &) 
  {
		
		initialized = false;
		BSTR err;
		initialize((BSTR)"", &err);

		try
    {
			ToolAdaptor::getBackplaneToolAdaptorTypes(types, 0, NULL);
		}
    catch (udmcorba_exception &e)
    {
      errtxt = std::string(e.what());
      exc= true;
		}
    catch (udm_exception &e)
    {
      errtxt = std::string(e.what());
      exc= true;
		}
	
	}
  catch (udm_exception &)
  {

		initialized = false;
		BSTR err;
		initialize((BSTR)"", &err);

		try
    {
			ToolAdaptor::getBackplaneToolAdaptorTypes(types, 0, NULL);
		}
    catch (udmcorba_exception &e)
    {
      errtxt = std::string(e.what());
      exc= true;
		}
    catch (udm_exception &e)
    {
      errtxt = std::string(e.what());
      exc= true;
		}
	}

  if (!exc)
  {
	  for(list<string>::iterator i=types.begin(); i!=types.end(); i++)
    {
		  CString item = (CString)(*i).c_str() + "\n";
		  ta_types += item;
	  }
  }
  else
  {
    ta_types = "hiba";
    ta_types += CString(errtxt.c_str());
  }


	*ret = ta_types.AllocSysString();

	return S_OK;
}

//==========================================
// STDMETHODIMP CASP::remoteTypes(BSTR *ret) {
// 
// 	list<string> types;
// 	CString ta_types = "";
// 	
// 	try 
//   {
// 		ToolAdaptor::getBackplaneToolAdaptorTypes(types, 0, NULL);
// 	} 
//   catch (udmcorba_exception &) 
//   {
// 		
// 		initialized = false;
// 		BSTR err;
// 		initialize((BSTR)"", &err);
// 
// 		try
//     {
// 			ToolAdaptor::getBackplaneToolAdaptorTypes(types, 0, NULL);
// 		}
//     catch (udmcorba_exception &e)
//     {
// 			m_piResponse->Write( (CComVariant)"<p><b>Error: " );
// 			m_piResponse->Write( (CComVariant)e.what() );
// 			m_piResponse->Write( (CComVariant)"</b></p>" );
// 			return S_FALSE;
// 		}
//     catch (udm_exception &e)
//     {
// 			m_piResponse->Write( (CComVariant)"<p><b>Error: " );
// 			m_piResponse->Write( (CComVariant)e.what() );
// 			m_piResponse->Write( (CComVariant)"</b></p>" );
// 			return S_FALSE;
// 		}
// 	
// 	}
//   catch (udm_exception &)
//   {
// 
// 		initialized = false;
// 		BSTR err;
// 		initialize((BSTR)"", &err);
// 
// 		try
//     {
// 			ToolAdaptor::getBackplaneToolAdaptorTypes(types, 0, NULL);
// 		}
//     catch (udmcorba_exception &e)
//     {
// 			m_piResponse->Write( (CComVariant)"<p><b>Error: " );
// 			m_piResponse->Write( (CComVariant)e.what() );
// 			m_piResponse->Write( (CComVariant)"</b></p>" );
// 			return S_FALSE;
// 		}
//     catch (udm_exception &e)
//     {
// 			m_piResponse->Write( (CComVariant)"<p><b>Error: " );
// 			m_piResponse->Write( (CComVariant)e.what() );
// 			m_piResponse->Write( (CComVariant)"</b></p>" );
// 			return S_FALSE;
// 		}
// 	}
// 
// 	for(list<string>::iterator i=types.begin(); i!=types.end(); i++) {
// 		CString item = (CString)(*i).c_str() + "\n";
// 		ta_types += item;
// 	}
// 	
// 	*ret = ta_types.AllocSysString();
// 
// 	return S_OK;
// }

