#include "uml.h"
#include "UdmDom.h"
#include "UdmCORBA.h"
#include "UdmCORBA_Logger.h"
Logger logger = Logger("Generic_XSLT_Translator");

#include "TA_XSLT_Support.h"

#include <fstream>
#include <windows.h>
#include <process.h>

using namespace UdmCORBA;
void dummy(void) {};

Udm::UdmDiagram inputparadigm_udmDataDiagram;
Udm::UdmDiagram outputparadigm_udmDataDiagram;
string input_name;
string input_version;
string output_name;
string output_version;
string stylesheet;

// *************************************************************************************************************************
//	my_ToolAdaptor
// *************************************************************************************************************************
class my_ToolAdaptor : public ToolAdaptor {
public:

	// create the UML publisher-receiver tooladaptor
	my_ToolAdaptor(int argc, char **argv) :
		ToolAdaptor(argc, argv) {};

	void notify(const char *paradigm_name, const char *paradigm_version, \
		const char *name, const char *version, const char *remarks, long docId, long keepAlive) {};
};

my_ToolAdaptor *ta;


// *************************************************************************************************************************
// my_Translator
// *************************************************************************************************************************
class myTranslator : public UdmCORBA::Translator {

private:
	Uml::Diagram input_meta_diagram;
	Uml::Diagram output_meta_diagram;

private:
	string ff_name;
	string output;
	string xsd_file;
	string filename;
	Uml::Class input_rootclass;

public:
	myTranslator(const Udm::UdmDiagram &from_metainfo, const char *from_name, const char *from_version, const Uml::Class &from_rootclass,
			const Udm::UdmDiagram &to_metainfo, const char *to_name, const char *to_version, const Uml::Class &to_rootclass,
			int argc, char **argv, Uml::Diagram input_meta_diagram, Uml::Diagram output_meta_diagram, Logger *logger)
			: Translator(from_metainfo, from_name, from_version, from_rootclass, 
						to_metainfo, to_name, to_version, to_rootclass, 
						argc, argv, logger), 
						input_meta_diagram(input_meta_diagram), output_meta_diagram(output_meta_diagram) {};


	void fetching_process(const char *name, const char *version, const char *remarks, const long docId, const unsigned long numberOfDocuments) {

// *********************************************************************************************************************
//		Fetch document
		if (logger) logger->log(3, "Get a notification, starts fetching");

		try {
		
			// Get the rootobject
			input_rootclass = getDocumentRootobjectMeta(docId, input_meta_diagram);
			if (!input_rootclass) {
				cerr << "Could not find the meta of the rootobject of the document! Could not save it." << endl;
				if (logger) logger->log(1, "Could not find the meta of the rootobject of the document! Could not save it.");
				return;
			}

			// xml layer specific metalocator
			ff_name = "__temp_input_" + fromDN->getParadigmName() + "_v" + fromDN->getParadigmVersion();
			output = "__temp_output_" + fromDN->getParadigmName() + "_v" + fromDN->getParadigmVersion() + ".xml";
			xsd_file = ff_name + ".xsd";
			filename = ff_name + ".xml";

			// Create the temporary xsd for the paradigm			
			OTIF_XSLT::GenerateXMLSchemaFile(input_meta_diagram, xsd_file.c_str());

			// Create the new datanetwork
			fromDN->CreateNew(filename.c_str(), ff_name.c_str(), input_rootclass, Udm::CHANGES_PERSIST_ALWAYS);

			fromDN->fetchDocument(docId);
			fromDN->SaveAs(filename.c_str());

			this->working_docID = docId;
			notify(name, version, "after the XSLT transformation...", 1);

			if (logger) logger->log(3, "Finished a translation process");
		} catch (udm_exception &e) {
			cerr << e.what() << endl;
			if (logger) logger->log(1, "Error", e.what());
		} catch (udmcorba_exception &e) {
			if (e.code()==32) {
				cerr << "The document does not exist in the Backplane. The translation is terminated." << endl;
				if (logger) logger->log(1, "Error", "The document does not exist in the Backplane. The translation is terminated.");
			} else {
				cerr << e.what() << endl;
				if (logger) logger->log(1, "Error", e.what());
			}
		} catch (...) {
			cerr << "Fatal exception" << endl;
			if (logger) logger->log(1, "Fatal exception");
		}
	}

	
	void notify(const char *name, const char *version, const char *remarks, const unsigned long numOfDocs) {
	
		try {

// *********************************************************************************************************************
//		XSLT translation		
			string debug = "Starts the XSLT transformation on: " + filename + ":" + stylesheet + ":" + output + ":" + xsd_file;
			if (logger) logger->log(3, debug.c_str());

			int result = 0;

			try {
				OTIF_XSLT::XSLTransformer xslt;
				result = xslt.executeTransform(filename.c_str(), stylesheet.c_str(), output.c_str(), true, ((string)input_rootclass.name()).c_str());

			} catch (OTIF_XSLT::xslt_exception &e) {
				DeleteFile((ff_name + ".xsd").c_str());
				DeleteFile((ff_name + ".xml").c_str());
				DeleteFile(output.c_str());
				cerr << "XSLT translation error: " << e.what() << endl;
				if (logger) logger->log(1, "XSLT Error", e.what());
				return;
			}

			if (result!=0) {
				DeleteFile((ff_name + ".xsd").c_str());
				DeleteFile((ff_name + ".xml").c_str());
				DeleteFile(output.c_str());
				cerr << "Error during XSLT translation! The XSLT file may be corrupt!" << endl;
				if (logger) logger->log(1, "Error during XSLT translation! The XSLT file may be corrupt!");
				return;
			}


// *********************************************************************************************************************
//		Publish translated document
			if (logger) logger->log(3, "Starts publishing the translated document");
			
			try {
				toDN->OpenExisting(output.c_str(), "", Udm::CHANGES_LOST_DEFAULT);
			} catch (udm_exception &e) {
				string error = e.what();
				if (!error.compare("Not found")) {
					cerr << "Invalid input file!" << endl;
					if (logger) logger->log(1, "Invalid input file!");
				} else { 
					cerr << e.what() << endl;
					if (logger) logger->log(1, "Error", e.what());
				}
			}
	
			toDN->copyAdditionalFilesFrom(fromDN);

			publish(name, version, "After the XSLT translation");

			DeleteFile((ff_name + ".xsd").c_str());
			DeleteFile((ff_name + ".xml").c_str());
			DeleteFile(output.c_str());

			fromDN->CloseNoUpdate();
			fromDN->release();
			toDN->CloseNoUpdate();
			toDN->release();

		} catch (udm_exception &e) {
			cout << e.what() << endl;
			if (logger) logger->log(1, "Error", e.what());
		} catch (udmcorba_exception &e) {
			cout << e.what() << endl;
			if (logger) logger->log(1, "Error", e.what());
		} catch (...) {
			cerr << "Fatal exception" << endl;
			if (logger) logger->log(1, "Fatal exception");
		}	
	}

};


int main(int argc, char **argv) {

	if (argc!=9) {
		cerr << "XSLT translator invalid usage!" << endl;
		exit(-1);
	}

	try {

		my_ToolAdaptor *ta = new my_ToolAdaptor(argc, argv);
		ta->getDatanetwork(Uml::diagram);

		input_name = argv[4];
		input_version = argv[5];
		output_name = argv[6];
		output_version = argv[7];
		stylesheet = argv[8];

		Uml::Diagram input_meta_diagram = ta->remoteFetchMyParadigm(input_name.c_str(), input_version.c_str());
		Uml::Diagram output_meta_diagram = ta->remoteFetchMyParadigm(output_name.c_str(), output_version.c_str());

		inputparadigm_udmDataDiagram.dgr = &input_meta_diagram;
		inputparadigm_udmDataDiagram.init = dummy;
		
		outputparadigm_udmDataDiagram.dgr = &output_meta_diagram;
		outputparadigm_udmDataDiagram.init = dummy;

		delete ta;

		myTranslator tr(inputparadigm_udmDataDiagram, input_name.c_str(), input_version.c_str(), NULL, 
						outputparadigm_udmDataDiagram, output_name.c_str(), output_version.c_str(), NULL, 
						argc, argv, input_meta_diagram, output_meta_diagram, &logger);
		tr.run();

	} catch (udm_exception &e) {
		cerr << e.what() << endl;
	} catch (udmcorba_exception &e) {
		if ( e.code()<11 ) {
			cerr << endl << "CORBA exception: " << e.what() << endl;
		} else if (e.code()<21) {
			cerr << endl << "Paradigm exception: " << e.what() << endl;
		} else if (e.code()<41) {
			cerr << endl << "UdmCORBA exception: " << e.what() << endl;
		} else if (e.code()<91) {
			cerr << endl << "ToolAdaptor exception: " << e.what() << endl;
		}
	}

	return 0;
}