/*
Copyright (c) Vanderbilt University, 2000-2001
ALL RIGHTS RESERVED
Vanderbilt University disclaims all warranties with regard to this
software, including all implied warranties of merchantability
and fitness.  In no event shall Vanderbilt University be liable for
any special, indirect or consequential damages or any damages
whatsoever resulting from loss of use, data or profits, whether
in an action of contract, negligence or other tortious action,
arising out of or in connection with the use or performance of
this software.
*/

#ifndef UDMCORBA_LOGGER_H
#define UDMCORBA_LOGGER_H

#include <time.h>
#include "SYS\TIMEB.H"
#include <string>
#include <fstream>

class Logger {

	private:
		ofstream *logFile;
		string application;
		int level;

	public:
		Logger() {
			logFile = NULL;
			application = "";
			level = 0;
		};
		
		Logger(const char *application, const char *log = "") {
			if (!strcmp(log, "")) logFile = new ofstream("BackPlane.log", ios::app);
			else logFile = new ofstream(log, ios::app);
			this->application = application;
			level = 0;
		};

		~Logger() {
			(*logFile).close();
		}

		void setLevel(int level = 0) {
			this->level = level;
		}

		void log(int level, const char *str, string param = "") {
			if (level<=this->level && logFile) {
				char date_buf[128], time_buf[128];
				struct _timeb tstruct;
				_strdate(date_buf);
				_strtime(time_buf);
				_ftime( &tstruct );

				if (tstruct.millitm<10)
					(*logFile) << "[" << application << "::" << date_buf << " " << time_buf << ".00" << tstruct.millitm << "] " << str << " " << param << endl;
				else if (tstruct.millitm<100)
					(*logFile) << "[" << application << "::" << date_buf << " " << time_buf << ".0" << tstruct.millitm << "] " << str << " " << param << endl;
				else
					(*logFile) << "[" << application << "::" << date_buf << " " << time_buf << "." << tstruct.millitm << "] " << str << " " << param << endl;
			}
		};

		void log(int level, const char *str, long param) {
			if (level<=this->level && logFile) {
				char date_buf[128], time_buf[128];
				struct _timeb tstruct;
				_strdate(date_buf);
				_strtime(time_buf);
				_ftime( &tstruct );

				if (tstruct.millitm<10)
					(*logFile) << "[" << application << "::" << date_buf << " " << time_buf << ".00" << tstruct.millitm << "] " << str << " " << param << endl;
				else if (tstruct.millitm<100)
					(*logFile) << "[" << application << "::" << date_buf << " " << time_buf << ".0" << tstruct.millitm << "] " << str << " " << param << endl;
				else
					(*logFile) << "[" << application << "::" << date_buf << " " << time_buf << "." << tstruct.millitm << "] " << str << " " << param << endl;
			}
		};

};


#endif
