/*
Copyright (c) Vanderbilt University, 2000-2001
ALL RIGHTS RESERVED
Vanderbilt University disclaims all warranties with regard to this
software, including all implied warranties of merchantability
and fitness.  In no event shall Vanderbilt University be liable for
any special, indirect or consequential damages or any damages
whatsoever resulting from loss of use, data or profits, whether
in an action of contract, negligence or other tortious action,
arising out of or in connection with the use or performance of
this software.
*/

#include "TA_XSLT_Support.h"


void GenerateDTD(const Uml::Diagram &diagram, ostream &output);
void GenerateXMLSchema(const Uml::Diagram &diagram,  ostream &output);


	typedef pair<__int64, __int64> card_prod;			//cardinality specification: min, max
	typedef pair<Uml::Class, card_prod> class_w_cp;		//class with cardinality prod
	typedef vector<class_w_cp> cwcp_order;				//list of classes with cardinality info


	string GetTime()	
	{
		time_t t;
		time(&t);
		char *c = ctime(&t);

		c[24] = '\0';
		return c;
	}

	// -------------------------------------------------------------------------------------------------------------------------




	card_prod sum_cardinalities(const vector<card_prod> &ccrs)
	{
		card_prod ret;
		ret.first = 1;
		bool max_set = false;//these are the default cardinalities

		vector<card_prod>::const_iterator ccr_i = ccrs.begin();
		while (ccr_i != ccrs.end())
		{
			//the product minimum is the minimum of the minimums
			if (ccr_i->first < ret.first) ret.first = ccr_i->first;
			
			//the product maximum is the sum of the maximums or *
			if (ccr_i->second != -1)
			{
				if (max_set) 
				{
					if (ret.second != -1)
						ret.second += ccr_i->second;
				}
				else 
				{
					ret.second = ccr_i->second;
					max_set = true;
				}
			}
			else
			{
				max_set = true;
				ret.second = -1;
			}
			ccr_i ++;
		}

		if (!max_set) ret.second = 1;


		return ret;
	};	

	cwcp_order get_cwcp_order(const Uml::Class & parent)
	{
		cwcp_order ret;

		set<Uml::CompositionChildRole> ccr_set = Uml::AncestorCompositionPeerChildRoles(parent);
		

		//stage 1: create the map of classes with all their childroles
		typedef vector<card_prod> cp_vector_type;
		map<const Uml::Class, cp_vector_type> class_cps;

		set<Uml::CompositionChildRole>::iterator ccr_set_i = ccr_set.begin();
		while (ccr_set_i != ccr_set.end())
		{
			Uml::Class target = ccr_set_i->target();
			set<Uml::Class> target_descs = Uml::DescendantClasses(target);
			set<Uml::Class>::iterator td_i = target_descs.begin();

			while (td_i != target_descs.end())
			{
			
				
				
				if (!td_i->isAbstract())
				{
					card_prod cp;
					cp.first = ccr_set_i->min();
					cp.second = ccr_set_i->max();

					//if (*td_i != target)
					if (target_descs.size() > 1)
					
						//inherited childroles, 
						//minimum should be set to 0,
						//since it's not known which inherited type is going to be there
						cp.first = 0;
					
					
					class_cps[*td_i].push_back(cp);
				}
				
				td_i++;
				
			}

			ccr_set_i++;
		}

		//stage 2: create the product of the cardinalities as a sequence sorted by typename
		map<const string, class_w_cp> cwcp_seq_map;

		map<const Uml::Class, cp_vector_type>::iterator cc_i = class_cps.begin();
		while (cc_i != class_cps.end())
		{
			cwcp_seq_map[(cc_i->first).name()] = class_w_cp(cc_i->first, sum_cardinalities(cc_i->second));
			cc_i++;
		}

		//stage 3: retrieve the order as vector from the ordered sequence
		// (so the vector will be ordered)
		map<const string, class_w_cp>::iterator cwcp_seq_map_i = cwcp_seq_map.begin();
		while(cwcp_seq_map_i != cwcp_seq_map.end())
		{
			if ((string)parent.name() == "Location")
			{
		
				cout << " cwcp_seq_map key : " << cwcp_seq_map_i->first << ", class: " << (string) cwcp_seq_map_i->second.first.name() << endl;
			}

			ret.push_back(cwcp_seq_map_i->second);
			cwcp_seq_map_i++;
		}

		return ret;
	};

	string card_to_dtdspec(const class_w_cp& cwcp)
	{
		/*

		Mail from Gabor Karsai on 10/24/2002: ;-)

		How about the following.

		Assume P contains <c1> C1 and <c2> C2 and ... and <cn> Cn,
		where <c1>, <c2>, ... <cn> are cardinalities. 

		You translate a cardinality <c> (for a class say <C>) as follows:
		<c> form: translation:
		n (integer) C ... C (n times)
		0..1 C?
		0..* C*
		0..n C? | CC | ... | C...C (n times) <= THIS SUCKS!
		n..* C...C C* // where the first hale repeats C n times
		1..* C+

		endre, 05/06/03
		Obviously the case n..m, n < m is missing
		n..m C...C(n times)|C...C(n+1 times).......|C...C(m times)

		*/

		

		string ret;
		if ((cwcp.second.first == cwcp.second.second))
		{
			for (int i = 0; i < cwcp.second.first; i++)
			{
				ret += cwcp.first.name();
			}
		}
		else if ((cwcp.second.first == 0) && (cwcp.second.second == -1))
		{
			ret = (string)cwcp.first.name() + "*";
		}
		else if ((cwcp.second.first == 0) && (cwcp.second.second > 0))
		{
			ret += "(";
			for (int i = 0; i <= cwcp.second.second; i++)
			{
				if (i == 0)
					ret += (string)cwcp.first.name() + "?";
				else if (i > 1)
				{
					ret += "|(";
					
					for (int j = 0; j < i; j++)
					{
						if ((i>1)&&(j>0)) ret += ",";
						ret += (string)cwcp.first.name();
					}
					ret += ")";
				}
			}
			ret += ")";
		}
		else if ((cwcp.second.first >1) && (cwcp.second.second == -1))
		{
			for (int i = 0; i < cwcp.second.first; i++)
			{
				ret += (string)cwcp.first.name();
				ret += " ";
			}
			ret += cwcp.first.name();
			ret += "*";

		}
		else if ((cwcp.second.first == 1) && (cwcp.second.second == -1))
		{
			ret = (string)cwcp.first.name() + "+";
		}
		else
		{
			//the generic n..m, n<m case
			if (cwcp.second.first > cwcp.second.second)
				throw udm_exception("Invalid cardinality occured: min > max");

			ret+="(";
			for (__int64 i = cwcp.second.first; i <= cwcp.second.second; i++)
			{
				if (i != cwcp.second.first)
					ret += "|";
				
				if (i > 1) ret += "(";
				
				for (int j = 0; j < i; j++)
				{
					if ((i>1)&&(j>0)) ret += ",";
					ret += (string)cwcp.first.name();
					
				}

				if (i > 1) ret += ")";
			}
			ret += ")";

		}

		return ret;

	};


	// --------------------------- DTD generation

	void GenerateDTDElement(const Uml::Class &c,  ostream &output)	
	{
		string name = c.name();
		output << "<!ELEMENT " << name;
		cwcp_order  cwcps = get_cwcp_order(c);

		bool empty = true;
		bool need_delimiter = false;
		cwcp_order::iterator cwcps_i = cwcps.begin();
		while (cwcps_i != cwcps.end())
		{
			
			if (empty)
			{
				empty = false;
				output << " (";
			}

			
			if (need_delimiter)
				output << ", ";
			
			output  << card_to_dtdspec(*cwcps_i);

			need_delimiter = true;
			cwcps_i++;

		}
		
		if (!empty)
			output << ")";

		else 
			output << " EMPTY";

		output << ">" << endl;
	};

	void GenerateDTDAttlist(const Uml::Class &c,  ostream &output)	
	{
			string name = c.name();
			
			/*string topr("<!ATTLIST " + name + "\n");*/
			
			output << "<!ATTLIST " << name << endl;

			set<Uml::AssociationRole> associations = Uml::AncestorAssociationTargetRoles(c);
			set<Uml::Class> ccl = Uml::AncestorClasses(c);
			Uml::Association ass;
			for(set<Uml::Class>::iterator iii = ccl.begin(); iii != ccl.end(); iii++) {

				if(Uml::Association(iii->association())) {
					if(ass) {
						throw udm_exception("Class " + name + "has multiple associations"); 
					}
					else ass = iii->association();
				}
			}

			// we always need the _id, because every object may be derived/instance objects

			//if( !associations.empty() || ass) {
				/*{ output << topr; topr.erase(); }*/
				output << "\t_id\tID\t#IMPLIED" << endl;
			//}

			set<Uml::Class> cls = Uml::AncestorContainerClasses(c);
			set<Uml::Class>::iterator cl = cls.begin();

			
			while( cl != cls.end() )
			{
				if(!Uml::matchChildToParent(c,*cl)) {
						/*{ output << topr; topr.erase(); }*/

						// Jonathan requested that __child_as should be an enumerations
						// of all possible childRoles instead of an open NMTOKENS
						//endre, 04/22/02
						
						//get all my childRoles
						

						set<Uml::Class> anc_s = Uml::AncestorClasses(c);
						set<Uml::CompositionChildRole> ccr_s;
						set<Uml::Class>::iterator cl_i = anc_s.begin();

						while (cl_i != anc_s.end())
						{
							set<Uml::CompositionChildRole> this_ccr_s = (*cl_i).childRoles();
							
							set<Uml::CompositionChildRole>::iterator this_ccr_i = this_ccr_s.begin();
							
							while(this_ccr_i != this_ccr_s.end())
							{
								ccr_s.insert(*this_ccr_i);
								++this_ccr_i;
							}
							++cl_i;
						}
						output << "\t__child_as\t(";
						

						set<Uml::CompositionChildRole>::iterator ccr_i = ccr_s.begin();
						while (ccr_i != ccr_s.end())
						{
							//compname = GetANameFor(Uml::Composition(role.parent())).c_str();
							if (ccr_i != ccr_s.begin())  output << "|"; //delimiter
							
							if ( !((string)(*ccr_i).name()).empty())
								output << (string)(*ccr_i).name();
							else
							{
								Uml::Class target = ccr_i->target();
								output << (string)(target.name());
							}


							//output << (string)(*ccr_i).name();
							++ccr_i;
						}


						output << ")\t #IMPLIED" << endl;
						//output << "\t__child_as\tNMTOKENS\t#IMPLIED" << endl;
						break;
				}
				++cl;
			}

			set<Uml::Attribute> attributes = Uml::AncestorAttributes(c);
			set<Uml::Attribute>::iterator i = attributes.begin();
			while( i != attributes.end() )
			{
				name = i->name();
				if (i->nonpersistent())
				{
					//non-persistent attributes should not show up in the DTD
					i++;
					continue;
				}
				/*{ output << topr; topr.erase(); }*/
				output << "\t" << name.c_str() << "\t";

				string type = i->type();
				/*
					Boolean arrays are CDATA,
					Boolean simplevalues are either true or false.

					Todo: is there a DTD way to enforce "[true|false];" sequence?
				*/
				if( (type == "Boolean") && ((i->max()==1)||(i->max() == 0) ) )
					output << "(true|false)";
				else
					output << "CDATA";

	//int k = i->min();
	//int s = i->max();
				
	//			if(((string)(i->defvalue())).length()) {
				if(i->defvalue() ) 
				{
					//if it's non array, then the first element of defval(),
					//otherwise the DTD encoded version of the array

					if ((i->max()==1)||(i->max() == 0) )
						output << "\t\"" << (string)i->defvalue()[0] << "\"" << endl;
					else
					{
						//the attribute is array type

						vector<string> defvalstrarr = i->defvalue();
						vector<string>::const_iterator dv_i = defvalstrarr.begin();//((vector<string>)i->defvalue()).begin();

						output << "\t\"";
						while (dv_i != defvalstrarr.end())
						{
							
							const string a_item = *dv_i++;
							string::const_iterator a_item_ci = a_item.begin();
							while (a_item_ci != a_item.end())
							{

								switch (*a_item_ci)
								{
								case ';':
									output << "\\";			//escape the semicolon(;)
									output << *a_item_ci;	//write the character
									break;
								case '\\':
									output << "\\";			//escape the escape(\)
															//I think the grammar does not allow this
															//for the time being.
									output	 << *a_item_ci;	//write the character
									break;
								case '"':
									break;				//just ignore the ending and beginning "-s
								default:
									output << *a_item_ci;	//write the character

								}
						
								a_item_ci++;
							};  
							output << ";";
							
						}
						output << "\"" << endl;
					}
				}
				else {
					//output << (i->optional() ?"\t#IMPLIED" : "\t#REQUIRED") << endl;
					output << ( ((long)( i->min()) < 1) ?"\t#IMPLIED" : "\t#REQUIRED") << endl;
				}
				++i;
			}

			set<Uml::AssociationRole>::iterator j = associations.begin();
			while( j != associations.end() )
			{
				name = Uml::MakeRoleName(*j);
				/*{ output << topr; topr.erase(); }*/
				output << "\t" << name.c_str() << "\t";

				__int64 min = j->min();
				__int64 max = j->max();
				ASSERT( 0 <= min );
				ASSERT( max == -1 || min <= max );

				if( min == 1 && max == 1 )
					output << "IDREF\t#REQUIRED" << endl;
				else if( min == 0 && max == 1 )
					output << "IDREF\t#IMPLIED" << endl;
				else if( min >= 1 )
					output << "IDREFS\t#REQUIRED" << endl;
				else
					output << "IDREFS\t#IMPLIED" << endl;

				++j;
			}

			if(ass) {
				set<Uml::AssociationRole> rs(ass.roles());
				set<Uml::AssociationRole>::iterator j = rs.begin();
				while( j != rs.end() ) {
					name = Uml::MakeRoleName(*j);
					/*{ output << topr; topr.erase(); }*/
					output << "\t" << name.c_str() << "_end_\tIDREF\t#IMPLIED" << endl;
					++j;
				}
			}

			//add the archetype/derived/instances/desynched_atts
		output << "\t" << UdmDom::_udm_dom_ia_archetype <<"\tIDREF\t#IMPLIED" << endl;
		output << "\t" << UdmDom::_udm_dom_ia_derived <<"\tIDREFS\t#IMPLIED" << endl;
		output << "\t" << UdmDom::_udm_dom_ia_instances << "\tIDREFS\t#IMPLIED" << endl;
		output << "\t" << UdmDom::_udm_dom_ia_desynched_atts <<"\tCDATA\t#IMPLIED" << endl;
		output << "\t" << UdmDom::_udm_dom_ia_real_archetype <<"\t(true|false)\t#IMPLIED" << endl;
		output << "\t" << UdmDom::_udm_dom_ia_subtype << "\t(true|false)\t#IMPLIED" << endl;
		/*
			output << "\tarchetype \tIDREF\t#IMPLIED" << endl;
			output << "\tderived \tIDREFS\t#IMPLIED" << endl;
			output << "\tinstances \tIDREFS\t#IMPLIED" << endl;
			output << "\tdesynched_atts \tCDATA\t#IMPLIED" << endl;
			output << "\treal_archetype \t(true|false)\t#IMPLIED" << endl;
			output << "\tsubtype \t(true|false)\t#IMPLIED" << endl;
		*/
			/*if(topr.empty())*/ output << ">" << endl << endl;
	}

	void GenerateDTD(const Uml::Diagram &diagram,  ostream &output)	
	{
			output << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << endl;
			output << "<?udm interface=\"" << diagram.name() << "\" version=\"" << diagram.version() << "\"?>" << endl;
			output << "<!-- generated on " << GetTime().c_str() << " -->" << endl << endl;

			set<Uml::Class> classes = diagram.classes();
			set<Uml::Class>::iterator i = classes.begin();
			while( i != classes.end() )
			{
				if( !(bool)i->isAbstract() )
				{
					GenerateDTDElement(*i,  output);
					GenerateDTDAttlist(*i,  output);
				}

				++i;
			}
	}

	// --------------------------- XML Schema generation

void GenerateXMLSchemaAttributes(const Uml::Class &c,  ostream &output)	
{
		string name = c.name();
		set<Uml::AssociationRole> associations = Uml::AncestorAssociationTargetRoles(c);
		set<Uml::Class> ccl = Uml::AncestorClasses(c);
		Uml::Association ass;
		for(set<Uml::Class>::iterator iii = ccl.begin(); iii != ccl.end(); iii++) {

			if(Uml::Association(iii->association())) {
				if(ass) {
					throw udm_exception("Class " + name + "has multiple associations"); 
				}
				else ass = iii->association();
			}
		}

		// we always need the _id, because every object may be derived/instance objects
		output << "  <xsd:attribute name=\"_id\" type=\"xsd:ID\"/>" << endl;

		set<Uml::Class> cls = Uml::AncestorContainerClasses(c);
		set<Uml::Class>::iterator cl = cls.begin();
		while( cl != cls.end() )
		{
			if(!Uml::matchChildToParent(c,*cl)) {
					// Jonathan requested that __child_as should be an enumerations
					// of all possible childRoles instead of an open NMTOKENS
					//endre, 04/22/02
					
					//get all my childRoles
					set<Uml::Class> anc_s = Uml::AncestorClasses(c);
					set<Uml::CompositionChildRole> ccr_s;
					set<Uml::Class>::iterator cl_i = anc_s.begin();

					while (cl_i != anc_s.end())
					{
						set<Uml::CompositionChildRole> this_ccr_s = (*cl_i).childRoles();
						set<Uml::CompositionChildRole>::iterator this_ccr_i = this_ccr_s.begin();
						while(this_ccr_i != this_ccr_s.end())
						{
							ccr_s.insert(*this_ccr_i);
							++this_ccr_i;
						}
						++cl_i;
					}

					output << "  <xsd:attribute name=\"__child_as\">" << endl <<
						      "   <xsd:simpleType>" << endl <<
							  "    <xsd:restriction base=\"xsd:string\">" << endl;

					set<Uml::CompositionChildRole>::iterator ccr_i = ccr_s.begin();
					while (ccr_i != ccr_s.end())
					{
						//compname = GetANameFor(Uml::Composition(role.parent())).c_str();
						if ( !((string)(*ccr_i).name()).empty())
							output << "     <xsd:enumeration value=\"" <<
								(string)(*ccr_i).name() << "\"/>" << endl;
						else
						{
							Uml::Class target = ccr_i->target();
							output << "     <xsd:enumeration value=\"" <<
								(string)(target.name()) << "\"/>" << endl;
						}
						++ccr_i;
					}
					output << "    </xsd:restriction>" << endl <<
						      "   </xsd:simpleType>" << endl <<
							  "  </xsd:attribute>" << endl;
					break;
			}
			++cl;
		}

		set<Uml::Attribute> attributes = Uml::AncestorAttributes(c);
		set<Uml::Attribute>::iterator i = attributes.begin();
		while( i != attributes.end() )
		{
			name = i->name();
			if (i->nonpersistent())
			{
				//non-persistent attributes should not show up in the DTD
				i++;
				continue;
			}

			output << "  <xsd:attribute name=\"" << name.c_str() << "\" type=\"";

			/*
			   For compatibility, array of attributes are encoded as strings.
			   It is possible to abuse to abuse the "list" type to encode them,
			   but the item separator would be white-space and, in the case of
			   arrays of strings, this separator will not work correctly.
			*/
			string type = i->type();
			bool isArray = !( (i->max()==1) || (i->max() == 0) );
			if (type == "Boolean")
				output << (isArray ? "xsd:string" : "xsd:boolean");
			else if (type == "Integer")
				output << (isArray ? "xsd:string" : "xsd:long");
			else if (type == "Real")
				output << (isArray ? "xsd:string" : "xsd:double");
			else if (type == "String")
				output << (isArray ? "xsd:string" : "xsd:string");
			output << "\"";

			if(i->defvalue() ) 
			{
				//if it's non array, then the first element of defval(),
				//otherwise the DTD encoded version of the array

				if ((i->max()==1)||(i->max() == 0) )
					output << " default=\"" << (string)i->defvalue()[0] << "\"";
				else
				{
					//the attribute is array type

					vector<string> defvalstrarr = i->defvalue();
					vector<string>::const_iterator dv_i = defvalstrarr.begin();//((vector<string>)i->defvalue()).begin();

					output << " default=\"";
					while (dv_i != defvalstrarr.end())
					{
						const string a_item = *dv_i++;
						string::const_iterator a_item_ci = a_item.begin();
						while (a_item_ci != a_item.end())
						{

							switch (*a_item_ci)
							{
							case ';':
								output << "\\";			//escape the semicolon(;)
								output << *a_item_ci;	//write the character
								break;
							case '\\':
								output << "\\";			//escape the escape(\)
														//I think the grammar does not allow this
														//for the time being.
								output	 << *a_item_ci;	//write the character
								break;
							case '"':
								break;				//just ignore the ending and beginning quotes
							default:
								output << *a_item_ci;	//write the character

							}
					
							a_item_ci++;
						};  
						output << ";";

					}
					output << "\"";
				}
			}
			else {
				if ((long)( i->min() ) >= 1)
					output << " use=\"required\"";
			}
			output << "/>" << endl;
			++i;
		}

		set<Uml::AssociationRole>::iterator j = associations.begin();
		while( j != associations.end() )
		{
			name = Uml::MakeRoleName(*j);
			output << "  <xsd:attribute name=\"" << name.c_str() << "\"";

			__int64 min = j->min();
			__int64 max = j->max();
			ASSERT( 0 <= min );
			ASSERT( max == -1 || min <= max );

			if( min == 1 && max == 1 )
				output << " type=\"xsd:IDREF\" use=\"required\"/>" << endl;
			else if( min == 0 && max == 1 )
				output << " type=\"xsd:IDREF\"/>" << endl;
			else if( min >= 1 )
				output << " type=\"xsd:IDREFS\" use=\"required\"/>" << endl;
			else
				output << " type=\"xsd:IDREFS\"/>" << endl;

			++j;
		}

		if(ass) {
			set<Uml::AssociationRole> rs(ass.roles());
			set<Uml::AssociationRole>::iterator j = rs.begin();
			while( j != rs.end() ) {
				name = Uml::MakeRoleName(*j);
				output << "  <xsd:attribute name=\"" << name.c_str() <<
					"_end_\" type=\"xsd:IDREF\"/>" << endl;
				++j;
			}
		}

		//add the archetype/derived/instances/desynched_atts
		output << "  <xsd:attribute name=\"" << UdmDom::_udm_dom_ia_archetype << "\" type=\"xsd:IDREF\"/>" << endl;
		output << "  <xsd:attribute name=\"" << UdmDom::_udm_dom_ia_derived << "\" type=\"xsd:IDREFS\"/>" << endl;
		output << "  <xsd:attribute name=\"" << UdmDom::_udm_dom_ia_instances << "\" type=\"xsd:IDREFS\"/>" << endl;
		output << "  <xsd:attribute name=\"" << UdmDom::_udm_dom_ia_desynched_atts << "\" type=\"xsd:string\"/>" << endl;
		output << "  <xsd:attribute name=\"" << UdmDom::_udm_dom_ia_real_archetype << "\" type=\"xsd:boolean\"/>" << endl;
		output << "  <xsd:attribute name=\"" << UdmDom::_udm_dom_ia_subtype << "\" type=\"xsd:boolean\"/>" << endl;
}

void GenerateXMLSchemaElement(const Uml::Class &c,  ostream &output)	
{
	string name = c.name();
	cwcp_order  cwcps = get_cwcp_order(c);

	output << " <xsd:complexType name=\"" << name << "Type\">" << endl;
	if (!cwcps.empty()) {
		output << "  <xsd:sequence>" << endl;

		cwcp_order::iterator cwcps_i = cwcps.begin();
		while (cwcps_i != cwcps.end())
		{
			string i_name = cwcps_i->first.name();
			output << "   <xsd:element name=\"" << i_name << "\" type=\"" <<
				i_name << "Type\"";
			if (cwcps_i->second.first != 1)
				output << " minOccurs=\"" << cwcps_i->second.first << "\"";
			if (cwcps_i->second.second == -1)
				output << " maxOccurs=\"unbounded\"";
			else if	(cwcps_i->second.second != 1)
				output << " maxOccurs=\"" << cwcps_i->second.second << "\"";
			output << "/>" << endl;
			cwcps_i++;
		}

		output << "  </xsd:sequence>" << endl;
	}
	GenerateXMLSchemaAttributes(c, output);

	output << " </xsd:complexType>" << endl << endl;
}

void GenerateXMLSchema(const Uml::Diagram &diagram,  ostream &output)	
{

		output << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << endl;
		output << "<?udm interface=\"" << diagram.name() << "\" version=\"" << diagram.version() << "\"?>" << endl;
		output << "<xsd:schema xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\">" << endl;
		output << "<!-- generated on " << GetTime().c_str() << " -->" << endl << endl;

		vector<Uml::Class> globalElements;
		set<Uml::Class> classes = diagram.classes();
		set<Uml::Class>::iterator i = classes.begin();
		while( i != classes.end() )
		{
			if( !(bool)i->isAbstract() )
			{
				GenerateXMLSchemaElement(*i,  output);
				set<Uml::Class> p_c = Uml::AncestorContainerClasses(*i);		//all possible containers
				
				//the condition: It's not contained or it's only container is itself
				if ( (p_c.size() == 0) || ( (p_c.size() == 1) && ( *(p_c.begin()) == *i )) )
				//if  (Uml::AncestorContainerClasses(*i).size() < 1 ) //it's not self contained
				
					globalElements.push_back(*i);
			}
			++i;
		}

		vector<Uml::Class>::iterator j = globalElements.begin();
		while (j != globalElements.end()) {
				output << " <xsd:element name=\"" << j->name() <<
					"\" type=\"" << j->name() << "Type\"/>" << endl;
				j++;
		}

		output << endl << "</xsd:schema>" << endl;
}
