#include "TA_XSLT_Support.h"

// xerces
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/util/XMLUni.hpp>
#include <xercesc/framework/XMLFormatter.hpp>
#include <xercesc/util/TranscodingException.hpp>

#include <xercesc/dom/deprecated/DOM_DOMException.hpp>
#include <xercesc/dom/deprecated/DOM_NamedNodeMap.hpp>
#include <xercesc/dom/deprecated/DOMParser.hpp>
#include <xercesc/dom/DOM.hpp>

XERCES_CPP_NAMESPACE_USE

// xalan
#include <Include/PlatformDefinitions.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <XalanTransformer/XalanTransformer.hpp>

XALAN_CPP_NAMESPACE_USE

#include <iostream>
#include <fstream>
#include <windows.h>

extern void GenerateDTD(const Uml::Diagram &diagram, ostream &output);
extern void GenerateXMLSchema(const Uml::Diagram &diagram, ostream &output);

int OTIF_XSLT::XSLTransformer::executeTransform(const char *input, const char *xslt, const char *output, const bool generate_xsd, const char *rootclass) {

	int	theResult = 0;

	// Call the static initializer for Xerces.
	XMLPlatformUtils::Initialize();

	// Initialize Xalan.
	XalanTransformer::initialize();

	{
		// Create a XalanTransformer.
		XalanTransformer theXalanTransformer;

		// Do the transform.
		if (generate_xsd)
			theResult = theXalanTransformer.transform(input, xslt, ((string)"_" + (string)output).c_str());
		else
			theResult = theXalanTransformer.transform(input, xslt, output);

		if(theResult != 0) {
			throw xslt_exception(theXalanTransformer.getLastError(), 0);

/*			
			ofstream out_f;
			out_f.open("__xslt_error.txt");
			out_f << "XSLT transform error: " << theXalanTransformer.getLastError();
			out_f.close();
			out_f.clear();
*/
		}
	}

	// Terminate Xalan...
	XalanTransformer::terminate();

	// Terminate Xerces...
	XMLPlatformUtils::Terminate();

	// Clean up the ICU, if it's integrated...
	XalanTransformer::ICUCleanUp();


	// generate DOCTYPE
	if (theResult == 0 && generate_xsd) {

		ifstream in_f;
		in_f.open(((string)"_" + (string)output).c_str());
				
		ofstream out_f;
		out_f.open(output);

		char ch[1024];
		string a;
		while(!in_f.eof()) {
			in_f.getline(ch,1024);
			a = ch;
			if (!a.find("<?xml version=")) continue;
			out_f << ch << endl;
		}
				
		in_f.close();
		in_f.clear();
		out_f.close();
		out_f.clear();

		DeleteFile(((string)"_" + (string)output).c_str());
	}

	return theResult;
};

void OTIF_XSLT::GenerateDTDFile(const Uml::Diagram &diagram, const char *dtd_file) {
	ofstream ff;
	ff.open(dtd_file);
	GenerateDTD(diagram, ff);
	ff.close();
	ff.clear();
}

void OTIF_XSLT::GenerateXMLSchemaFile(const Uml::Diagram &diagram, const char *xsd_file) {
	ofstream ff;
	ff.open(xsd_file);
	GenerateXMLSchema(diagram, ff);
	ff.close();
	ff.clear();
}
