% Definition for the electronic throttle control monitor.

clear Definition

% Input definitions.

Definition.Input = struct([]);

Definition.Input(1).Name = 'trig_etc_monitor_fast';
Definition.Input(1).Description = 'trigger signal';
Definition.Input(1).Unit = 'none';
Definition.Input(1).FloatingPointTargetType = 'function call';
Definition.Input(1).FixedPointTargetType.Signed = [];
Definition.Input(1).FixedPointTargetType.Bytes = [];
Definition.Input(1).FixedPointTargetType.BinPoint = [];
Definition.Input(1).Size = [1 1];
Definition.Input(1).Min = [];
Definition.Input(1).Max = [];

Definition.Input(2).Name = 'tps1_volts';
Definition.Input(2).Description = 'throttle position sensor reading - sensor 1';
Definition.Input(2).Unit = 'volts';
Definition.Input(2).FloatingPointTargetType = 'double';
Definition.Input(2).FixedPointTargetType.Signed = 0;
Definition.Input(2).FixedPointTargetType.Bytes = 2;
Definition.Input(2).FixedPointTargetType.BinPoint = 2;
Definition.Input(2).Size = [1 1];
Definition.Input(2).Min = 5;
Definition.Input(2).Max = 12;

Definition.Input(3).Name = 'tps2_volts';
Definition.Input(3).Description = 'throttle position sensor reading - sensor 2';
Definition.Input(3).Unit = 'volts';
Definition.Input(3).FloatingPointTargetType = 'double';
Definition.Input(3).FixedPointTargetType.Signed = 0;
Definition.Input(3).FixedPointTargetType.Bytes = 2;
Definition.Input(3).FixedPointTargetType.BinPoint = 2;
Definition.Input(3).Size = [1 1];
Definition.Input(3).Min = 5;
Definition.Input(3).Max = 12;

Definition.Input(4).Name = 'actual_current';
Definition.Input(4).Description = 'measured throttle motor current';
Definition.Input(4).Unit = 'amps';
Definition.Input(4).FloatingPointTargetType = 'double';
Definition.Input(4).FixedPointTargetType.Signed = 0;
Definition.Input(4).FixedPointTargetType.Bytes = 2;
Definition.Input(4).FixedPointTargetType.BinPoint = 2;
Definition.Input(4).Size = [1 1];
Definition.Input(4).Min = -24;
Definition.Input(4).Max = 24;

Definition.Input(5).Name = 'task_manager_cntr';
Definition.Input(5).Description = 'counter to monitor servo task ';
Definition.Input(5).Unit = 'none';
Definition.Input(5).FloatingPointTargetType = 'int32';
Definition.Input(5).FixedPointTargetType.Signed = 0;
Definition.Input(5).FixedPointTargetType.Bytes = 2;
Definition.Input(5).FixedPointTargetType.BinPoint = 2;
Definition.Input(5).Size = [1 1];
Definition.Input(5).Min = 0;
Definition.Input(5).Max = 65535;

Definition.Input(6).Name = 'task_servo_cntr';
Definition.Input(6).Description = 'counter to monitor servo task ';
Definition.Input(6).Unit = 'none';
Definition.Input(6).FloatingPointTargetType = 'int32';
Definition.Input(6).FixedPointTargetType.Signed = 0;
Definition.Input(6).FixedPointTargetType.Bytes = 2;
Definition.Input(6).FixedPointTargetType.BinPoint = 2;
Definition.Input(6).Size = [1 1];
Definition.Input(6).Min = 0;
Definition.Input(6).Max = 65535;

Definition.Input(7).Name = 'desired_current';
Definition.Input(7).Description = 'desired throttle motor current';
Definition.Input(7).Unit = 'amps';
Definition.Input(7).FloatingPointTargetType = 'double';
Definition.Input(7).FixedPointTargetType.Signed = 0;
Definition.Input(7).FixedPointTargetType.Bytes = 2;
Definition.Input(7).FixedPointTargetType.BinPoint = 2;
Definition.Input(7).Size = [1 1];
Definition.Input(7).Min = -24;
Definition.Input(7).Max = 24;

% Output definitions.

Definition.Output = struct([]);

Definition.Output = struct([]);

Definition.Output(1).Name = 'which_faults';
Definition.Output(1).Description = 'fault conditions (FaultNoFault = 0, FaultHardwareTPS = 1, FaultHardwareActuator  = 2, FaultSoftwareManagerTask = 4)';
Definition.Output(1).Unit = 'enum'; 
Definition.Output(1).FloatingPointTargetType = 'int32';
Definition.Output(1).FixedPointTargetType.Signed = 0;
Definition.Output(1).FixedPointTargetType.Bytes = 2;
Definition.Output(1).FixedPointTargetType.BinPoint = 2;
Definition.Output(1).Size = [1 1];
Definition.Output(1).Min = 0;
Definition.Output(1).Max = 7;

% Parameter definitions.

Definition.Parameter = struct([]);

Definition.Parameter(1).Name = 'etc_monitor1_cpu_usage';
Definition.Parameter(1).Description = 'percentage of CPU troughput used by etc_monitor1';
Definition.Parameter(1).Unit = '';
Definition.Parameter(1).FloatingPointTargetType = 'double';
Definition.Parameter(1).FixedPointTargetType.Signed = 0;
Definition.Parameter(1).FixedPointTargetType.Bytes = 2;
Definition.Parameter(1).FixedPointTargetType.BinPoint = 10;
Definition.Parameter(1).Size = [1 1];
Definition.Parameter(1).Min = 0;
Definition.Parameter(1).Max = 100;
Definition.Parameter(1).DefaultValue = 2;

Definition.Parameter(2).Name = 'etc_monitor1_ROM';
Definition.Parameter(2).Description = 'bytes of ROM used by etc_monitor1';
Definition.Parameter(2).Unit = '';
Definition.Parameter(2).FloatingPointTargetType = 'double';
Definition.Parameter(2).FixedPointTargetType.Signed = 0;
Definition.Parameter(2).FixedPointTargetType.Bytes = 2;
Definition.Parameter(2).FixedPointTargetType.BinPoint = 10;
Definition.Parameter(2).Size = [1 1];
Definition.Parameter(2).Min = 0;
Definition.Parameter(2).Max = 1000000;
Definition.Parameter(2).DefaultValue = 50000;

Definition.Parameter(3).Name = 'etc_monitor1_RAM';
Definition.Parameter(3).Description = 'bytes of RAM used by etc_monitor1, includes stack';
Definition.Parameter(3).Unit = '';
Definition.Parameter(3).FloatingPointTargetType = 'double';
Definition.Parameter(3).FixedPointTargetType.Signed = 0;
Definition.Parameter(3).FixedPointTargetType.Bytes = 2;
Definition.Parameter(3).FixedPointTargetType.BinPoint = 10;
Definition.Parameter(3).Size = [1 1];
Definition.Parameter(3).Min = 0;
Definition.Parameter(3).Max = 32000;
Definition.Parameter(3).DefaultValue = 4000;
