#include "aif.h"
#include "UdmDom.h"
#include "UdmCORBA.h"

using namespace AIF;

class myToolAdaptor : public UdmCORBA::ToolAdaptor {

public:
	myToolAdaptor(const char *type, int argc, char **argv) 
		: ToolAdaptor(type, argc, argv) {};

	void notify(const char *par_name, const char *par_version, const char *name, const char *version, const char *remarks, long docId, long keepAlive) {

		cout << endl << "***Restore CORBADataNetwork***" << endl;

		dn->CreateNew("__pull_aif.xml", "aif", AIF::System::meta, Udm::CHANGES_PERSIST_ALWAYS);
		dn->fetchDocument(docId);

		cout << endl << "DataNetwork pulled..." << endl;

		dn->SaveAs("__pull_aif.xml");
		dn->CloseNoUpdate();
		dn->release();

		terminate();
	};

	void getFetchStatus(long percent) {
		cout << "#";
//		cout << percent << "%" << endl;
	}
};

int main(int argc, char* argv[]) {

	if (argc!=2) {
		cout << "Usage: Pull_AIF <tooladaptor type>" << endl;
		exit(1);
	}

	try {
		myToolAdaptor ta(argv[1], argc, argv);

		ta.subscribe(diagram);
		ta.run();
	
	} catch (udm_exception &e) {
		cerr << e.what() << endl;
	} catch (udmcorba_exception &e) {
		if ( e.code()<11 ) {
			cerr << endl << "CORBA exception: " << e.what() << endl;
		} else if (e.code()<21) {
			cerr << endl << "Paradigm exception: " << e.what() << endl;
		} else if (e.code()<41) {
			cerr << endl << "UdmCORBA exception: " << e.what() << endl;
		} else if (e.code()<91) {
			cerr << endl << "ToolAdaptor exception: " << e.what() << endl;
		}
	}

	return 0;
}
