#include "test.h"
#include "UdmDom.h"
#include "UdmCORBA.h"

using namespace test;

class myToolAdaptor : public UdmCORBA::ToolAdaptor {

public:
	myToolAdaptor(const char *type, int argc, char **argv) 
		: ToolAdaptor(type, argc, argv) {};

	void notify(const char *par_name, const char *par_version, const char *name, const char *version, const char *remarks, long docId, long keepAlive) {
	
		cout << endl << "Restore CORBADataNetwork (paradigm:" << par_name << " v" << par_version << ")" << endl;
		cout << name << ":" << version << ":" << remarks << endl;
		
		dn->CreateNew("__pull_test.xml", "test", Root::meta, Udm::CHANGES_PERSIST_ALWAYS);

//		cout << dn->getParadigmName() << endl;
//		cout << dn->getParadigmVersion() << endl;
		
		dn->fetchDocument(docId);
		cout << endl << "DataNetwork pulled..." << endl;

/*		
		A t_a = A::Cast(dn->GetObjectById(1));
		cout << (string) t_a.name() << endl;
		Root r = Root::Cast(dn->GetObjectById(100));
		set<A> as = r.A_kind_children();
		set<A>::iterator i = as.begin();
		A a1 = (*i);
		cout << "Root: " << dn->GetDatanetworkObjectID(r) << endl;
		cout << dn->GetDatanetworkObjectID(a1) << endl;
		cout << "Host info: " << dn->GetObjectHostInfo(r) << endl;
*/

		dn->SaveAs("__pull_test.xml");
		dn->CloseNoUpdate();
		dn->release();

		terminate();
	};

	void getFetchStatus(long percent) {
		cout << "#";
//		cout << percent << "%" << endl;
	}
};

int main(int argc, char* argv[]) {

	if (argc!=2) {
		cout << "Usage: Pull_Test <tooladaptor type>" << endl;
		exit(1);
	}

	try {
		myToolAdaptor ta(argv[1], argc, argv);

		ta.subscribe(diagram, "TEST", "1.0");
//		ta.subscribe(diagram, "TEST", "1.1");
		ta.run();
	
	} catch (udm_exception &e) {
		cerr << e.what() << endl;
	} catch (udmcorba_exception &e) {
		if ( e.code()<11 ) {
			cerr << endl << "CORBA exception: " << e.what() << endl;
		} else if (e.code()<21) {
			cerr << endl << "Paradigm exception: " << e.what() << endl;
		} else if (e.code()<41) {
			cerr << endl << "UdmCORBA exception: " << e.what() << endl;
		} else if (e.code()<91) {
			cerr << endl << "ToolAdaptor exception: " << e.what() << endl;
		}
	}

	return 0;
}


/*		
		UdmCORBA::setMetaModel("TEST", "1.0");
		UdmCORBA::CORBADataNetwork dn( diagram );

		dn.fetchDocument(1);

		cout << "DataNetwork pulled..." << endl;

		A t_a = A::Cast(dn.GetObjectById(1));
		cout << (string) t_a.name() << endl;

		Root r = Root::Cast(dn.GetObjectById(100));
//		Root r = Root::Cast(dn.GetRootObject());

		set<A> as = r.A_kind_children();
		set<A>::iterator i = as.begin();
		A a1 = (*i);

		cout << "Root: " << dn.GetDatanetworkObjectID(r) << endl;
		cout << dn.GetDatanetworkObjectID(a1) << endl;

/*		
		cout << "Root count: " << r.count() << endl;

		set<A> as = r.A_kind_children();
		for(set<A>::iterator i=as.begin(); i!=as.end(); i++) {
			cout << "A kind children: " << (string) (*i).name() << endl;
			cout << "  -parent: " << ((Root) (*i).parent()).count() << endl;

			set<A> t_as = (*i).A_kind_children();
			for(set<A>::iterator p=t_as.begin(); p!=t_as.end(); p++) {
				cout << "  -A kind children: " << (string) (*p).name() << endl;
			}

			set<Assoc_Class> assoc = (*i).dst();
			for(set<Assoc_Class>::iterator q=assoc.begin(); q!=assoc.end(); q++) {
				cout << "  -Assoc_Class kind children: " << (string) (*q).name() << endl;
			}

		}

		set<B> as2 = r.B_children();
		for(set<B>::iterator j=as2.begin(); j!=as2.end(); j++) {
			cout << "B kind children: " << (string) (*j).name() << endl;
			cout << "  -parent: " << ((Root) (*j).parent()).count() << endl;
			cout << "  -parent: " << ((Root) (*j).Root_parent()).count() << endl;

			set<Assoc_Class> assoc_class = (*j).src();
			for(set<Assoc_Class>::iterator p=assoc_class.begin(); p!=assoc_class.end(); p++) {
				cout << "  -Assoc_Class kind assoc: " << (string) (*p).name() << endl;
			}
		}


		{
			set<C> as3 = r.C_children();
			for(set<C>::iterator k=as3.begin(); k!=as3.end(); k++) {
				cout << "C kind children: " << (string) (*k).name() << endl;

				set<A> assoc = (*k).dst();
				for(set<A>::iterator q=assoc.begin(); q!=assoc.end(); q++) {
					cout << "  -A kind assoc: " << (string) (*q).name() << endl;
				}
			}
		}


		{
			set<Assoc_Class> assoc = r.Assoc_Class_children();
			for(set<Assoc_Class>::iterator k=assoc.begin(); k!=assoc.end(); k++) {
				cout << "Assoc_Class kind children: " << (string) (*k).name() << endl;

			}
		}
*/
