#include "esml.h"
#include "prism.h"
#include "UdmDom.h"
#include "UdmGme.h"
#include "UdmCORBA.h"

class myToolAdaptor : public UdmCORBA::ToolAdaptor {

public:
	myToolAdaptor(const char *type, int argc, char **argv) 
		: ToolAdaptor(type, argc, argv) {};

	void notify(const char *par_name, const char *par_version, const char *name, const char *version, const char *remarks, long docId, long keepAlive) {};

	void getPublishStatus(long percent) {
		cout << "#";
//		cout << percent << "%" << endl;
	}
};

int main(int argc, char* argv[]) {

	if (argc!=5) {
		cout << "Usage: Push_ESCM_UDM_ESML <tooladaptor type> <escm_udm filename.xml> <esml filename.mga> <keepAlive in second>" << endl;
		exit(1);
	}

	try {
		myToolAdaptor ta(argv[1], argc, argv);
		UdmCORBA::CORBADataNetwork *dn = ta.getDatanetwork(PRISM::diagram, "PRISM", "1.3");
		UdmCORBA::CORBADataNetwork *dn2 = ta.getDatanetwork(ESML::diagram, "ESML", "1.0");

		dn->OpenExisting(argv[2], "PRISM");
		dn2->OpenExisting(argv[3], "ESML");

		list<UdmCORBA::CORBADataNetwork*> docs;
		docs.push_back(dn);
		docs.push_back(dn2);

		ta.publishMultipleDocuments(docs, "escm_udm_test update", "1.0", "escm_udm test update file", atol(argv[4]));

		cout << endl << "DataNetwork pushed..." << endl;

	} catch (udm_exception &e) {
		cerr << e.what() << endl;
	} catch (udmcorba_exception &e) {
		if ( e.code()<11 ) {
			cerr << endl << "CORBA exception: " << e.what() << endl;
		} else if (e.code()<21) {
			cerr << endl << "Paradigm exception: " << e.what() << endl;
		} else if (e.code()<41) {
			cerr << endl << "UdmCORBA exception: " << e.what() << endl;
		} else if (e.code()<91) {
			cerr << endl << "ToolAdaptor exception: " << e.what() << endl;
		}
	}

	return 0;
}
