#include "matlab.h"
#include "UdmDom.h"
#include "UdmCORBA.h"

using namespace matlab;

class myToolAdaptor : public UdmCORBA::ToolAdaptor {

public:
	myToolAdaptor(const char *type, int argc, char **argv) 
		: ToolAdaptor(type, argc, argv) {};

	void notify(const char *par_name, const char *par_version, const char *name, const char *version, const char *remarks, long docId, long keepAlive) {};

	void getPublishStatus(long percent) {
		cout << "#";
//		cout << percent << "%" << endl;
	}
};

int main(int argc, char* argv[]) {
	
	if (argc!=4) {
		cout << "Usage: Push_Matlab <tooladaptor type> <filename.xml> <keepAlive in second>" << endl;
		exit(1);
	}

	try {
		myToolAdaptor ta(argv[1], argc, argv);
		UdmCORBA::CORBADataNetwork *dn = ta.getDatanetwork(diagram);

		dn->OpenExisting(argv[2], "matlab");

		dn->publishDocument("etc_control", "1.0", "xml file from etc_v2.0", atol(argv[3]));

		cout << endl << "DataNetwork pushed..." << endl;

	} catch (udm_exception &e) {
		cerr << e.what() << endl;
	} catch (udmcorba_exception &e) {
		if ( e.code()<11 ) {
			cerr << endl << "CORBA exception: " << e.what() << endl;
		} else if (e.code()<21) {
			cerr << endl << "Paradigm exception: " << e.what() << endl;
		} else if (e.code()<41) {
			cerr << endl << "UdmCORBA exception: " << e.what() << endl;
		} else if (e.code()<91) {
			cerr << endl << "ToolAdaptor exception: " << e.what() << endl;
		}
	}

	return 0;
}
