#include "test.h"
#include "UdmDom.h"
#include "UdmCORBA.h"

using namespace test;

class myToolAdaptor : public UdmCORBA::ToolAdaptor {

public:
	myToolAdaptor(const char *type, int argc, char **argv) 
		: ToolAdaptor(type, argc, argv) {};

	void notify(const char *par_name, const char *par_version, const char *name, const char *version, const char *remarks, long docId, long keepAlive) {};

	void getPublishStatus(long percent) {
		cout << "#";
//		cout << percent << "%" << endl;
	}
};

int main(int argc, char** argv) {

	if (argc!=4) {
		cout << "Usage: Push_TEST <tooladaptor type> <filename.xml> <keepAlive in second>" << endl;
		exit(1);
	}

	try {
		myToolAdaptor ta(argv[1], argc, argv);

		UdmCORBA::CORBADataNetwork *dn = ta.getDatanetwork(diagram, "TEST", "1.0");
		dn->OpenExisting(argv[2], "test");

		Root r = Root::Cast(dn->GetRootObject());
		dn->SetDatanetworkObjectID(r, 100);
		set<A> as = r.A_kind_children();
		set<A>::iterator i = as.begin();
		A a1 = (*i);
		dn->SetDatanetworkObjectID(a1, 1);


		dn->addAdditionalFile("bs2cfg.xsl");
		

		long doc_id = dn->publishDocument("test_doc", "1.0", "Document of the TESTv1.0 paradigm", atol(argv[3]));
		cout << endl << "DataNetwork pushed. DocID: " << doc_id << endl;

//		cout << "Host info: " << ta.getHostInfoString() << endl;
//		cout << "Host info: " << ta.getHostInfoLong() << endl;
		
//		UdmCORBA::CORBADataNetwork *dn2 = ta.getDatanetwork(diagram, "TEST", "1.1");
//		dn2->OpenExisting(argv[2], "test");
//		dn2->publishDocument("test_doc", "1.0", "Document of the TESTv1.1 paradigm", atol(argv[3]));
//		cout << "DataNetwork pushed..." << endl;

	} catch (udm_exception &e) {
		cerr << e.what() << endl;
	} catch (udmcorba_exception &e) {
		if ( e.code()<11 ) {
			cerr << endl << "CORBA exception: " << e.what() << endl;
		} else if (e.code()<21) {
			cerr << endl << "Paradigm exception: " << e.what() << endl;
		} else if (e.code()<41) {
			cerr << endl << "UdmCORBA exception: " << e.what() << endl;
		} else if (e.code()<91) {
			cerr << endl << "ToolAdaptor exception: " << e.what() << endl;
		}
	}

	return 0;
}


/*
		UdmCORBA::setMetaModel("TEST", "1.0");
		UdmCORBA::CORBADataNetwork dn(diagram);

		r.count() = 1000;

		A a_inst = A::Create(r);
		A a_inst2 = A::Create(r);
		A a_inst3 = A::Create(a_inst);
		B b_inst = B::Create(r);
		C c_inst = C::Create(r);

		Assoc_Class assoc_class_inst = Assoc_Class::Create(r);

		a_inst.name() = "a_inst";
		a_inst2.name() = "a_inst2";
		a_inst3.name() = "a_inst3";
		b_inst.name() = "b_inst";
		c_inst.name() = "c_inst";
		assoc_class_inst.name() = "assoc_class_inst";

		c_inst.dst() += a_inst;
		c_inst.dst() += a_inst2;
		c_inst.dst() += a_inst3;
		a_inst.src() += c_inst;
		a_inst2.src() += c_inst;

		assoc_class_inst.src_end() = a_inst;
		assoc_class_inst.dst_end() = b_inst;

		Root r2 = a_inst.parent();
		cout << "Root count: " << r2.count() << endl;
*/
