/*
Copyright (c) Vanderbilt University, 2000-2001
ALL RIGHTS RESERVED
Vanderbilt University disclaims all warranties with regard to this
software, including all implied warranties of merchantability
and fitness.  In no event shall Vanderbilt University be liable for
any special, indirect or consequential damages or any damages
whatsoever resulting from loss of use, data or profits, whether
in an action of contract, negligence or other tortious action,
arising out of or in connection with the use or performance of
this software.
*/

// ToolAdaptor_DSME_Publisher.cpp : Defines the class behaviors for the application.
//
#pragma warning( disable : 4666 )

#include "stdafx.h"
#include "ToolAdaptor_DSME_Publisher.h"
#include "TA_MFC_Support.h"

#include "DSME.h"
#include "UdmDom.h"
#include "UdmGME.h"
#include "UdmCORBA.h"

#include <map>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CToolAdaptor_DSME_PublisherApp

BEGIN_MESSAGE_MAP(CToolAdaptor_DSME_PublisherApp, CWinApp)
	//{{AFX_MSG_MAP(CToolAdaptor_DSME_PublisherApp)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG
	ON_COMMAND(ID_HELP, CWinApp::OnHelp)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CToolAdaptor_DSME_PublisherApp construction

CToolAdaptor_DSME_PublisherApp::CToolAdaptor_DSME_PublisherApp()
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CToolAdaptor_DSME_PublisherApp object

CToolAdaptor_DSME_PublisherApp theApp;


// *************************************************************************************************************************
// myToolAdaptor
// *************************************************************************************************************************
class myToolAdaptor : public UdmCORBA::ToolAdaptor {

public:
	myToolAdaptor(const char *type, int argc, char **argv) 
		: ToolAdaptor(type, argc, argv) {};

	void notify(const char *par_name, const char *par_version, const char *name, const char *version, const char *remarks, long docId, long keepAlive) {
	};
};

myToolAdaptor *ta;


/////////////////////////////////////////////////////////////////////////////
// CToolAdaptor_DSME_PublisherApp initialization

BOOL CToolAdaptor_DSME_PublisherApp::InitInstance()
{
	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.

	try {
	
		// publish Document
		CString name, version, remarks, pathname;
		long keepAlive;
		map<CString, CString> attachments;

		int result = showPublishAttachmentDlg(&name, &version, &remarks, &keepAlive, &pathname, &attachments);

		// Get the publishable filename
		int offset = pathname.ReverseFind('\\');
		CString filename = pathname.Right(pathname.GetLength()-(offset+1));
		
		if (result==IDOK) {
			ta = new myToolAdaptor("DSME_PUBLISHER", __argc, __argv);

			UdmCORBA::CORBADataNetwork *dn = ta->getDatanetwork(DSME::diagram);

			try {
				dn->OpenExisting((LPCSTR)pathname, "DSME");
			} catch (udm_exception &e) {
				string error = e.what();
				if (!error.compare("Not found")) MessageBox(NULL, "Invalid input file!", "Error", MB_OK);
				else MessageBox(NULL, e.what(), "Error", MB_OK);
				return false;
			}

			// Fill in the prjName with the filename except the extension
			DSME::RootFolder dsme_rf = DSME::RootFolder::Cast(dn->GetRootObject());
			string infilename = filename;	
			string dsme_nm = infilename.substr(0, infilename.rfind(".mga"));
			dsme_rf.prjName() = dsme_nm;

			// attach the attachments
			for(map<CString, CString>::iterator i=attachments.begin(); i!=attachments.end(); i++) dn->addAdditionalFile((*i).first, (*i).second);

			// publish the document
			dn->publishDocument((LPCSTR)name, (LPCSTR)version, (LPCSTR)remarks, keepAlive);

			dn->CloseNoUpdate();

			ta->logout();
		}

	} catch (udm_exception &e) {
		MessageBox(NULL, e.what(), "Error", MB_OK);
	} catch (udmcorba_exception &e) {
		MessageBox(NULL, e.what(), "Error", MB_OK);
	}

	// Since the dialog has been closed, return FALSE so that we exit the
	//  application, rather than start the application's message pump.
	return FALSE;
}
