/*
Copyright (c) Vanderbilt University, 2000-2001
ALL RIGHTS RESERVED
Vanderbilt University disclaims all warranties with regard to this
software, including all implied warranties of merchantability
and fitness.  In no event shall Vanderbilt University be liable for
any special, indirect or consequential damages or any damages
whatsoever resulting from loss of use, data or profits, whether
in an action of contract, negligence or other tortious action,
arising out of or in connection with the use or performance of
this software.
*/

// ToolAdaptor_ECSL.cpp : Defines the class behaviors for the application.
//
#pragma warning( disable : 4666 )

#include "stdafx.h"
#include "ToolAdaptor_ECSL.h"
#include "TA_MFC_Support.h"

#include "ecsl.h"
#include "matlab.h"
#include "UdmDom.h"
#include "UdmGME.h"
#include "UdmCORBA.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CToolAdaptor_ECSLApp

BEGIN_MESSAGE_MAP(CToolAdaptor_ECSLApp, CWinApp)
	//{{AFX_MSG_MAP(CToolAdaptor_ECSLApp)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG
	ON_COMMAND(ID_HELP, CWinApp::OnHelp)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CToolAdaptor_ECSLApp construction

CToolAdaptor_ECSLApp::CToolAdaptor_ECSLApp()
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
}


// *************************************************************************************************************************
// myToolAdaptor
// *************************************************************************************************************************
class myToolAdaptor : public UdmCORBA::ToolAdaptor {

public:
	myToolAdaptor(const char *type, int argc, char **argv) 
		: ToolAdaptor(type, argc, argv) {};

	void notify(const char *par_name, const char *par_version, const char *name, const char *version, const char *remarks, long docId, long keepAlive) {

		try {
			CString filename;
			
			while (filename.GetLength()<5 || filename.Right(4)!=".mga") {
				int result = showDatanetworkArrivedDlg(name, version, remarks, keepAlive, &filename, "mga");
				if (result!=IDOK) return;
				if (filename.GetLength()<5 || filename.Right(4)!=".mga") 
					MessageBox(NULL, "Invalid filename! Filename must end with .mga", "Error", MB_OK);
			}
		
			dn->CreateNew((LPCTSTR)filename, "ecsl", ecsl::RootFolder::meta, Udm::CHANGES_PERSIST_ALWAYS);
			dn->fetchDocument(docId);
			addStringFetchDlg("Datanetwork fetched.");

			dn->SaveAs((LPCSTR)filename);
			addStringFetchDlg("Datanetwork saved as " + filename);

			dn->CloseNoUpdate();
			dn->release();

			addStringFetchDlg("");
			addStringFetchDlg("Waiting for Datanetworks...");

		} catch (udm_exception &e) {
			addStringFetchDlg(e.what());
		} catch (udmcorba_exception &e) {
			addStringFetchDlg(e.what());
		} catch (...) {
			addStringFetchDlg("Fatal Error!");
			return;
		}

	};
};

myToolAdaptor *ta;

// *************************************************************************************************************************
// working Thread
// *************************************************************************************************************************
CWinThread *working_thread;

UINT MyWorkingThread( LPVOID pParam ) {

	try {
		myToolAdaptor *ta = new myToolAdaptor("ECSL_RECEIVER", __argc, __argv);

		addStringFetchDlg("Subscribe to ECSL paradigm v1.0");
		ta->subscribe(ecsl::diagram);
		addStringFetchDlg("Waiting for Datanetworks...");
		ta->run();

		ta->logout();
		delete ta;

	} catch (udm_exception &e) {
		MessageBox(NULL, e.what(), "Error!", MB_OK);
		return -1;
	} catch (udmcorba_exception &e) {
		MessageBox(NULL, e.what(), "Error", MB_OK);
		return -1;
	}

	return 0;
}




/////////////////////////////////////////////////////////////////////////////
// The one and only CToolAdaptor_ECSLApp object

CToolAdaptor_ECSLApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CToolAdaptor_ECSLApp initialization

BOOL CToolAdaptor_ECSLApp::InitInstance()
{
	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

	try {
	
		// publish Document
		CString name, version, remarks, filename;
		long keepAlive;

		int result = showPublishDlg(&name, &version, &remarks, &keepAlive, &filename);

		if (result==IDOK) {
			ta = new myToolAdaptor("MATLAB_PUBLISHER", __argc, __argv);

			UdmCORBA::CORBADataNetwork *dn = ta->getDatanetwork(matlab::diagram);

			try {
				dn->OpenExisting((LPCSTR)filename, "matlab");
			} catch (udm_exception &e) {
				string error = e.what();
				if (!error.compare("Not found")) MessageBox(NULL, "Invalid input file!", "Error", MB_OK);
				else MessageBox(NULL, e.what(), "Error", MB_OK);
				return false;
			}

			dn->publishDocument((LPCSTR)name, (LPCSTR)version, (LPCSTR)remarks, keepAlive);
			
			ta->logout();
		}

		// fetch Document
		working_thread = AfxBeginThread(MyWorkingThread, NULL);
		showFetchDlg("ToolAdaptor ECSL");

	} catch (udm_exception &e) {
		MessageBox(NULL, e.what(), "Error", MB_OK);
	} catch (udmcorba_exception &e) {
		MessageBox(NULL, e.what(), "Error", MB_OK);
	}

	// Since the dialog has been closed, return FALSE so that we exit the
	//  application, rather than start the application's message pump.
	return FALSE;
}
