#include <time.h>

#include <fstream>
#include <cstdio>

#include "SL.h"
#include "UdmDom.h"
#include "Uml.h"

#define DEF_OFFSET		20
#define DEF_PORT_SIZE	20
#define DEF_BUS_XSIZE	10
#define DEF_BUS_YSIZE	40
#define DEF_WIDTH		80
#define DEF_HEIGHT		40
#define POS_SCALE_FACTOR 1

char *indent[] = {
	"",
	" ",
	"  ",
	"   ",
	"    ",
	"     ",
	"      ",
	"       ",
	"        "
};

void Generate(SL::InputPort& sl_this, ofstream& mf, int level, string& path);
void Generate(SL::OutputPort& sl_this, ofstream& mf, int level, string& path);
void Generate(SL::Compound& sl_this, ofstream& mf, int level, string& path);
void Generate(SL::Primitive& sl_this, ofstream& mf, int level, string& path);
void Generate(SL::BusCreator& sl_this, ofstream& mf, int level, string& path);
void Generate(SL::BusSelector& sl_this, ofstream& mf, int level, string& path);

/*
int main(int argc, char *argv[])
{
	if (argc < 2)
	{
		cerr << "Usage: SL2M.exe <sl_file.xml> [m_file.m]" << endl;
		return 1;
	}

	string slfile = argv[1];
	string mfile;
	if (argc == 2)
	{
		mfile = slfile;
		int ext_pos = mfile.rfind('.');
		mfile.replace(ext_pos, 4, ".m");
	}
	else
		mfile = argv[2];

	cerr << slfile << " --> " << mfile << endl;
	cerr << "Generating .";

	try {
		UdmDom::DomDataNetwork sl(SL::diagram);
		sl.OpenExisting(slfile, "", Udm::CHANGES_LOST_DEFAULT);

		SL::Compound sl_root = SL::Compound::Cast(sl.GetRootObject());
		
		ofstream mf;
		mf.open(mfile.c_str());

		time_t ltime;
		time( &ltime );

		mf << "% " << mfile << " Matlab script for Simulink Model assembly" << endl;
		mf << "% Generated on: " << ctime( &ltime ) << endl << endl;

		int level = 1;
		string msys = mfile.substr(0, mfile.rfind(".m"));

		mf << indent[level] << "new_system('" << msys << "');"<< endl;
			Generate( sl_root, mf, level+1, msys );
		mf << indent[level] << "save_system('" << msys << "');"<< endl;
		mf << indent[level] << "close_system('" << msys << "');"<< endl;
	} catch (udm_exception e) {
		cerr << "ERROR: " << e.what() << endl;
		return 1;
	}

	cerr << endl << "Done." << endl;
	return 0;
}
*/



void Generate(SL::Compound& sl_this, ofstream& mf, int level, string& path)
{
	cerr << ".";

	string pos_str = sl_this.position();

	// first create a block for this compound
	mf  << indent[level] 
		<< "add_block(" 
		<< "'built-in/SubSystem',"		// block source
		<< "'" << path << "/" << string(sl_this.name()) << "',"			// block destination
		<< "'Position'," << pos_str.c_str()
		<< ");"
		<< endl;


	// initialize path/level to be used by children
	level++;
	string ch_path = path + string("/") + string(sl_this.name());

	// ports
	{
		set<SL::Port> ports = sl_this.Port_kind_children();
		for(set<SL::Port>::iterator i=ports.begin(); i!=ports.end(); i++)
		{
			SL::Port p = *i;
			if (p.type() == SL::InputPort::meta)
				Generate( SL::InputPort::Cast(p), mf, level, ch_path );
			else
				Generate( SL::OutputPort::Cast(p), mf, level, ch_path );
		}
	}

	// children
	{
		set<SL::Base> children = sl_this.Base_kind_children();
		for(set<SL::Base>::iterator i=children.begin(); i!=children.end(); i++)
		{
			SL::Base chb = *i;
			if (chb.type() == SL::Compound::meta)
				Generate( SL::Compound::Cast(chb), mf, level, ch_path );
			else if (chb.type() == SL::Primitive::meta)
				Generate( SL::Primitive::Cast(chb), mf, level, ch_path );
			else if (chb.type() == SL::BusSelector::meta)
				Generate( SL::BusSelector::Cast(chb), mf, level, ch_path );
			else if (chb.type() == SL::BusCreator::meta)
				Generate( SL::BusCreator::Cast(chb), mf, level, ch_path );
		}
	}

	// signals
	{
		set<SL::Signal> sigs = sl_this.Signal_kind_children();
		for(set<SL::Signal>::iterator i=sigs.begin(); i!=sigs.end(); i++)
		{
			SL::Signal sig = *i;
			SL::Port s = sig.src_end();
			SL::Base sp = s.Base_parent();
			SL::Port d = sig.dst_end();
			SL::Base dp = d.Base_parent();

			mf  << indent[level] 
				<< "l=add_line(" 
				<< "'" << ch_path << "',"			// context
				<< "'" << ((sp == sl_this) ? string(s.name()) : string(sp.name())) << "/" << ((sp == sl_this) ? 0 : s.id())+1 << "',"	// source
				<< "'" << ((dp == sl_this) ? string(d.name()) : string(dp.name())) << "/" << ((dp == sl_this) ? 0 : d.id())+1 << "',"	// destination
				<< "'autorouting', 'on'"
				<< ");"
				<< endl;
			mf	<< indent[level]
				<< "set_param(l,'Name','" << string(sig.name())	<< "');"
				<< endl;

		}
	}
}

void Generate(SL::InputPort& sl_this, ofstream& mf, int level, string& path)
{
	string pos_str = sl_this.position();

	mf  << indent[level] 
		<< "add_block(" 
		<< "'built-in/Inport',"		// block source
		<< "'" << path << "/" << string(sl_this.name()) << "',"			// block destination
		<< "'Position'," << pos_str.c_str() << ","
		<< "'Port','" << sl_this.id()+1 << "'"
		<< ");"
		<< endl;
}

void Generate(SL::OutputPort& sl_this, ofstream& mf, int level, string& path)
{
	string pos_str = sl_this.position();

	mf  << indent[level] 
		<< "add_block(" 
		<< "'built-in/Outport',"		// block source
		<< "'" << path << "/" << string(sl_this.name()) << "',"			// block destination
		<< "'Position'," << pos_str.c_str() << ","
		<< "'Port','" << sl_this.id()+1 << "'"
		<< ");"
		<< endl;
}

void Generate(SL::Primitive& sl_this, ofstream& mf, int level, string& path)
{
	cerr << ".";

	string pos_str = sl_this.position();

	string sys_name = string(sl_this.mdl_name());
	int mpos = sys_name.find(".mdl");
	if (mpos != string::npos) sys_name.erase(mpos);

	mf  << "load_system('" << sys_name << "')" << endl;

	mf  << indent[level] 
		<< "add_block(" 
		<< "'" << sys_name << "/" << string(sl_this.model_name()) << "',"		// block source
		<< "'" << path << "/" << string(sl_this.name()) << "',"			// block destination
		<< "'Position'," << pos_str.c_str()
		<< ");"
		<< endl;
}

void Generate(SL::BusCreator& sl_this, ofstream& mf, int level, string& path)
{
	cerr << ".";

	string pos_str = sl_this.position();

	mf  << indent[level] 
		<< "add_block(" 
		<< "'built-in/BusCreator',"		// block source
		<< "'" << path << "/" << string(sl_this.name()) << "',"			// block destination
		<< "'Position'," << pos_str.c_str() << ","
		<< "'Inputs','" << ((set<SL::InputPort>)(sl_this.InputPort_kind_children())).size() << "',"
		<< "'DisplayOption','bar',"
		<< "'ShowName','off'"
		<< ");"
		<< endl;
}

void Generate(SL::BusSelector& sl_this, ofstream& mf, int level, string& path)
{
	cerr << ".";

	string pos_str = sl_this.position();

	mf  << indent[level] 
		<< "add_block(" 
		<< "'built-in/BusSelector',"		// block source
		<< "'" << path << "/" << string(sl_this.name()) << "',"			// block destination
		<< "'Position'," << pos_str.c_str() << ","
		<< "'MuxedOutput'," << (sl_this.muxed() ? "'on'" : "'off'") << ","
		<< "'ShowName','off',"
		<< "'OutputSignals','";
	vector<string> out_sigs = sl_this.outputSignals();
	for(unsigned int i=0; i<out_sigs.size(); i++)
		mf	<< (i==0 ? "" : ",")
			<< out_sigs[i];
	mf	<< "');"
		<< endl;
}
