// ToolAdaptor_SL2M.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "ToolAdaptor_SL2M.h"
#include "TA_MFC_Support.h"

#include "SL.h"
#include "UdmDOM.h"
#include "UdmCORBA.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern char *indent[];
extern void Generate(SL::InputPort& sl_this, ofstream& mf, int level, string& path);
extern void Generate(SL::OutputPort& sl_this, ofstream& mf, int level, string& path);
extern void Generate(SL::Compound& sl_this, ofstream& mf, int level, string& path);
extern void Generate(SL::Primitive& sl_this, ofstream& mf, int level, string& path);
extern void Generate(SL::BusCreator& sl_this, ofstream& mf, int level, string& path);
extern void Generate(SL::BusSelector& sl_this, ofstream& mf, int level, string& path);


/////////////////////////////////////////////////////////////////////////////
// CToolAdaptor_SL2MApp

BEGIN_MESSAGE_MAP(CToolAdaptor_SL2MApp, CWinApp)
	//{{AFX_MSG_MAP(CToolAdaptor_SL2MApp)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG
	ON_COMMAND(ID_HELP, CWinApp::OnHelp)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CToolAdaptor_SL2MApp construction

CToolAdaptor_SL2MApp::CToolAdaptor_SL2MApp()
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
}

// *************************************************************************************************************************
// myToolAdaptor
// *************************************************************************************************************************
class myToolAdaptor : public UdmCORBA::ToolAdaptor {

public:
	myToolAdaptor(const char *type, int argc, char **argv) 
		: ToolAdaptor(type, argc, argv) {};

	void notify(const char *par_name, const char *par_version, const char *name, const char *version, const char *remarks, long docId, long keepAlive) {

		try {
			CString filename;
			
			while (filename.GetLength()<3 || filename.Right(2)!=".m") {
				int result = showDatanetworkArrivedDlg(name, version, remarks, keepAlive, &filename, "m");
				if (result!=IDOK) return;
				if (filename.GetLength()<3 || filename.Right(2)!=".m") 
					MessageBox(NULL, "Invalid filename! Filename must end with .m", "Error", MB_OK);
			}
		
//			dn->CreateNew((LPCTSTR)(filename+".xml"), "SL", SL::Compound::meta, Udm::CHANGES_PERSIST_ALWAYS);
			dn->CreateNew((LPCTSTR)(filename+".xml"), "SL", SL::Compound::meta, Udm::CHANGES_LOST_DEFAULT);
			dn->fetchDocument(docId);
			addStringFetchDlg("Datanetwork fetched.");

			string mfile = (LPCTSTR)filename;

			if (!isalpha(mfile[0])) // msys is not a valid matlab varname
				mfile.insert(0, "M");

			string relative_mfile = mfile.substr( mfile.rfind("\\")+1, mfile.size() );

			
			// translation process to .m
			SL::Compound sl_root = SL::Compound::Cast(dn->GetRootObject());
		
			ofstream mf;
			mf.open(mfile.c_str());

			time_t ltime;
			time( &ltime );

			mf << "% " << relative_mfile << " Matlab script for Simulink Model assembly" << endl;
			mf << "% Generated on: " << ctime( &ltime ) << endl << endl;
			
			int level = 1;
			string msys = relative_mfile.substr(0, relative_mfile.rfind(".m"));

			mf << indent[level] << "new_system('" << msys << "');"<< endl;
			Generate( sl_root, mf, level+1, msys );
			mf << indent[level] << "save_system('" << msys << "');"<< endl;
			mf << indent[level] << "close_system('" << msys << "');"<< endl;
			
			addStringFetchDlg("Translated matlab file saved.");
			
			dn->CloseNoUpdate();
//			dn->CloseWithUpdate();
			dn->release();

			addStringFetchDlg("");
			addStringFetchDlg("Waiting for Datanetworks...");

		} catch (udm_exception &e) {
			addStringFetchDlg(e.what());
		} catch (udmcorba_exception &e) {
			addStringFetchDlg(e.what());
		} catch (...) {
			addStringFetchDlg("Fatal Error!");
			return;
		}

	};
};

myToolAdaptor *ta;


// *************************************************************************************************************************
// working Thread
// *************************************************************************************************************************
CWinThread *working_thread;

UINT MyWorkingThread( LPVOID pParam ) {

	try {
		myToolAdaptor *ta = new myToolAdaptor("SL2M_TOOL", __argc, __argv);

		addStringFetchDlg("Subscribe to SL paradigm v1.0");
		ta->subscribe(SL::diagram);
		addStringFetchDlg("Waiting for Datanetworks...");
		ta->run();

		ta->logout();
		delete ta;

	} catch (udm_exception &e) {
		MessageBox(NULL, e.what(), "Error!", MB_OK);
		return -1;
	} catch (udmcorba_exception &e) {
		MessageBox(NULL, e.what(), "Error", MB_OK);
		return -1;
	}

	return 0;
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CToolAdaptor_SL2MApp object

CToolAdaptor_SL2MApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CToolAdaptor_SL2MApp initialization

BOOL CToolAdaptor_SL2MApp::InitInstance()
{
	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.

	try {
	
		// fetch Document
		working_thread = AfxBeginThread(MyWorkingThread, NULL);
		showFetchDlg("ToolAdaptor SL->M");

	} catch (udm_exception &e) {
		MessageBox(NULL, e.what(), "Error", MB_OK);
	} catch (udmcorba_exception &e) {
		MessageBox(NULL, e.what(), "Error", MB_OK);
	}

	// Since the dialog has been closed, return FALSE so that we exit the
	//  application, rather than start the application's message pump.
	return FALSE;
}
