// DSME2DESERT.cpp
// Author: Sandeep Neema
//
// Copyright (C) Vanderbilt University, 2002
// All Rights Reserved

#ifdef __INTEGRATED_INTO_OTIF__

	#include "UdmCORBA.h"
	#include "UdmCORBA_Logger.h"
	Logger logger = Logger("Translator_DSME2DESERT");

#endif


#include "Uml.h"
#include "UmlExt.h"
#include "UdmBase.h"
#include "UdmDom.h"
#include "UdmGme.h"

#include "DSME.h"
#include "DesertIface.h"

#include "Mfile.h"

struct ltpair
{
	bool operator()(const pair<DSME::Base,DSME::Base>& p1, const pair<DSME::Base,DSME::Base>& p2) const {
		return (p1.first == p2.first) ? (p1.second < p2.second) : (p1.first < p2.first);
	}
};

static void BuildPass1(	DSME::Base& dsme_this,
						DesertIface::Element& desert_this,
						DesertIface::ConstraintSet& desert_cset,
						DesertIface::DesertSystem& desert_root,
						map<string, DesertIface::NaturalDomain, ltstr>& desert_domains,
						map<DSME::Base, DesertIface::Element>& dsme_desert_map,
						int& id,
						int& eid);



void MainPass(DesertIface::DesertSystem &desert_root, DSME::RootFolder &dsme_rf) {

    int id = 0;
	int eid = 0;

	// create space
	DesertIface::Space desert_sp = DesertIface::Space::Create(desert_root);
	desert_sp.id() = id++;
	desert_sp.externalID() = -1;
	desert_sp.name() = "processes";
	desert_sp.decomposition() = true;

	// create a constraint set
	DesertIface::ConstraintSet desert_cs = DesertIface::ConstraintSet::Create(desert_root);
	desert_cs.id() = id++;
	desert_cs.externalID() = -1;
	desert_cs.name() = "constraints";

	map<DSME::Base, DesertIface::Element> dsme_desert_map;
	map<string, DesertIface::NaturalDomain, ltstr> desert_domains;

	// walk the input tree to create desert elements, properties, and constraints
#ifndef __INTEGRATED_INTO_OTIF__
	cerr << "Build Pass ";
#endif

	set<DSME::SignalFlow> dsme_sfs = dsme_rf.SignalFlow_children();
	for(set<DSME::SignalFlow>::iterator i=dsme_sfs.begin(); i!= dsme_sfs.end(); i++)
	{
		DSME::SignalFlow dsme_sf = *i;
		set<DSME::Base> dsme_children = dsme_sf.Base_kind_children();
		for(set<DSME::Base>::iterator i=dsme_children.begin(); i!=dsme_children.end(); i++)
		{
			DSME::Base dsme_child = *i;
			// create desert element
			DesertIface::Element desert_el = DesertIface::Element::Create(desert_sp);
			desert_el.id() = id++;
			desert_el.externalID() = desert_el.id();
			// set the ID of the DSME object
			dsme_child.ID() = desert_el.externalID();

			desert_el.name() = dsme_child.name();
			desert_el.decomposition() = false;	// true = Compound, false = Alternative|Primitive
			if (dsme_child.type() == DSME::Compound::meta)
			{
				set<DSME::Base> gc = DSME::Container::Cast(dsme_child).Base_kind_children();
				if (!gc.empty()) 
					desert_el.decomposition() = true;
			}
			dsme_desert_map[ dsme_child ] = desert_el;

			BuildPass1(dsme_child, desert_el, desert_cs, desert_root, desert_domains, dsme_desert_map, id, eid);
		}
	}

#ifndef __INTEGRATED_INTO_OTIF__
	cerr << "Done." << endl;
#endif
}


#ifndef __INTEGRATED_INTO_OTIF__

int main(int argc, char **argv)
{
  if(argc < 3)
  {
    cerr << "Usage: " << argv[0] << " <DSME_FileName.mga> <DESERT_FileName.xml>" << endl;
    return 1;
  }

  try
  {	
    string infilename = argv[1];
	string outfilename = argv[2];
	cerr << infilename << " --> " << outfilename << endl;

    // open input data network
    Udm::SmartDataNetwork input(DSME::diagram);
    input.OpenExisting(infilename, "", Udm::CHANGES_LOST_DEFAULT);

    // open output data network
    Udm::SmartDataNetwork output(DesertIface::diagram);
    output.CreateNew(outfilename, "DesertIface", DesertIface::DesertSystem::meta,Udm::CHANGES_PERSIST_ALWAYS);

	// dsme root object (root folder)
	DSME::RootFolder dsme_rf = DSME::RootFolder::Cast(input.GetRootObject());
	string dsme_nm = infilename.substr(0, infilename.rfind(".mga"));
	dsme_rf.prjName() = dsme_nm;

	// desert root object (system)
	DesertIface::DesertSystem desert_root = DesertIface::DesertSystem::Cast(output.GetRootObject());
	string dn = infilename + string("->") + outfilename;
    desert_root.SystemName() = dn.c_str();

	MainPass(desert_root, dsme_rf);

	// done
    input.CloseWithUpdate();
    output.CloseWithUpdate();
  }
  catch (const udm_exception &e)
  {
    cerr << "Exception: " << e.what()<<endl;
    return 1;
    throw;
  }

  return 0;
}


#endif


#ifdef __INTEGRATED_INTO_OTIF__

	static UdmCORBA::CORBADataNetwork *Input_DN;


	class myTranslator : public UdmCORBA::Translator {

	public:
		myTranslator(const Udm::UdmDiagram &from_metainfo, const char *from_name, const char *from_version, const Uml::Class &from_rootclass,
				const Udm::UdmDiagram &to_metainfo, char *to_name, char *to_version, const Uml::Class &to_rootclass,
				int argc, char **argv, Logger *logger)
				: Translator(from_metainfo, from_name, from_version, from_rootclass, 
							to_metainfo, to_name, to_version, to_rootclass, 
							argc, argv, logger) {};


		void notify(const char *name, const char *version, const char *remarks, const unsigned long numOfDocs) {

			try {
				Input_DN = fromDN;

				// dsme root object (root folder)
				DSME::RootFolder dsme_rf = DSME::RootFolder::Cast(fromDN->GetRootObject());

				string prjName = dsme_rf.prjName();
				if (prjName.empty()) {
					string dsme_nm = name;
					dsme_rf.prjName() = dsme_nm;
				}

				// desert root object (system)
				DesertIface::DesertSystem desert_root = DesertIface::DesertSystem::Cast(toDN->GetRootObject());
				string dn = name;
				desert_root.SystemName() = dn.c_str();

				MainPass(desert_root, dsme_rf);

				// pass the additional files through
//				toDN->copyAdditionalFilesFrom(fromDN);

				list<UdmCORBA::CORBADataNetwork*> docs;
				docs.push_back(toDN);
				docs.push_back(fromDN);
		
				publishMultipleDocuments(docs, name, version, "After the DSME->DESERT translation");

			} catch (udm_exception &e) {
				cout << e.what() << endl;
			}
		}
	};


	int main(int argc, char **argv) {

		try {
			myTranslator tr(DSME::diagram, "DSME", "1.0", DSME::RootFolder::meta, 
							DesertIface::diagram, "DESERT", "1.0", DesertIface::DesertSystem::meta, 
							argc, argv, &logger);

			tr.run();
		} catch (udm_exception &e) {
			cout << e.what() << endl;
			logger.log(1,  "Error", e.what());
		} catch (udmcorba_exception &e) {
			cout << e.what() << endl;
			logger.log(1,  "Error", e.what());
		} catch (...) {
			cerr << "Fatal exception" << endl;
			logger.log(1,  "Error", "Fatal exception");
		}	
		return 0;
	}

#endif


//////////////////////////////////////////////////////////////////////////////////////////


void BuildPass1(DSME::Base& dsme_this,
				DesertIface::Element& desert_this,
				DesertIface::ConstraintSet& desert_cset,
				DesertIface::DesertSystem& desert_root,
				map<string, DesertIface::NaturalDomain, ltstr>& desert_domains,
				map<DSME::Base, DesertIface::Element>& dsme_desert_map,
				int& id,
				int& eid)
{
#ifndef __INTEGRATED_INTO_OTIF__
	cerr << ".";
#endif

	// create properties
	{
		string def_name = string(dsme_this.name()) + string("_definition.m");
		string str = dsme_this.DefFile();
		if (!str.empty() && (str.find(".m") != string::npos)) def_name = str;

#ifndef __INTEGRATED_INTO_OTIF__
		MFile *m = new MFile(def_name.c_str());
#endif

#ifdef __INTEGRATED_INTO_OTIF__
		string temp_filename = Input_DN->getPhysicalFilename(def_name.c_str());
		if (!temp_filename.compare("")) temp_filename = def_name;
		MFile *m = new MFile(temp_filename.c_str());
#endif

		if (m->Parse())
		{
			int i;
			// create parameters
			for(i=m->parameters.size()-1; i>=0; i--)
			{
				MObject *m_obj = m->parameters[i];
				if (m_obj == NULL) continue;
				string pn = m_obj->name;
				string tn = dsme_this.name();
				// TBD: HACK HACK
				// property names are prefixed with the name of the component
				// eg: RAM --> etc_monitor_RAM
				// we strip out this prefix
				size_t pos = pn.find(tn);
				if (pos != string::npos)
				{
					pn.erase( 0, pos+tn.size()+1 );
#ifdef _DEBUG
					cout << endl << pn << endl;
#endif
				}

				// find or create a domain for the property
				DesertIface::NaturalDomain nd;
				map<string, DesertIface::NaturalDomain, ltstr>::iterator itr = desert_domains.find(pn);
				if (itr == desert_domains.end())
				{
					nd = DesertIface::NaturalDomain::Create(desert_root);
					nd.id() = id++;
					nd.externalID() = -1;
					nd.name() = pn;
					nd.minimum() = 0;
					nd.maximum() = 0;
					desert_domains[pn] = nd;
				}
				else
					nd = (*itr).second;

				// create the property
				//DesertIface::ConstantProperty prp = DesertIface::ConstantProperty::Create(desert_this);
				DesertIface::VariableProperty prp = DesertIface::VariableProperty::Create(desert_this);
				prp.id() = id++;
				prp.externalID() = -1;
				prp.name() = pn;
				prp.PCM_STR() = "PCM_ADD";
				prp.Max() = 0;
				prp.CUSTName() = "";
				prp.domain() = nd;
				nd.Property() += prp;

				int mxv = m_obj->typical;
				if (mxv < nd.minimum()) nd.minimum() = mxv;
				if (mxv > nd.maximum()) nd.maximum() = mxv;
				
				// create or find a domain member corresponding to the property value
				DesertIface::NaturalMember mbr;
				{
					bool found = false;
					set<DesertIface::NaturalMember> mbrs = nd.members();
					for(set<DesertIface::NaturalMember>::iterator i=mbrs.begin(); i!=mbrs.end() && !found; i++)
					if ( (*i).value() == mxv ) { found = true; mbr = *i; }

					if (!found)
					{
						mbr = DesertIface::NaturalMember::Create( nd );
						mbr.id() = id++;
						mbr.externalID() = -1;
						mbr.name() = pn;
						mbr.value() = mxv;
					}
				}

				// associate the domain member with the property
				DesertIface::AssignedValues av = DesertIface::AssignedValues::Create(desert_this);
				av.id() = id++;
				av.externalID() = -1;
				char buf[64];
				string avn = pn + string("=") + itoa(mxv, buf, 10);
				av.name() = avn;
				av.values_end() = mbr;
				av.properties_end() = prp;
			}
		} 
		else
		{
#if _DEBUG
			cerr << endl << "Warning: Definition file " << def_name << " not found!";
#endif
		}
		delete m;
	}


	// create constraints
	{
		set<DSME::Constraint> dsme_cons = dsme_this.Constraint_kind_children();
		for(set<DSME::Constraint>::iterator i=dsme_cons.begin(); i!=dsme_cons.end(); i++)
		{
			DSME::Constraint dsme_c = *i;
			DesertIface::Constraint desert_cc = DesertIface::Constraint::Create(desert_cset);
			desert_cc.id() = id++;
			desert_cc.externalID() = -1;;
			desert_cc.name() = string(dsme_c.name()) + string("(") + string(dsme_this.name()) + string(")");
			desert_cc.context() = desert_this;
			desert_cc.expression() = string("constraint ") + string(dsme_c.name()) + 
				string("() {\n") + string(dsme_c.Expression()) + string("\n}");
		}
	}

	// create elements -- and recurse
	if (Uml::IsDerivedFrom(dsme_this.type(), DSME::Container::meta))
	{
		set<DSME::Base> children = DSME::Container::Cast(dsme_this).Base_kind_children();
		for(set<DSME::Base>::iterator i=children.begin(); i!=children.end(); i++)
		{
			DSME::Base dsme_child = *i;
			DesertIface::Element desert_el = DesertIface::Element::Create(desert_this);
			desert_el.id() = id++;
			desert_el.externalID() = desert_el.id();
			// set the ID of the DSME object
			dsme_child.ID() = desert_el.externalID();

			desert_el.name() = dsme_child.name();
			desert_el.decomposition() = false;
			if (dsme_child.type() == DSME::Compound::meta)
			{
				set<DSME::Base> gc = DSME::Container::Cast(dsme_child).Base_kind_children();
				if (!gc.empty()) 
					desert_el.decomposition() = true;
			}
			dsme_desert_map[ dsme_child ] = desert_el;

			BuildPass1(dsme_child, desert_el, desert_cset, desert_root, desert_domains, dsme_desert_map, id, eid );
		}
	}
}
