#pragma warning(disable : 4786)


#include <iostream>
#include "Mfile.h"


void mk_string(string& pval, char *val)
{
  // matlab strings have a leading & trailing '
  // this eats the trailing '
  val[strlen(val) - 1] = 0;
  // this eats the leading '
  char *s = strchr(val, '\'');
  pval = s && *s ? s+1 : val;
}

void mk_vector(vector<int>& pval, char *val)
{
  char *s = strchr(val, '[') + 1;
  while ( s && *s && *s != '[' )
  {
    int n;
    if (sscanf(s, "%d", &n) > 0)
      pval.push_back(n);
    s = strchr(s, ' ');
    if (s) s++;
  }
}

void mk_integer(int& pval, char *val)
{
  pval = atoi(val);
}

#define MkParamFn( param, cvt_fn ) \
static void Set##param(MObject *obj, char *val) { cvt_fn(obj->##param, val); }

#define MkParamCvtFn( param, cvt_fn ) \
static void Set##param(MObject *obj, char *val) { cvt_fn(obj->##param, val); }

#define MkParam( actual_name, param_name ) \
{ actual_name, Set##param_name }

MkParamFn( name, mk_string );
MkParamFn( description, mk_string );
MkParamFn( unit, mk_string );
MkParamFn( flop_tt, mk_string );
MkParamFn( size, mk_vector );
MkParamFn( min, mk_integer );
MkParamFn( max, mk_integer );
MkParamFn( typical, mk_integer );
MkParamFn( fixp_tt_signed, mk_integer );
MkParamFn( fixp_tt_bytes, mk_integer );
MkParamFn( fixp_tt_binp, mk_integer );

struct {
  const char *param_nm;
  void (* set_fn)(MObject *, char *value);
} param_table[] = {
  MkParam( "Name", name ),
  MkParam( "Description", description ),
  MkParam( "Unit", unit ),
  MkParam( "FloatingPointTargetType", flop_tt ),
  MkParam( "FixedPointTargetType.Signed", fixp_tt_signed ),
  MkParam( "FixedPointTargetType.Bytes", fixp_tt_bytes ),
  MkParam( "FixedPointTargetType.BinPoint", fixp_tt_binp ),
  MkParam( "Size", size ),
  MkParam( "Min", min ),
  MkParam( "Max", max ),
  MkParam( "DefaultValue", typical ),
  { 0, 0 }
};

#define INPUT_PATTERN  		"Definition.Input"
#define OUTPUT_PATTERN 		"Definition.Output"
#define PARAMETER_PATTERN 	"Definition.Parameter"

bool MFile::Parse()
{
  if (!is_open()) return false;


  int lp;
  vector<MObject *> *vp;
  map<string, MObject *, ltstr> *mp;
  MObject *mobj;

  int id;
  char param[128];
  char value[128];
  char line[1024];

  while(!eof())
  {
    getline(line, 1024);

    if (strlen(line) == 0) continue;

    if (line[0] == '%') continue;

    if (0 == strncmp(line, INPUT_PATTERN, strlen(INPUT_PATTERN)))
    {
      lp = strlen(INPUT_PATTERN);
      vp = &inputs;
      mp = &input_nms;
    }
    else if (0 == strncmp(line, OUTPUT_PATTERN, strlen(OUTPUT_PATTERN)))
    {
      lp = strlen(OUTPUT_PATTERN);
      vp = &outputs;
      mp = &output_nms;
    }
    else if (0 == strncmp(line, PARAMETER_PATTERN, strlen(PARAMETER_PATTERN)))
    {
      lp = strlen(PARAMETER_PATTERN);
      vp = &parameters;
      mp = &parameter_nms;
    }
    else
      continue;

    if (2 > sscanf(&line[lp], "(%d).%128s", &id, param))
      continue;

	char *svp = strchr(&line[lp], '=');
	if (svp == NULL) 
	{
		cerr << endl << "Warning: syntax error: \"" << line << "\" in " << filename << endl;
		continue;
	}
    strncpy( value, svp + 1, 128-1 );

    id--;                               // matlab uses a one-based index

    if (id < 0) continue;
    if (id > (int)vp->size()-1)
      vp->resize( vp->size() + 10, NULL );

    // kill the ; at the end of the value
    value[strlen(value)-1] = 0;

    if ( (*vp)[id] == NULL )
    {
      (*vp)[id] = new MObject;
      (*vp)[id]->id = id;
    }

    mobj = (*vp)[id];
    
    int i=0;
    while ( param_table[i].param_nm ) 
    {
      if ( 0 == strcmp(param_table[i].param_nm, param) )
      {
		if (0 == strcmp(param, "Name") && !mobj->name.empty())
		{
			cerr << endl << "Warning: Definition file <" << filename.c_str()
				 << "> contains multiple ports/parameters with ID <" << id+1 << ">" << endl; 
		}
        param_table[i].set_fn( mobj, value );
        break;
      }

      i++;
    }
    (*mp)[mobj->name] = mobj;
  }

  return true;
}

bool MFile::IsCompatible(MFile *other)
{
  if (!other)
    return true;
  map<string, MObject *, ltstr>& oins = other->input_nms;
  for(unsigned int i=0; i<outputs.size(); i++)
  {
    MObject *out = outputs[i];
    if (!out)
      continue;
    map<string, MObject *, ltstr>::iterator itr = oins.find(out->name);
    if (itr == oins.end())
      continue;
    MObject *in = (*itr).second;
    if (!in)
      continue;
    if (!out->IsCompatible(in))
	{
#if 1 // def _DEBUG

	  cout << endl << filename << "::" << out->name << " not compatible with " << other->filename << "::" << in->name << ". ";
#endif
      return false;
	}
  }
  return true;
}

bool MObject::IsCompatible(MObject *other)
{
  bool ret;
  ret = (unit == other->unit);
  ret = ret && (flop_tt == other->flop_tt);
  ret = ret && (size.size() == other->size.size());
  for(unsigned int i=0; ret && i<size.size(); i++)
    ret = ret && (size[i] == other->size[i]);
  ret = ret && (min == other->min);
  ret = ret && (max == other->max);
  ret = ret && (typical == other->typical);
  ret = ret && (fixp_tt_signed == other->fixp_tt_signed);
  ret = ret && (fixp_tt_bytes == other->fixp_tt_bytes);
  ret = ret && (fixp_tt_binp == other->fixp_tt_binp);

  return ret;
}



#ifdef TEST_MPARSER
void main(int argc, char *argv[])
{
  if (argc < 2) { printf("Usage: %s <m_fileName>", argv[0]); exit(1); }
  MFile mfile(argv[1]);
  mfile.Parse();
  {
    int i,n;
    for(i=0, n=mfile.inputs.size()-1; n>=0; i++, n--)
    {
      if (mfile.inputs[i] == NULL) continue;
      printf("%d: Input(%d).Name = %s\n", i,
             mfile.inputs[i]->id, mfile.inputs[i]->name);
    }
    for(i=0, n=mfile.outputs.size()-1; n>=0; i++, n--)
    {
      if (mfile.outputs[i] == NULL) continue;
      printf("%d: Output(%d).Name = %s\n", i,
             mfile.outputs[i]->id, mfile.outputs[i]->name);
    }
    for(i=0, n=mfile.parameters.size()-1; n>=0; i++, n--)
    {
      if (mfile.parameters[i] == NULL) continue;
      printf("%d: Parameter(%d).Name = %s\n", i,
             mfile.parameters[i]->id, mfile.parameters[i]->name);
    }
  }
  exit(0);
}
#endif
