// -*-C++-*-
#ifndef MFILE_H
#define MFILE_H

#include <fstream>
#include <vector>
#include <map>
#include <string>

using namespace std;

struct ltstr
{
  bool operator()(const string& s1, const string& s2) const {
    return s1.compare(s2) < 0;
  }
};



struct MObject
{
  int id;

  string name;
  string description;
  string unit;
  string flop_tt;

  vector<int> size;

  int min;
  int max;
  int typical;
  int fixp_tt_signed;
  int fixp_tt_bytes;
  int fixp_tt_binp;

  MObject() : min(0),max(0),typical(0),fixp_tt_signed(0),fixp_tt_bytes(0),fixp_tt_binp(0) { }

  bool IsCompatible(MObject *other);
};

class MFile : public ifstream
{
public:
  MFile() : ifstream() { }
  MFile(const char *file) : ifstream(file, ios::in), inputs(10),
                            outputs(10), parameters(10), filename(file) { }
  virtual ~MFile() {
    int i;
    for(i=inputs.size()-1; i>=0; i--)  if (inputs[i]) delete inputs[i];
    for(i=outputs.size()-1; i>=0; i--)  if (outputs[i]) delete outputs[i];
    for(i=parameters.size()-1; i>=0; i--)  if (parameters[i]) delete parameters[i];
  }
  bool Parse();
  bool IsCompatible(MFile *other);

  string filename;
  vector<MObject *> inputs;
  vector<MObject *> outputs;
  vector<MObject *> parameters;
  map<string, MObject *, ltstr> input_nms;
  map<string, MObject *, ltstr> output_nms;
  map<string, MObject *, ltstr> parameter_nms;
};

#endif // whole file
