#ifdef __INTEGRATED_INTO_OTIF__

	#include "UdmCORBA.h"
	#include "UdmCORBA_Logger.h"
	Logger logger = Logger("Translator_DSME2SL");

#endif

#include <cstdio>

#pragma warning(disable:4244)

#include "DSME.h"
#include "SL.h"
#include "MFile.h"
#include "UdmDom.h"
#include "UdmGme.h"
#include "Uml.h"

#include <queue>


#define DEF_OFFSET		20
#define DEF_PORT_SIZE	20
#define DEF_BUS_XSIZE	10
#define DEF_BUS_YSIZE	40
#define DEF_XSIZE		160
#define DEF_YSIZE		40


static char name_buf[64];
static map<SL::Compound, SL::Primitive> singletons;
static map<DSME::ConnBase, SL::Port> portMap;
static map<DSME::ConnBase, SL::Bus> busMap;
#ifndef __INTEGRATED_INTO_OTIF__
static void Generate(DSME::RootFolder& dsme_root, SL::Compound& sl_root, UdmDom::DomDataNetwork& sl_dn);
#endif
#ifdef __INTEGRATED_INTO_OTIF__
static void Generate(DSME::RootFolder& dsme_root, SL::Compound& sl_root, UdmCORBA::CORBADataNetwork *sl_dn);
#endif
static void Generate(DSME::Base& dsme_this, SL::Compound& sl_parent);
static void Generate1(SL::Compound& sl_comp);
static void Generate2(SL::Base& sl_comp);
#ifndef __INTEGRATED_INTO_OTIF__
static void Generate3(SL::Compound& sl_comp, UdmDom::DomDataNetwork& sl_dn);
#endif
#ifdef __INTEGRATED_INTO_OTIF__
static void Generate3(SL::Compound& sl_comp, UdmCORBA::CORBADataNetwork *sl_dn);
#endif
static void Generate4(SL::Compound& sl_comp);
static void Generate5(SL::Compound& sl_comp);
#ifndef __INTEGRATED_INTO_OTIF__
static bool FindSignalPath(const string& sig_name, string& sig_path, SL::Signal& sig_this, UdmDom::DomDataNetwork& sl_dn, int nesting=0);
#endif
#ifdef __INTEGRATED_INTO_OTIF__
static bool FindSignalPath(const string& sig_name, string& sig_path, SL::Signal& sig_this, UdmCORBA::CORBADataNetwork *sl_dn, int nesting=0);
#endif



#ifndef __INTEGRATED_INTO_OTIF__

int main(int argc, char *argv[])
{
	if (argc < 3)
	{
		cerr << "Usage: HSF2M.exe <dsme_file.mga> <sl_file.xml>" << endl;
		return 1;
	}

	string dsmefile = argv[1];
	string slfile = argv[2]; 

	cerr << dsmefile << " --> " << slfile << endl;
	try {
		UdmGme::GmeDataNetwork dsme(DSME::diagram);
		dsme.OpenExisting(dsmefile, "", Udm::CHANGES_LOST_DEFAULT);
		DSME::RootFolder dsme_root = DSME::RootFolder::Cast(dsme.GetRootObject());

		UdmDom::DomDataNetwork sl(SL::diagram);
		sl.CreateNew(slfile, "SL", SL::Compound::meta, Udm::CHANGES_PERSIST_ALWAYS);
		SL::Compound sl_root = SL::Compound::Cast(sl.GetRootObject());
		string dsme_nm = dsme_root.prjName();
		sl_root.name() = dsme_nm;
		sl_root.def_file() = dsme_nm + string("_definition.m");

		sl_root.isRoot() = true;

		Generate(dsme_root, sl_root, sl);

		singletons.clear();
		portMap.clear();
		busMap.clear();

		dsme.CloseNoUpdate();
		sl.CloseWithUpdate();
		
	} catch (udm_exception e) {
		cerr << endl << "ERROR: " << e.what() << endl;
		singletons.clear();
		portMap.clear();
		busMap.clear();
		return 1;
	}

	cerr << endl << "Done." << endl;

	return 0;
}

#endif


#ifdef __INTEGRATED_INTO_OTIF__

	static UdmCORBA::CORBADataNetwork *Input_DN;


	class myTranslator : public UdmCORBA::Translator {

	public:
		myTranslator(const Udm::UdmDiagram &from_metainfo, const char *from_name, const char *from_version, const Uml::Class &from_rootclass,
				const Udm::UdmDiagram &to_metainfo, char *to_name, char *to_version, const Uml::Class &to_rootclass,
				int argc, char **argv, Logger *logger)
				: Translator(from_metainfo, from_name, from_version, from_rootclass, 
							to_metainfo, to_name, to_version, to_rootclass, 
							argc, argv, logger) {};


		void notify(const char *name, const char *version, const char *remarks, const unsigned long numOfDocs) {

			try {
				Input_DN = fromDN;

				// dsme root object (root folder)
				DSME::RootFolder dsme_root = DSME::RootFolder::Cast(fromDN->GetRootObject());

				// SL root object
				SL::Compound sl_root = SL::Compound::Cast(toDN->GetRootObject());

				string dsme_nm = dsme_root.prjName();
				sl_root.name() = dsme_nm;
				sl_root.def_file() = dsme_nm + string("_definition.m");

				sl_root.isRoot() = true;

				Generate(dsme_root, sl_root, toDN);

				singletons.clear();
				portMap.clear();
				busMap.clear();

				publish(name, version, "After the DSME->SL translation");

			} catch (udm_exception &e) {
				cout << e.what() << endl;
				singletons.clear();
				portMap.clear();
				busMap.clear();
			}
		}
	};


	int main(int argc, char **argv) {

		try {
			myTranslator tr(DSME::diagram, "DSME", "1.0", DSME::RootFolder::meta, 
							SL::diagram, "SL", "1.0", SL::Compound::meta, 
							argc, argv, &logger);

			tr.run();
		} catch (udm_exception &e) {
			cout << e.what() << endl;
			logger.log(1,  "Error", e.what());
		} catch (udmcorba_exception &e) {
			cout << e.what() << endl;
			logger.log(1,  "Error", e.what());
		} catch (...) {
			cerr << "Fatal exception" << endl;
			logger.log(1,  "Error", "Fatal exception");
		}	
		return 0;
	}

#endif


//////////////////////////////////////////////////////////////////////////////////////////



#ifndef __INTEGRATED_INTO_OTIF__
void Generate(DSME::RootFolder& dsme_root, SL::Compound& sl_root, UdmDom::DomDataNetwork& sl_dn)
#endif
#ifdef __INTEGRATED_INTO_OTIF__
void Generate(DSME::RootFolder& dsme_root, SL::Compound& sl_root, UdmCORBA::CORBADataNetwork *sl_dn)
#endif
{
#ifndef __INTEGRATED_INTO_OTIF__
	cerr << "Build Pass1 ";
#endif
	set<DSME::SignalFlow> dsme_sfs = dsme_root.SignalFlow_kind_children();
	for(set<DSME::SignalFlow>::iterator i=dsme_sfs.begin(); i!=dsme_sfs.end(); i++)
	{
		DSME::SignalFlow dsme_sf = *i;
		set<DSME::Base> dsme_children = dsme_sf.Base_kind_children();
		for(set<DSME::Base>::iterator j=dsme_children.begin(); j!=dsme_children.end(); j++)
		{
			DSME::Base dsme_child = *j;
			Generate(dsme_child, sl_root);
		}
	}
#ifndef __INTEGRATED_INTO_OTIF__
	cerr << endl;

	cerr << "Build Pass2 ";
#endif
	Generate1(sl_root);	// create ports/signals per definition files
#ifndef __INTEGRATED_INTO_OTIF__
	cerr << endl;

	cerr << "Build Pass3 ";
#endif
	Generate2(sl_root);	// resolve bus-membership and equivalence
#ifndef __INTEGRATED_INTO_OTIF__
	cerr << endl;

	cerr << "Build Pass4 ";
#endif
	Generate3(sl_root, sl_dn);	// assign output-names to bus selectors
#ifndef __INTEGRATED_INTO_OTIF__
	cerr << endl;

	cerr << "Graphical Layout Pass1 ";
#endif
	Generate4(sl_root);	// assign output-names to bus selectors
#ifndef __INTEGRATED_INTO_OTIF__
	cerr << endl;

	cerr << "Graphical Layout Pass2 ";
#endif
	{
		int rins = ((set<SL::InputPort>)sl_root.InputPort_kind_children()).size();
		int routs = ((set<SL::OutputPort>)sl_root.OutputPort_kind_children()).size();
		int np = max(rins,routs);
		sprintf(name_buf, "[%d %d %d %d]", DEF_OFFSET, DEF_OFFSET, DEF_OFFSET+DEF_XSIZE, DEF_OFFSET+DEF_YSIZE*np);
		sl_root.position() = string(name_buf);
	}
	Generate5(sl_root);	// assign output-names to bus selectors
#ifndef __INTEGRATED_INTO_OTIF__
	cerr << endl;
#endif

}

void Generate(DSME::Base& dsme_this, SL::Compound& sl_parent)
{
#ifndef __INTEGRATED_INTO_OTIF__
	cerr << ".";
#endif

	SL::Base sl_this;

	if (dsme_this.type() == DSME::Primitive::meta)
	{
		SL::Compound sl_comp = SL::Compound::Create(sl_parent);
		sl_comp.isSingleton() = true;
		DSME::Primitive dsme_prim = DSME::Primitive::Cast(dsme_this);
		SL::Primitive sl_prim = SL::Primitive::Create(sl_comp);
		sl_prim.name() = dsme_prim.name();
		sl_prim.def_file() = dsme_prim.DefFile();
		sl_prim.mdl_name() = dsme_prim.SimulinkFile();
		sl_prim.model_name() = dsme_prim.SimulinkModel();
		singletons[sl_comp] = sl_prim;
		sl_this = SL::Base::Cast(sl_comp);
	}
	else if (dsme_this.type() == DSME::Compound::meta)
	{
		DSME::Compound dsme_comp = DSME::Compound::Cast(dsme_this);
		SL::Compound sl_comp = SL::Compound::Create(sl_parent);
		sl_this = SL::Base::Cast(sl_comp);

		set<DSME::Base> dsme_children = dsme_comp.Base_kind_children();
		for(set<DSME::Base>::iterator i=dsme_children.begin(); i!=dsme_children.end(); i++)
		{
			DSME::Base dsme_child = *i;
			Generate(dsme_child, sl_comp);
		}
	}
	else
	{
		ASSERT(false);
		throw udm_exception("Unknown DSME type -- Alternatives are not allowed in this translation");
	}

	// set attribute
	sl_this.name() = dsme_this.name();

	// create ports 
	{
		// input ports
		set<DSME::InputPort> dsme_ins = dsme_this.InputPort_kind_children();
		int pid=0;
		for(set<DSME::InputPort>::iterator i=dsme_ins.begin(); i!=dsme_ins.end(); i++)
		{
			DSME::InputPort dsme_in = *i;
			SL::InputPort sl_in = SL::InputPort::Create(sl_this);
			sl_in.id() = pid++;
			sl_in.name() = dsme_in.name();
			portMap[dsme_in] = sl_in;
		}
	}
	{
		// output ports
		set<DSME::OutputPort> dsme_outs = dsme_this.OutputPort_kind_children();
		int pid=0;
		for(set<DSME::OutputPort>::iterator i=dsme_outs.begin(); i!=dsme_outs.end(); i++)
		{
			DSME::OutputPort dsme_out = *i;
			SL::OutputPort sl_out = SL::OutputPort::Create(sl_this);
			sl_out.id() = pid++;
			sl_out.name() = dsme_out.name();
			portMap[dsme_out] = sl_out;
		}
	}
	// create muxes/demuxes/stars
	{
		int mxc=0;
		set<DSME::Mux> mxs = dsme_this.Mux_kind_children();
		for(set<DSME::Mux>::iterator i=mxs.begin(); i!=mxs.end(); i++,mxc++)
		{
			DSME::Mux dsme_mx = *i;
			// create a bus-creator with ports in it
			SL::BusCreator sl_bc = SL::BusCreator::Create(sl_this);
			sprintf(name_buf, "bc%d", mxc);
			sl_bc.name() = string(name_buf);
			busMap[dsme_mx] = sl_bc;
			int pid=0;
			// create one output port
			SL::OutputPort sl_op = SL::OutputPort::Create(sl_bc);
			sl_op.name() = "o";
			sl_op.id() = pid;
			// create n input ports corresponding to the number of inputs
			set<DSME::Signal> dsme_sigs = dsme_mx.srcSignal();
			for(set<DSME::Signal>::iterator dsi=dsme_sigs.begin(); dsi!=dsme_sigs.end(); dsi++)
			{
				SL::InputPort sl_ip = SL::InputPort::Create(sl_bc);
				sl_ip.name() = "i";
				sl_ip.id() = pid++;
			}
		}
	}
	{
		int dxc=0;
		set<DSME::Demux> dxs = dsme_this.Demux_kind_children();
		for(set<DSME::Demux>::iterator i=dxs.begin(); i!=dxs.end(); i++,dxc++)
		{
			DSME::Demux dsme_dx = *i;
			SL::BusSelector sl_bs = SL::BusSelector::Create(sl_this);
			sprintf(name_buf, "bs%d", dxc);
			sl_bs.name() = string(name_buf);
			sl_bs.muxed() = true;
			busMap[dsme_dx] = sl_bs;
			int pid=0;
			// create one output port
			SL::OutputPort sl_op = SL::OutputPort::Create(sl_bs);
			sl_op.name() = "o";
			sl_op.id() = pid;
			// create one input port
			SL::InputPort sl_ip = SL::InputPort::Create(sl_bs);
			sl_ip.name() = "i";
			sl_ip.id() = pid;
		}
	}
	{
		int scc=0;
		set<DSME::StarConnector> scs=dsme_this.StarConnector_kind_children();
		for(set<DSME::StarConnector>::iterator i=scs.begin(); i!=scs.end(); i++,scc++)
		{
			DSME::StarConnector dsme_sc = *i;
			// create a bus-creator with ports in it
			SL::BusCreator sl_bc = SL::BusCreator::Create(sl_this);
			sprintf(name_buf, "bsc%d", scc);
			sl_bc.name() = string(name_buf);
			busMap[dsme_sc] = sl_bc;
			int pid=0;
			// create one output port
			SL::OutputPort sl_op = SL::OutputPort::Create(sl_bc);
			sl_op.name() = "o";
			sl_op.id() = pid;
			// create n input ports corresponding to the number of inputs
			set<DSME::Signal> dsme_sigs = dsme_sc.srcSignal();
			for(set<DSME::Signal>::iterator dsi=dsme_sigs.begin(); dsi!=dsme_sigs.end(); dsi++)
			{
				SL::InputPort sl_ip = SL::InputPort::Create(sl_bc);
				sl_ip.name() = "i";
				sl_ip.id() = pid++;
			}
		}
	}
	// create signals
	set<DSME::Signal> sigs = dsme_this.Signal_kind_children();
	for(set<DSME::Signal>::iterator sigi=sigs.begin(); sigi!=sigs.end(); sigi++)
	{
		DSME::Signal dsme_sig = *sigi;
		SL::Signal sl_sig = SL::Signal::Create(sl_this);
		DSME::ConnBase dsme_sco = dsme_sig.srcSignal_end();
		DSME::ConnBase dsme_dco = dsme_sig.dstSignal_end();
		SL::Port sco = portMap[dsme_sco];
		SL::Port dco = portMap[dsme_dco];
		if (sco == Udm::null)
		{
			// port is a mux/demux
			if(Udm::IsDerivedFrom(dsme_sco.type(), DSME::Bus::meta) || Udm::IsDerivedFrom(dsme_sco.type(), DSME::StarConnector::meta))
			{
				SL::Bus sl_bus = busMap[dsme_sco];
				set<SL::OutputPort> sl_ops = sl_bus.OutputPort_kind_children();
				ASSERT(sl_ops.size() == 1);
				if (sl_ops.size() < 1) throw udm_exception("unconnected bus output");
				sco = (*sl_ops.begin());
			}
			else
			{
				ASSERT(false);
				throw udm_exception("Unknown DSME src connector type");
			}
		}
		if (dco == Udm::null)
		{
			if(Udm::IsDerivedFrom(dsme_dco.type(), DSME::Bus::meta) || Udm::IsDerivedFrom(dsme_dco.type(), DSME::StarConnector::meta))
			{
				SL::Bus sl_bus = busMap[dsme_dco];
				set<SL::InputPort> sl_ips = sl_bus.InputPort_kind_children();
				// find an unused input port
				for(set<SL::InputPort>::iterator i=sl_ips.begin(); i!=sl_ips.end(); i++)
				{
					SL::InputPort sl_ip = *i;
					if ( set<SL::Signal>(sl_ip.src()).empty() )
					{
						dco = sl_ip;
						break;
					}
				}
			}
			else
			{
				ASSERT(false);
				throw udm_exception("Unknown DSME dst connector type");
			}

		}
		ASSERT(sco != Udm::null);
		ASSERT(dco != Udm::null);
		if (sco == Udm::null || dco == Udm::null)
			throw udm_exception("Unable to resolve source/destination connector");
		sl_sig.src_end() = sco;
		sl_sig.dst_end() = dco;
	}
}

void Generate1(SL::Compound& sl_comp)
{
#ifndef __INTEGRATED_INTO_OTIF__
	cerr << ".";
#endif

	int bcid=0;
	int bsid=0;

	SL::Base sl_this;
	bool isRoot = sl_comp.isRoot();
	bool isSingleton = sl_comp.isSingleton();

	if (isRoot)
		sl_this = SL::Base::Cast(sl_comp);
	else if (isSingleton)
	{
		map<SL::Compound, SL::Primitive>::iterator si=singletons.find(sl_comp);
		ASSERT(si!=singletons.end());
		if (si == singletons.end()) throw udm_exception("internal error - singleton not found");
		sl_this = SL::Base::Cast( (*si).second );
	}
	if(isRoot || isSingleton)
	{
		string def_file = sl_this.def_file();

#ifndef __INTEGRATED_INTO_OTIF__
		MFile *mf = new MFile(def_file.c_str());
#endif

#ifdef __INTEGRATED_INTO_OTIF__
		string temp_filename = Input_DN->getPhysicalFilename(def_file.c_str());
		if (!temp_filename.compare("")) temp_filename = def_file;
		MFile *mf = new MFile(temp_filename.c_str());
#endif

		if(mf->Parse())
		{
			{
				SL::Bus bus;
				for(int i=mf->inputs.size()-1,ins=0; i>=0; i--)
				{
					if(mf->inputs[i] == NULL) continue;
					SL::InputPort sl_min = SL::InputPort::Create(sl_this);
					sl_min.name() = mf->inputs[i]->name;
					sl_min.id() = mf->inputs[i]->id;
					if (bus == Udm::null)
					{
						if(isRoot)	// if we are at the root node then we wire all inputs into a bus-creator
									// create a single outputport in the bus-creator and wire that to the inputport of the
									// the children of root node -- there should be just one child (but we are not asserting)
						{
							SL::BusCreator sl_bc = SL::BusCreator::Create(sl_comp);
							sprintf(name_buf,"cbc%d",bcid++);
							sl_bc.name() = string(name_buf);
							bus = SL::Bus::Cast(sl_bc);
							SL::OutputPort sl_bout = SL::OutputPort::Create(bus);
							sl_bout.id() = 0;
							sl_bout.name() = "o";
							set<SL::Base> sl_children = sl_comp.Base_kind_children();
							for(set<SL::Base>::iterator ci=sl_children.begin(); ci!=sl_children.end(); ci++)
							{
								SL::Base sl_child = *ci;
								if (Udm::IsDerivedFrom(sl_child.type(), SL::Bus::meta)) continue;
								set<SL::InputPort> sl_ins = sl_child.InputPort_kind_children();
								ASSERT(sl_ins.size() > 0);
								if (sl_ins.size() == 0)
									throw udm_exception("No input ports in the root model, invalid architecture model");
								for(set<SL::InputPort>::iterator ii=sl_ins.begin(); ii!=sl_ins.end(); ii++)
								{
									SL::InputPort sl_in = *ii;
									SL::Signal sl_sig = SL::Signal::Create(sl_comp);
									sl_sig.src_end() = sl_bout;
									sl_sig.dst_end() = sl_in;
								}
							}
						}
						else		// else we create a bus-selector and wire the outputs into each input port
									// create an input port and wire that from the inputport of this node
						{
							SL::BusSelector sl_bs = SL::BusSelector::Create(sl_comp);
							sl_bs.muxed() = false;
							sprintf(name_buf,"cbs%d",bsid++);
							sl_bs.name() = string(name_buf);
							bus = SL::Bus::Cast(sl_bs);
							SL::InputPort sl_bin = SL::InputPort::Create(bus);
							sl_bin.id() = 0;
							sl_bin.name() = "i";
							set<SL::InputPort> sl_ins = sl_comp.InputPort_kind_children();
							for(set<SL::InputPort>::iterator ii=sl_ins.begin(); ii!=sl_ins.end(); ii++)
							{
								SL::InputPort sl_in = *ii;
								SL::Signal sl_sig = SL::Signal::Create(sl_comp);
								sl_sig.src_end() = sl_in;
								sl_sig.dst_end() = sl_bin;
							}

						}
					}
					// now that we have a bus -- as we walk along the created 
					if (isRoot)
					{
						// we are root - we have a bus creator -- we need to create inputport in this bus
						// and connect it to the inputport being created
						SL::InputPort sl_bin = SL::InputPort::Create(bus);
						sl_bin.id() = mf->inputs[i]->id;
						sl_bin.name() = mf->inputs[i]->name;
						SL::Signal sl_sig = SL::Signal::Create(sl_comp);
						sl_sig.src_end() = sl_min;
						sl_sig.dst_end() = sl_bin;
					}
					else
					{
						// we create outputport in the busselector -- and wire it to the inputports
						SL::OutputPort sl_bout = SL::OutputPort::Create(bus);
						sl_bout.id() = mf->inputs[i]->id;
						sl_bout.name() = mf->inputs[i]->name;
						SL::Signal sl_sig = SL::Signal::Create(sl_comp);
						sl_sig.src_end() = sl_bout;
						sl_sig.dst_end() = sl_min;
					}
				}
			}
			{
				SL::Bus bus;
				for(int i=mf->outputs.size()-1,outs=0; i>=0; i--)
				{
					if(mf->outputs[i] == NULL) continue;
					SL::OutputPort sl_mout = SL::OutputPort::Create(sl_this);
					sl_mout.name() = mf->outputs[i]->name;
					sl_mout.id() = mf->outputs[i]->id;
					if (bus == Udm::null)
					{
						if(isRoot)	// if we are at the root node then we wire all outputs into a bus-selector
									// create a single inputport in the bus-selector and wire that from the outputport of the
									// the children of root node -- there should be just one child (but we are not asserting)
						{
							SL::BusSelector sl_bs = SL::BusSelector::Create(sl_comp);
							sl_bs.muxed() = false;
							sprintf(name_buf,"cbs%d",bsid++);
							sl_bs.name() = string(name_buf);
							bus = SL::Bus::Cast(sl_bs);
							SL::InputPort sl_bin = SL::InputPort::Create(bus);
							sl_bin.id() = 0;
							sl_bin.name() = "i";
							set<SL::Base> sl_children = sl_comp.Base_kind_children();
							for(set<SL::Base>::iterator ci=sl_children.begin(); ci!=sl_children.end(); ci++)
							{
								SL::Base sl_child = *ci;
								if (Udm::IsDerivedFrom(sl_child.type(), SL::Bus::meta)) continue;
								set<SL::OutputPort> sl_outs = sl_child.OutputPort_kind_children();
								ASSERT(sl_outs.size() > 0);
								if (sl_outs.size() == 0)
									throw udm_exception("No output ports in the root model, invalid architecture model");
								for(set<SL::OutputPort>::iterator ii=sl_outs.begin(); ii!=sl_outs.end(); ii++)
								{
									SL::OutputPort sl_out = *ii;
									SL::Signal sl_sig = SL::Signal::Create(sl_comp);
									sl_sig.src_end() = sl_out;
									sl_sig.dst_end() = sl_bin;
								}
							}
							// should perhaps add an assert here
						}
						else		// else we create a bus-creator and wire the inputs from each output port
									// create an output port and wire that to the outputport of this node
						{
							SL::BusCreator sl_bc = SL::BusCreator::Create(sl_comp);
							sprintf(name_buf,"cbc%d",bcid++);
							sl_bc.name() = string(name_buf);
							bus = SL::Bus::Cast(sl_bc);
							SL::OutputPort sl_bout = SL::OutputPort::Create(bus);
							sl_bout.id() = 0;
							sl_bout.name() = "o";
							set<SL::OutputPort> sl_outs = sl_comp.OutputPort_kind_children();
							for(set<SL::OutputPort>::iterator ii=sl_outs.begin(); ii!=sl_outs.end(); ii++)
							{
								SL::OutputPort sl_out = *ii;
								SL::Signal sl_sig = SL::Signal::Create(sl_comp);
								sl_sig.src_end() = sl_bout;
								sl_sig.dst_end() = sl_out;
							}
						}
					}
					// now that we have a bus -- as we walk along the created 
					if (isRoot)
					{
						// we are root - we have a bus selector -- we need to create outputport in this bus
						// and connect it to the oututport being created
						SL::OutputPort sl_bout = SL::OutputPort::Create(bus);
						sl_bout.id() = mf->outputs[i]->id;
						sl_bout.name() = mf->outputs[i]->name;
						SL::Signal sl_sig = SL::Signal::Create(sl_comp);
						sl_sig.src_end() = sl_bout;
						sl_sig.dst_end() = sl_mout;
					}
					else
					{
						// we create inputport in the buscreator -- and wire it from the outputports
						SL::InputPort sl_bin = SL::InputPort::Create(bus);
						sl_bin.id() = mf->outputs[i]->id;
						sl_bin.name() = mf->outputs[i]->name;
						SL::Signal sl_sig = SL::Signal::Create(sl_comp);
						sl_sig.src_end() = sl_mout;
						sl_sig.dst_end() = sl_bin;
					}
				}
			}
		}
#ifndef __INTEGRATED_INTO_OTIF__
		else
			cout << endl << "Warning: Definition file <" << def_file.c_str() << "> not found" << endl;
#endif
		delete mf;
	}
	set<SL::Compound> bases = sl_comp.Compound_kind_children();
	for(set<SL::Compound>::iterator i=bases.begin(); i!=bases.end(); i++)
	{
		SL::Compound hc = *i;
		Generate1(hc);
	}
}

void Generate2(SL::Base& sl_this)
{
#ifndef __INTEGRATED_INTO_OTIF__
	cerr << ".";
#endif

	SL::Compound sl_comp;
	if (sl_this.type() == SL::Compound::meta)
		sl_comp = SL::Compound::Cast(sl_this);
	// we want to assign the membership  and set equivalence
	// then we will assign the name, as well as populate the outputsignals list of
	// busselectors
	{
		set<SL::InputPort> inputs = sl_this.InputPort_kind_children();
		for(set<SL::InputPort>::iterator i=inputs.begin(); i!=inputs.end(); i++)
		{
			SL::InputPort inp = *i;
			string sig_name;
			// find a source signal into this input port if there is one
			SL::Signal src_sig;
			set<SL::Signal> insigs = inp.src();
			ASSERT(insigs.size() <= 1);
			if (insigs.size() > 1) throw udm_exception("multiple inputs connecting into an input port");
			if (insigs.size() == 1)
			{
				SL::Signal isig = *(insigs.begin());
				src_sig = isig;
				sig_name = isig.name();
			}
			else
				sig_name = inp.name();

			set<SL::Signal> outsigs = inp.dst();
			for(set<SL::Signal>::iterator j=outsigs.begin(); j!=outsigs.end(); j++)
			{
				SL::Signal osig = *j;
				if (src_sig != Udm::null)
					osig.sameAs() = src_sig.uniqueId();
				osig.name() = sig_name;
			}
		}
	}
	if(sl_comp != Udm::null)
	{
		set<SL::BusCreator> bus_creats = sl_comp.BusCreator_kind_children();
		for(set<SL::BusCreator>::iterator i=bus_creats.begin(); i!=bus_creats.end(); i++)
		{
			SL::BusCreator bc = *i;
			set<SL::OutputPort> bc_outs = bc.OutputPort_kind_children();
			set<SL::InputPort> bc_ins = bc.InputPort_kind_children();
			ASSERT(bc_outs.size() == 1);
			if (bc_outs.size() != 1) 
				throw udm_exception("internal error - multiple output ports in bus");
			SL::OutputPort bc_out = *(bc_outs.begin());
			sprintf(name_buf, "s%lx", bc.uniqueId());
			string osig_name = name_buf;//string("s") + string(bc.name());
			set<SL::Signal> bc_osigs = bc_out.dst();

			vector<SL::InputPort> sorted_ins(bc_ins.size());
			{
				for(set<SL::InputPort>::iterator k=bc_ins.begin(); k!=bc_ins.end(); k++)
					sorted_ins[ (int)((*k).id()) ] = (*k);
			}

			for(set<SL::Signal>::iterator j=bc_osigs.begin(); j!=bc_osigs.end(); j++)
			{
				SL::Signal bc_osig = *j;
				bc_osig.name() = osig_name;
				for(unsigned int k=0; k<bc_ins.size(); k++)
				{
					SL::InputPort bc_in = sorted_ins[k];
					set<SL::Signal> bc_isigs = bc_in.src();
					ASSERT(bc_isigs.size() == 1);
					if (bc_isigs.size() != 1) throw udm_exception("internal error - insufficient inputs to bus inport");
					SL::Signal bc_isig = *(bc_isigs.begin());
					bc_osig.members() += bc_isig.uniqueId();
				}
			}
		}
	}
	if(sl_comp != Udm::null)
	{
		set<SL::Primitive> bases = sl_comp.Primitive_kind_children();
		for(set<SL::Primitive>::iterator i=bases.begin(); i!=bases.end(); i++)
		{
			SL::Primitive hc = *i;
			Generate2(hc);
		}
	}
	if(sl_comp != Udm::null)
	{
		set<SL::Compound> bases = sl_comp.Compound_kind_children();
		for(set<SL::Compound>::iterator i=bases.begin(); i!=bases.end(); i++)
		{
			SL::Compound hc = *i;
			Generate2(hc);
		}
	}
	if(sl_comp != Udm::null)
	{
		set<SL::BusSelector> bus_sels = sl_comp.BusSelector_kind_children();
		for(set<SL::BusSelector>::iterator i=bus_sels.begin(); i!=bus_sels.end(); i++)
		{
			SL::BusSelector bs = *i;
			set<SL::InputPort> bs_ins = bs.InputPort_kind_children();
			set<SL::OutputPort> bs_outs = bs.OutputPort_kind_children();
			ASSERT(bs_ins.size() == 1);
			if (bs_ins.size() != 1) throw udm_exception("internal error - bus selector with insufficient inputs");
			SL::InputPort bs_in = *(bs_ins.begin());
			set<SL::Signal> bs_isigs = bs_in.src();
			ASSERT(bs_isigs.size() == 1);
			if (bs_isigs.size() != 1) throw udm_exception("internal error - bus selector with insufficient inputs");
			SL::Signal bs_isig = *(bs_isigs.begin());
			if (bs.muxed())
			{
				ASSERT(bs_outs.size() == 1);
				if (bs_outs.size() != 1) throw udm_exception("internal error - muxed bus with insufficient outputs");
				SL::OutputPort bs_out = *(bs_outs.begin());
				set<SL::Signal> bs_osigs = bs_out.dst();
				ASSERT(bs_osigs.size() == 1);
				if (bs_osigs.size() != 1) throw udm_exception("internal error - muxed bus with insufficient outputs");
				SL::Signal bs_osig = *(bs_osigs.begin());
				bs_osig.sameAs() = bs_isig.uniqueId();
				bs_osig.name() = bs_isig.name();
			}
			else
			{
				// get the name from the destination port
				for(set<SL::OutputPort>::iterator j=bs_outs.begin(); j!=bs_outs.end(); j++)
				{
					SL::OutputPort bs_out = *j;
					set<SL::Signal> bs_osigs = bs_out.dst();
					ASSERT(bs_osigs.size() == 1);
					if (bs_osigs.size() != 1) throw udm_exception("internal error - bus output with multiple or none signals");
					SL::Signal bs_osig = *(bs_osigs.begin());
					SL::Port bs_odst = bs_osig.dst_end();
					bs_osig.name() = bs_odst.name();
				}
			}
		}
	}
	{
		set<SL::OutputPort> outputs = sl_this.OutputPort_kind_children();
		for(set<SL::OutputPort>::iterator i=outputs.begin(); i!=outputs.end(); i++)
		{
			SL::OutputPort outp = *i;
			string sig_name;
			// find a source signal into this output port if there is one
			SL::Signal src_sig;
			set<SL::Signal> insigs = outp.src();
			ASSERT(insigs.size() <= 1);
			if (insigs.size() > 1) throw udm_exception("multiple input signals");
			if (insigs.size() == 1)
			{
				SL::Signal isig = *(insigs.begin());
				src_sig = isig;
				sig_name = isig.name();
			}
			else
				sig_name = outp.name();

			set<SL::Signal> outsigs = outp.dst();
			for(set<SL::Signal>::iterator j=outsigs.begin(); j!=outsigs.end(); j++)
			{
				SL::Signal osig = *j;
				if (src_sig != Udm::null)
					osig.sameAs() = src_sig.uniqueId();
				osig.name() = sig_name;
			}
		}
	}
}

#ifndef __INTEGRATED_INTO_OTIF__
void Generate3(SL::Compound& sl_comp, UdmDom::DomDataNetwork& sl_dn)
#endif
#ifdef __INTEGRATED_INTO_OTIF__
void Generate3(SL::Compound& sl_comp, UdmCORBA::CORBADataNetwork *sl_dn)
#endif
{
#ifndef __INTEGRATED_INTO_OTIF__
	cerr << ".";
#endif

	{
		set<SL::Compound> bases = sl_comp.Compound_kind_children();
		for(set<SL::Compound>::iterator i=bases.begin(); i!=bases.end(); i++)
		{
			SL::Compound hc = *i;
			Generate3(hc, sl_dn);
		}
	}
	// for each bus-selector 
	{
		set<SL::BusSelector> bus_sels = sl_comp.BusSelector_kind_children();
		for(set<SL::BusSelector>::iterator i=bus_sels.begin(); i!=bus_sels.end(); i++)
		{
			SL::BusSelector bs = *i;
			set<SL::InputPort> bs_ins = bs.InputPort_kind_children();
			set<SL::OutputPort> bs_outs = bs.OutputPort_kind_children();
			ASSERT(bs_ins.size() == 1);
			if (bs_ins.size() < 1) 
				throw udm_exception("disconnected bus");
			SL::InputPort bs_in = *(bs_ins.begin());
			set<SL::Signal> bs_isigs = bs_in.src();
			ASSERT(bs_isigs.size() == 1);
			if (bs_isigs.size() < 1) 
				throw udm_exception("disconnected bus");
			SL::Signal bs_isig = *(bs_isigs.begin());
			if (bs.muxed())
			{
				vector<__int64> mbrs = bs_isig.members();
				for(unsigned int j=0; j<mbrs.size(); j++)
				{
#ifndef __INTEGRATED_INTO_OTIF__
					Udm::Object obj = sl_dn.ObjectById((long)mbrs[j]);
#endif
#ifdef __INTEGRATED_INTO_OTIF__
					Udm::Object obj = sl_dn->ObjectById((long)mbrs[j]);
#endif
					SL::Signal sig = SL::Signal::Cast(obj);
					bs.outputSignals() += string(sig.name());
				}
			}
			else
			{
				int num_outs = bs_outs.size();
				vector<SL::OutputPort> sorted_outs(num_outs);
				for(set<SL::OutputPort>::iterator j=bs_outs.begin(); j!=bs_outs.end(); j++)
				{
					SL::OutputPort bs_out = *j;
					int id = (int)bs_out.id();
					ASSERT(id>=0 && id<num_outs);
					if (id<0 || id >=num_outs)
						throw udm_exception("invalid port id");
					sorted_outs[id] = bs_out;
				}
				for(int k=0; k<num_outs; k++)
				{
					SL::OutputPort bs_out = sorted_outs[k];
					set<SL::Signal> bs_osigs = bs_out.dst();
					ASSERT(bs_osigs.size() == 1);
					if (bs_osigs.size() < 1) 
						throw udm_exception("disconnected bus");
					SL::Signal bs_osig = *(bs_osigs.begin());
					string sn = bs_osig.name();
					string sig_path;
					bool found = FindSignalPath(sn, sig_path, bs_isig, sl_dn);
					ASSERT(found);
					if (found)
					{
						sig_path += sn;
						bs.outputSignals() += sig_path;
					}
#ifndef __INTEGRATED_INTO_OTIF__
					else
						cerr << endl << "Warning: Bus-Signal <" << sn << "> not found" << endl;
#endif

#ifdef _DEBUG
					cerr << endl << sig_path;
#endif
				}
#ifdef _DEBUG
				cerr << endl;
#endif
			}
		}
	}
}

#ifndef __INTEGRATED_INTO_OTIF__
bool FindSignalPath(const string& sig_name, string& sig_path, SL::Signal& sig_this, UdmDom::DomDataNetwork& sl_dn, int nesting)
#endif
#ifdef __INTEGRATED_INTO_OTIF__
bool FindSignalPath(const string& sig_name, string& sig_path, SL::Signal& sig_this, UdmCORBA::CORBADataNetwork *sl_dn, int nesting)
#endif
{
	string sig_this_nm = sig_this.name();
	if (sig_name.compare(sig_this_nm) == 0)
		return true;
	int uid = (int)sig_this.sameAs();
	if (uid > 0)
	{
#ifndef __INTEGRATED_INTO_OTIF__
		Udm::Object obj = sl_dn.ObjectById(uid);
#endif
#ifdef __INTEGRATED_INTO_OTIF__
		Udm::Object obj = sl_dn->ObjectById(uid);
#endif
		SL::Signal sig = SL::Signal::Cast(obj);
		return FindSignalPath(sig_name, sig_path, sig, sl_dn, nesting);
	}
	vector<__int64> mbrs = sig_this.members();
	for(unsigned int i=0; i<mbrs.size(); i++)
	{
		uid = (long)mbrs[i];
#ifndef __INTEGRATED_INTO_OTIF__
		Udm::Object obj = sl_dn.ObjectById(uid);
#endif
#ifdef __INTEGRATED_INTO_OTIF__
		Udm::Object obj = sl_dn->ObjectById(uid);
#endif
		SL::Signal sig = SL::Signal::Cast(obj);
		bool found = FindSignalPath(sig_name, sig_path, sig, sl_dn, nesting+1);
		if (!found) continue;
		if (nesting > 0) sig_path = string(sig_this.name()) + string(".") + sig_path;
		return true;
	}

	return false;
}

void Generate4(SL::Compound& sl_this)
{
#ifndef __INTEGRATED_INTO_OTIF__
	cerr << ".";
#endif

	queue<SL::Base> base_q;
	unsigned int max_cols=0;
	vector<int> max_rows(10);


	// walk input ports
	{
		set<SL::InputPort> ins = sl_this.InputPort_kind_children();
		vector<SL::InputPort> sorted_ins(ins.size());
		for(set<SL::InputPort>::iterator i=ins.begin(); i!=ins.end(); i++)
			sorted_ins[(int)((*i).id())] = *i;

		for(unsigned int j=0; j<ins.size(); j++)
		{
			SL::InputPort& ip = sorted_ins[j];
			ip.column() = 0;
			ip.row() = j;

			set<SL::Signal> sigs = ip.dst();
			for(set<SL::Signal>::iterator k=sigs.begin(); k!=sigs.end(); k++)
			{
				SL::Signal sig = *k;
				SL::Port port = sig.dst_end();
				SL::Base parent = port.Base_parent();
				if (parent == sl_this) continue;
				string p_pos = parent.position();
				if (!p_pos.empty()) continue;
				parent.position() = string("[-1 -1 -1 -1]");
				parent.column() = 1;
				base_q.push(parent);
			}
		}
	}

	// process queued bases
	for(int done=0; done!=1;)
	{
		while(!base_q.empty())
		{
			SL::Base base = base_q.front();
			base_q.pop();

			unsigned int c = (int)base.column();
			base.row() = max_rows[c]++;

			set<SL::OutputPort> ops = base.OutputPort_kind_children();
			for(set<SL::OutputPort>::iterator i=ops.begin(); i!=ops.end(); i++)
			{
				SL::OutputPort op=*i;
				set<SL::Signal> sigs = op.dst();
				for(set<SL::Signal>::iterator k=sigs.begin(); k!=sigs.end(); k++)
				{
					SL::Signal sig = *k;
					SL::Port port = sig.dst_end();
					SL::Base parent = port.Base_parent();
					if (parent == sl_this) continue;
					string p_pos = parent.position();
					if (!p_pos.empty()) continue;
					parent.position() = string("[-1 -1 -1 -1]");
					parent.column() = c+1;
					base_q.push(parent);
				}
			}
			if (c > max_cols)
				max_cols = c;

			if (max_cols >= max_rows.size())
				max_rows.resize( max_cols + 10);

		}	// end while(base_q not empty)

		// find all children that have not been reached yet and add them to queue
		{
			done = 1;
			set<SL::Base> bases = sl_this.Base_kind_children();
			for(set<SL::Base>::iterator i=bases.begin(); i!=bases.end(); i++)
			{
				SL::Base base = *i;
				string b_pos = base.position();
				if (!b_pos.empty()) continue;
				base.position() = string("[-1 -1 -1 -1]");
				base.column() = 1;
				base_q.push(base);
				done = 0;
				break;
			}
		}

	}
	// walk output ports
	{
		set<SL::OutputPort> outs = sl_this.OutputPort_kind_children();
		vector<SL::OutputPort> sorted_outs(outs.size());
		for(set<SL::OutputPort>::iterator i=outs.begin(); i!=outs.end(); i++)
			sorted_outs[(int)((*i).id())] = *i;

		int c=max_cols+1;
		for(unsigned int j=0; j<outs.size(); j++)
		{
			SL::OutputPort& op = sorted_outs[j];
			op.column() = c;
			op.row() = j;
		}
	}

	// recurse
	{
		set<SL::Compound> comps = sl_this.Compound_kind_children();
		for(set<SL::Compound>::iterator i=comps.begin(); i!=comps.end(); i++)
			Generate4(SL::Compound::Cast(*i));
	}
}

void Generate5(SL::Compound& sl_this)
{
#ifndef __INTEGRATED_INTO_OTIF__
	cerr << ".";
#endif

	// sort all children by row/col numbers
	vector< vector<SL::GraphicalObj> > grid(10);

	set<SL::Port> ports = sl_this.Port_kind_children();
	for(set<SL::Port>::iterator pi=ports.begin(); pi!=ports.end(); pi++)
	{
		SL::GraphicalObj g = SL::GraphicalObj::Cast(*pi);
		unsigned int gc = (int)g.column();
		unsigned int gr = (int)g.row();
		if (gc >= grid.size())
			grid.resize(gc + 10);
		vector<SL::GraphicalObj>& grid_col = grid[gc];
		if (gr >= grid_col.size())
			grid_col.resize(gr + 10);
		grid_col[gr] = g;
	}

	set<SL::Base> bases = sl_this.Base_kind_children();
	for(set<SL::Base>::iterator bi=bases.begin(); bi!=bases.end(); bi++)
	{
		SL::GraphicalObj g = SL::GraphicalObj::Cast(*bi);
		unsigned int gc = (int)g.column();
		unsigned int gr = (int)g.row();
		if (gc >= grid.size())
			grid.resize(gc + 10);
		vector<SL::GraphicalObj>& grid_col = grid[gc];
		if (gr >= grid_col.size())
			grid_col.resize(gr + 10);
		grid_col[gr] = g;
	}

	int x=DEF_OFFSET;
	for(unsigned int i=0; i<grid.size(); i++)
	{
		int y=DEF_OFFSET;
		int max_w=0;
		vector<SL::GraphicalObj>& grid_col = grid[i];
		for(unsigned int j=0; j<grid_col.size(); j++)
		{
			SL::GraphicalObj& g = grid_col[j];
			if (g == Udm::null) break;
			int xsize,ysize;
			if (Udm::IsDerivedFrom(g.type(), SL::Port::meta))
				xsize = ysize = DEF_PORT_SIZE;
			else
			{
				SL::Base gb = SL::Base::Cast(g);
				int nin = ((set<SL::InputPort>)gb.InputPort_kind_children()).size();
				int nout = ((set<SL::OutputPort>)gb.OutputPort_kind_children()).size();
				if (nin<nout) nin=nout;
				if (nin==0) nin=1;
				xsize = (Udm::IsDerivedFrom(g.type(), SL::Bus::meta)) ? DEF_BUS_XSIZE : DEF_XSIZE;
				ysize = (Udm::IsDerivedFrom(g.type(), SL::Bus::meta)) ? DEF_BUS_YSIZE*nin : DEF_YSIZE*nin;
			}
			sprintf(name_buf,"[%d %d %d %d]", x, y, x+xsize, y+ysize);
			g.position() = string(name_buf);

			if (xsize>max_w) max_w = xsize;
			y += ysize + DEF_OFFSET; 
		}
		x += max_w + 3*DEF_OFFSET;
	}


	// recurse
	{
		set<SL::Compound> comps = sl_this.Compound_kind_children();
		for(set<SL::Compound>::iterator i=comps.begin(); i!=comps.end(); i++)
			Generate5(SL::Compound::Cast(*i));
	}
}