#include "InteractionGenerator.h"

InteractionGenerator::InteractionGenerator(AIF::System aifSystem)
{
	aifInteraction = AIF::Interactions::Create(aifSystem);
	CreateAIFObject(aifInteraction, "");
}

void InteractionGenerator::CreateAIFObject(AIF::AIFObject aif_obj, string objName)
{
	aif_obj.name()=objName;

    char buffer [33];
	itoa (aif_obj.uniqueId(),buffer,10);
	aif_obj.guid() = buffer;

	aif_obj.creator()=""; aif_obj.creationDate()="";
	aif_obj.lastModifier()="";aif_obj.lastModDate()=""; 
	aif_obj.locked()=false; aif_obj.locker()="";
}

AIF::AIFObject InteractionGenerator::GetAIFObject(Udm::Object esmlObj, ESML::Processor p1)
{
	AIF::AIFObject aifObj = NULL;
	map<Udm::Object, ESML2AIF>::iterator pos = esmlObjs.find(esmlObj);

	ASSERT(pos!=esmlObjs.end());

	ESML2AIF tempMap = pos->second;
	for(ESML2AIF::iterator pos1=tempMap.begin();pos1!=tempMap.end();pos1++)
	{
		Udm::Object obj = pos1->first;
		set<Udm::Object>::iterator spos = esmlComRefs.find(obj);
		if(spos==esmlComRefs.end()) continue;
		if(GetProcessor(pos1->first)==p1) return pos1->second;
	}
	
	if(aifObj == Udm::null)
	{
		for(ESML2AIF::iterator i=tempMap.begin();i!=tempMap.end();i++)
		{
			Udm::Object comRef = i->first;
			ESML::ComponentType comType;
			if(!GetESMLComponent(comRef,comType)) aifObj = i->second;
		}
	}
	return aifObj;
}

ESML::Processor InteractionGenerator::GetProcessor(Udm::Object esmlComRef)
{
	ESML::Process proc;
	if((string)esmlComRef.type().name()=="NormalComponent")
	{
		ESML::NormalComponent nref = ESML::NormalComponent::Cast(esmlComRef);
		proc = nref.Process_parent();
	}
	else
	{
		ESML::BackupComponent bref = ESML::BackupComponent::Cast(esmlComRef);
		proc = bref.Process_parent();
	}  	
	return proc.Processor_parent();
}

AIF::AIFObject InteractionGenerator::GetAIFObject(Udm::Object esmlObj, Udm::Object esmlComRef)
{
	AIF::AIFObject aifObj = NULL;
	map<Udm::Object, ESML2AIF>::iterator pos = esmlObjs.find(esmlObj);

	ASSERT(pos!=esmlObjs.end());

	ESML2AIF tempMap = (*pos).second;
	ESML2AIF::iterator pos1 = tempMap.find(esmlComRef);
	if(pos1!=tempMap.end())	aifObj = (*pos1).second;
	else
	{
		for(ESML2AIF::iterator i=tempMap.begin();i!=tempMap.end();i++)
		{
			Udm::Object comRef = i->first;
			ESML::ComponentType comType;
			if(!GetESMLComponent(comRef,comType)) aifObj = i->second;
		}
	}	
	return aifObj;
}

void InteractionGenerator::GenerateInteractionDiagram()
{
	for(set<Udm::Object>::iterator i1=esmlComRefs.begin();i1!=esmlComRefs.end();i1++)
	{
		Udm::Object esmlComRef = *i1;
		ESML::ComponentType comType;
		GetESMLComponent(esmlComRef, comType);
		set<ESML::PublishPort> pubPorts = comType.PublishPort_kind_children();
		for(set<ESML::PublishPort>::iterator pi=pubPorts.begin();pi!=pubPorts.end();pi++)
		{
			PublishEventPass(*pi, esmlComRef);	
		}
	}

	for(set<Udm::Object>::iterator i2=esmlComRefs.begin();i2!=esmlComRefs.end();i2++)
	{
		Udm::Object esmlComRef = *i2;
		ESML::ComponentType comType;
		if(GetESMLComponent(esmlComRef, comType)) continue;

		set<ESML::SubscribePort> subPorts = comType.SubscribePort_kind_children();
		for(set<ESML::SubscribePort>::iterator si=subPorts.begin();si!=subPorts.end();si++)
		{
			SubscribePass(*si,esmlComRef);	
		}

		set<ESML::Receptacle> receps = comType.Receptacle_kind_children();
		for(set<ESML::Receptacle>::iterator ri=receps.begin();ri!=receps.end();ri++)
		{
			InvocationPass(*ri, esmlComRef);
		}
	}
}

bool InteractionGenerator::GetESMLComponent(Udm::Object esmlComRef, ESML::ComponentType& comType)
{
	if((string)esmlComRef.type().name()=="NormalComponent")
		comType = (ESML::NormalComponent::Cast(esmlComRef)).ref();
	else               //BackupComponent
		comType = (ESML::BackupComponent::Cast(esmlComRef)).ref();
	return false;
}

void InteractionGenerator::PublishEventPass(ESML::PublishPort esmlPort, Udm::Object comRef)
{
	set<ESML::Publish> publishes = esmlPort.dst();

	if(publishes.empty()) return;

	for(set<ESML::Publish>::iterator i=publishes.begin();i!=publishes.end();i++)
	{
		ESML::Publish publish = *i;
		ESML::EventBase eventBase = publish.dst_end();
		ESML::EventType esmlEvent = ESML::EventType::Cast(eventBase);

		pubEvent[esmlPort] = esmlEvent;	
		
		AIF::Event aifEvent = AIF::Event::Create(aifInteraction);
		CreateAIFObject(aifEvent, (string)esmlEvent.name());
		AIF::PublishPort aifPort = AIF::PublishPort::Cast(GetAIFObject(esmlPort, comRef));
		aifEvent.publish() = aifPort;
		
		InsertESMLObjEntry(esmlEvent,aifEvent,comRef);
	}
}

void InteractionGenerator::SubscribePass(ESML::SubscribePort esmlSubPort, Udm::Object comRef)
{
	set<ESML::Subscribe> subscribes = esmlSubPort.src();
	if(subscribes.empty()) return;

	AIF::SubscribePort aifSubPort = AIF::SubscribePort::Cast(GetAIFObject(esmlSubPort,comRef));
	for(set<ESML::Subscribe>::iterator sj=subscribes.begin();sj!=subscribes.end();sj++)
	{
		ESML::EventBase eventBase = (*sj).src_end();
		ESML::EventType esmlEvent;
		if((string)eventBase.type().name()=="EventType")
			esmlEvent = ESML::EventType::Cast(eventBase);
		else //EventChRef
		{
			ESML::EventChRef chRef = ESML::EventChRef::Cast(eventBase);
			esmlEvent = chRef.ref();
		}

		set<ESML::Publish> publishes = esmlEvent.src();
		if(publishes.size()!=1)
		{
			cerr<<"Error: "<<(string)esmlEvent.name()<<" must have one and only one Publisher."<<endl;
			exit(-1);
		}

		ESML::Publish publish = *publishes.begin();
		ESML::PublishBase pubBase = publish.src_end();
		if((string)pubBase.type().name()=="Timer")
		{
			ESML::Timer timer = ESML::Timer::Cast(pubBase);
			map<ESML::Timer,AIF::TimerComponent>::iterator pos=timerMap.find(timer);
			if(pos==timerMap.end())
			{
				AIF::TimerComponent aifTimer = AIF::TimerComponent::Create(aifInteraction);
				CreateAIFObject(aifTimer, (string)timer.name());

				string timerPeriod = timer.Period();
				int tp = atoi(timerPeriod.c_str());
			    char buffer [33];
				itoa (tp,buffer,10);
				aifTimer.period() = buffer;

				AIF::Event aifEvent = AIF::Event::Create(aifInteraction);
				CreateAIFObject(aifEvent, (string)esmlEvent.name());
				aifEvent.timer() = aifTimer;
				aifSubPort.src() += aifEvent;
				timerMap[timer] = aifTimer;
			}
			else
			{
				AIF::TimerComponent aifTimer = pos->second;
				set<AIF::Event> aifEvents = aifTimer.event();
				AIF::Event aifEvent = *aifEvents.begin();
				aifSubPort.src() += aifEvent;
			}
		}
		else
		{
// Consider the proxy component scenario
			AIF::Event aifEvent;
			ESML::PublishPort pubport = ESML::PublishPort::Cast(pubBase);
			ESML::ComponentType pubCom = pubport.ComponentType_parent();
			if(!IsEnable(pubCom))
			{
				map<ESML::ComponentType, ESML::ComponentType>::iterator mpos = proxy_master.find(pubCom);
				if(mpos==proxy_master.end()) continue;
				
				ESML::ComponentType mcom = mpos->second;
				if(!IsEnable(mcom)) continue;
				
				set<ESML::PublishPort> pports = mcom.PublishPort_kind_children();
				for(set<ESML::PublishPort>::iterator pi=pports.begin();pi!=pports.end();pi++)
				{
					ESML::PublishPort pport = *pi;
					if((ESML::PublishPort)pport.Archetype() == (ESML::PublishPort)pubport.Archetype())
					{
						ESML::EventType mevent = pubEvent[pport];
						AIF::AIFObject aifObj = GetAIFObject(mevent,GetProcessor(comRef));
						if(aifObj==Udm::null) continue;
						aifEvent = AIF::Event::Cast(aifObj);			
						break;
					}
				}
			}
			else
			{
				AIF::AIFObject aifObj = GetAIFObject(esmlEvent,GetProcessor(comRef));
				if(aifObj==Udm::null) continue;
				aifEvent = AIF::Event::Cast(aifObj);
			}
			if(aifEvent!=Udm::null) aifSubPort.src() += aifEvent;
		}
	}
}

bool InteractionGenerator::IsEnable(ESML::ComponentType esmlCom)
{
	const set<ESML::NormalComponent> &comns = esmlCom.normalComponent_ref();
	const ESML::NormalComponent &comn = *(comns.begin());
	set<Udm::Object>::iterator pos = esmlComRefs.find(comn);
	if(pos!=esmlComRefs.end()) return true;
	
	set<ESML::BackupComponent> combs = esmlCom.backupComponent_ref();
	for(set<ESML::BackupComponent>::iterator i=combs.begin();i!=combs.end();i++)
	{
		ESML::BackupComponent comb = *i;
		pos = esmlComRefs.find(comb);
		if(pos!=esmlComRefs.end()) return true;
	}

	return false;
}

void InteractionGenerator::InvocationPass(ESML::Receptacle esmlRecep, Udm::Object comRef)
{
	set<ESML::Invoke> invokes = esmlRecep.dst();
	if(invokes.empty()) return;

	AIF::Receptacle aifRecep = AIF::Receptacle::Cast(GetAIFObject(esmlRecep, comRef));

	for(set<ESML::Invoke>::iterator ii=invokes.begin();ii!=invokes.end();ii++)
	{
		ESML::Invoke esmlInvoke = *ii;
		ESML::Facet esmlFacet = esmlInvoke.dst_end();
		AIF::Facet aifFacet;

		ESML::ComponentType fcom = esmlFacet.ComponentType_parent();
		if(!IsEnable(fcom))
		{
			map<ESML::ComponentType, ESML::ComponentType>::iterator mpos = proxy_master.find(fcom);
			if(mpos==proxy_master.end()) continue;
			
			ESML::ComponentType mcom = mpos->second;
			if(!IsEnable(mcom)) continue;

			set<ESML::Facet> mfacets = mcom.Facet_kind_children();
			for(set<ESML::Facet>::iterator fi=mfacets.begin();fi!=mfacets.end();fi++)
			{
				ESML::Facet mfacet = *fi;
				if((ESML::Facet)mfacet.Archetype()==(ESML::Facet)esmlFacet.Archetype())
				{
					AIF::AIFObject aifObj = GetAIFObject(mfacet,GetProcessor(comRef));
					if(aifObj == Udm::null) continue;
					aifFacet = AIF::Facet::Cast(aifObj);		
					break;
				}
			}
		}
		else
		{
			AIF::AIFObject aifObj = GetAIFObject(esmlFacet,GetProcessor(comRef));
			if(aifObj == Udm::null) continue;
			aifFacet = AIF::Facet::Cast(aifObj);
		}

		if(aifFacet==Udm::null) continue;

		AIF::InvocationConnection aifInvoke = AIF::InvocationConnection::Create(aifInteraction);
		CreateAIFObject(aifInvoke, (string)esmlInvoke.name());
		aifInvoke.asynchronous() = esmlInvoke.Asynchronous();
		aifInvoke.invokes_end() = aifFacet;
		aifInvoke.callout_end() = aifRecep;

		AIF::ComponentInstance aifRCom = aifRecep.ComponentInstance_parent();
		AIF::Processor recepProcessor = aifRCom.Processor_parent();
		AIF::ComponentInstance aifFCom = aifFacet.ComponentInstance_parent();
		AIF::Processor facetProcessor = aifFCom.Processor_parent();
		if(recepProcessor==facetProcessor) continue;
		AIF::Link aifLink = GetAIFLink(recepProcessor, facetProcessor);
		if(aifLink!=Udm::null) aifInvoke.implements() = aifLink;
	}
}

AIF::Link InteractionGenerator::GetAIFLink(AIF::Processor p1, AIF::Processor p2)
{
	AIF::Link aifLink = NULL;
	set<AIF::CommPort> commPorts = p1.CommPort_kind_children();
	for(set<AIF::CommPort>::iterator i=commPorts.begin();i!=commPorts.end();i++)
	{
		AIF::CommPort commPort = *i;
		set<AIF::Link> slinks = commPort.src();
		for(set<AIF::Link>::iterator si=slinks.begin();si!=slinks.end();si++)
		{
			aifLink = *si;
			AIF::CommPort sPort = aifLink.src_end();
			if(p2==sPort.Processor_parent()) return aifLink;
		}

		set<AIF::Link> dlinks = commPort.dst();
		for(set<AIF::Link>::iterator di=dlinks.begin();di!=dlinks.end();di++)
		{
			aifLink = *di;
			AIF::CommPort dPort = aifLink.dst_end();
			if(p2==dPort.Processor_parent()) return aifLink;
		}
	}
	return aifLink;
}

void InteractionGenerator::InsertESMLObjEntry(Udm::Object esmlObj, AIF::AIFObject aifObj, Udm::Object esmlComRef)
{
	ESML2AIF comRefEntry;
	map<Udm::Object, ESML2AIF >::iterator pos = esmlObjs.find(esmlObj);
	if(pos!=esmlObjs.end())
	{
		comRefEntry = (*pos).second;
	}

	comRefEntry[esmlComRef] = aifObj;
	esmlObjs[esmlObj] = comRefEntry;
}
