#ifdef _WIN32
#include <UdmGme.h>			// these must be included to make SmartDataNetwork work
#endif
#include <UdmDom.h>
#include <Uml.h>
#include <UdmStatic.h>
#include <UmlExt.h>
#include <UdmBase.h>
#include <UdmInt.h>
#include <UdmCint.h>


#include "io.h"
#include "stdio.h"
#include <stdlib.h>


#if 1 // tbd baa 
#include <time.h>
//#include <fstream.h>


#include "config.h"
#include "aif_support.h"
#include "iif_support.h"


using namespace AIF;


//****************************************************************************************
// Gloabally accessed variables.
//****************************************************************************************

AIF::System aif_system;
Udm::SmartDataNetwork* aif; 

//****************************************************************************************
// Local function prototypes.
//****************************************************************************************
void check_for_null_wcets(void);
char *cur_date_and_time(void);


//****************************************************************************************
//                           Local function definitions.
//****************************************************************************************


//****************************************************************************************
// Function checks the AIF file for WCETs that do not have values assigned.
//****************************************************************************************
void aif_check_for_null_wcets(void)
{
	set<Network> znetworks = aif_system.Network_kind_children();
	set<Network>::iterator znetworks_i = znetworks.begin();
	for(int nxx=0;znetworks_i != znetworks.end();nxx++)
	{
		AIF::Network this_network;
		this_network = *znetworks_i;

		set<Processor> zprocessors = this_network.Processor_kind_children();
		set<Processor>::iterator zprocessors_i = zprocessors.begin();
		for(int xx=0;zprocessors_i != zprocessors.end();xx++)
		{
			Processor zprocessor = *zprocessors_i;

			set <ComponentInstance> zcompinsts = zprocessor.ComponentInstance_children();
			set <ComponentInstance>::iterator zcompinsts_i = zcompinsts.begin();
			for(int xx=0; zcompinsts_i != zcompinsts.end();xx++)
			{
				ComponentInstance zcompinst = *zcompinsts_i;
				string strCompInstName = zcompinst.name();
				bool has_trigs = false;
				bool all_trigs_null = true;

				set <TriggerQoSProperties> trigqosprops = zcompinst.TriggerQoSProperties_kind_children();
				set <TriggerQoSProperties>::iterator trigqosprops_i = trigqosprops.begin();
				for(int xx=0; trigqosprops_i != trigqosprops.end();xx++)
				{
					TriggerQoSProperties trigqosprop = *trigqosprops_i;
					Action action = trigqosprop.action_end();
					string action_name = action.name();
					SubscribePort subscribeport = trigqosprop.trigger_end();
					string strSubscribePortName = subscribeport.name();

                    has_trigs = true;

					// If the object is not locked.
					if (!action.locked()) {

						// If there is no WCET assigned.
						string strActionWCET = action.WCET();
						if (0 == strlen(strActionWCET.c_str())) {

							// Indicate that no WCET is assigned.
#ifndef __INTEGRATED_INTO_OTIF__
							outStream << "Warning: Subscribe port " << strSubscribePortName.c_str()
								<< " for component instance " << strCompInstName.c_str()
								<< " action " << "<" <<action_name << ">"
								<< " has WCET with no value." << endl;
#endif
							bWarningDetected = TRUE;
						}
						else
							all_trigs_null = false;
					}

					// If the object is not locked.
					if (!trigqosprop.locked()) {

						// If there is no WCET assigned.
						string strTrigqospropWCET = trigqosprop.aggregateWCET();
						if (0 == strlen(strTrigqospropWCET.c_str())) {

							// Indicate that no WCET is assigned.
#ifndef __INTEGRATED_INTO_OTIF__
							outStream << "Warning: Subscribe port " << strSubscribePortName.c_str()
								<< " for component instance " << strCompInstName.c_str()
								<< " action " << "<" <<action_name << ">"
								<< " has aggregate WCET with no value." << endl;
#endif
							bWarningDetected = TRUE;
						}
						else
							all_trigs_null = false;
					} 
					trigqosprops_i++;
				}
                 
				if(has_trigs && all_trigs_null)
				{
#ifndef __INTEGRATED_INTO_OTIF__
					outStream << "Warning: Component Instance " << strCompInstName.c_str()
						<< " has no execution values." << endl;
#endif
					bWarningDetected = TRUE;
				}

				zcompinsts_i++;
			}
			zprocessors_i++;
		}
		znetworks_i++;
	}
	return;
}

//****************************************************************************************
// Function returns current date/time as an ASCII string.
//****************************************************************************************
char *cur_date_and_time(void)
{
	static char strCurDateAndTime[128];
	struct tm *newtime;
	time_t aclock;

	// Get time in seconds.
   time(&aclock);                 

	// Convert seconds to time struct format.
	newtime = localtime(&aclock);

	// Convert time to ASCII string.
	strcpy(strCurDateAndTime, asctime(newtime));

	// Remove the trailing CR/LF.
	strCurDateAndTime[strlen(strCurDateAndTime)-1] = 0;

	// Return a pointer to the ASCII string.
	return(strCurDateAndTime);
}


//****************************************************************************************
//                            Global function definitions.
//****************************************************************************************

//****************************************************************************************
// Function updates and closes the AIF data object.
//****************************************************************************************
void aif_close(void)
{
	aif->CloseWithUpdate();
}

//****************************************************************************************
// Function initializes the AIF data object.
//****************************************************************************************
void aif_init(char* aif_file)
{
	aif = new Udm::SmartDataNetwork(AIF::diagram);

	try 
	{	
		if ((_access(aif_file, 0)) != -1) {

			aif->OpenExisting(aif_file,"", Udm::CHANGES_PERSIST_ALWAYS);
			aif_system = AIF::System::Cast(aif->GetRootObject());

			vector<AIFObject> aifobjects = aif_system.AIFObject_kind_children();
			vector<AIFObject>::iterator aifobjects_i = aifobjects.begin();
			int xx = 0;

			outStream << endl;
			while (aifobjects_i != aifobjects.end())
			{
				string aifobject_name = aifobjects_i->name();
				AIF::AIFObject* obj = aifobjects_i;
				outStream << "AIF object ";
				outStream << xx;
				outStream << " is ";
				outStream << aifobject_name;
				outStream<< endl;
				xx++;
				aifobjects_i++;
			}
		} else {

			outStream << "Error: AIF file " << aif_file << " not found." << endl;
			cerr << "Error: AIF file " << aif_file << " not found." << endl;
			_exit(__LINE__);
		}
		_sleep(3000);
	}

	catch(const udm_exception &e)
	{
		outStream << "Exception: initializing aif data: " << e.what() << endl;
		_sleep(3000);
		exit(__LINE__);

	} 

	return;
}


//****************************************************************************************
// Function helps sets the WCET for the specified component subscribe port data struct.
//****************************************************************************************

string strCurWcet;
string strCurAggregateWcet;
unsigned __int64 llCurWcet;
unsigned __int64 llCurAggregateWcet;
char strNewWcet[128], strNewAggregateWcet[128];
string strCompInstName;

void aif_update_wcet_helper(Action &action, SUBSCRIBE_PORT_LIST_ITEM &subscribe_port_item, TriggerQoSProperties &trigqosprop, int zx)
{
	// If the object is not locked.
	if (!action.locked()) 
		{

			// If the new WCET is greater than the current WCET.
			if (subscribe_port_item.llWCExecutionTime[zx] > llCurWcet) 
				{  
					// Update the WCET in the AIF.
					action.WCET()         = strNewWcet;
					action.lastModDate()  = cur_date_and_time();
					action.lastModifier() = "IIF2AIF";
				} 
			  else 
				{
					if(llCurWcet)
					{
						// Indicate that no change was made.
#ifndef __INTEGRATED_INTO_OTIF__
						outStream << "Warning: " << strCompInstName.c_str() << " Current WCET = "
							<< strCurWcet << " is greater than (or equal) the new WCET = " << strNewWcet
							<< ". The WCET was not updated." << endl;
#endif
						bWarningDetected = TRUE;
					}
				}
		} 
	  else 
		{
			// Indicate that no change was made.
#ifndef __INTEGRATED_INTO_OTIF__
			outStream << "Warning: " << strCompInstName.c_str() << " Action is locked."	<< " No WCET update was done." << endl;
#endif
			bWarningDetected = TRUE;
		}


	// If the object is not locked.
	if (!trigqosprop.locked()) 
		{
			// If the new WC wall clock time is greater than current WC wall clock time.
			if (subscribe_port_item.llWCWallClockTime[zx] > llCurAggregateWcet) 
				{ 
					// Update the wall clock time in the AIF.
					trigqosprop.aggregateWCET() = strNewAggregateWcet;
					trigqosprop.lastModDate() = cur_date_and_time();
					trigqosprop.lastModifier() = "IIF2AIF";
				} 
			  else 
				{
					if(llCurAggregateWcet)
					{
						// Indicate that no change was made.
#ifndef __INTEGRATED_INTO_OTIF__
						outStream << "Warning: " << strCompInstName.c_str() << " Current WC wall clock time = "
							<< strCurAggregateWcet << " is greater than (or equal) the new WC wall clock time = "
							<< strNewAggregateWcet << ". The WC wall clock time was not updated." << endl;
#endif
						bWarningDetected = TRUE;
					}
				}
		} 
  	  else 
		{
			// Indicate that no change was made.
#ifndef __INTEGRATED_INTO_OTIF__
			outStream << "Warning: " << strCompInstName.c_str() << " TriggerQoSProperties is locked."<< " No WCET aggregrate update was done." << endl;
#endif
			bWarningDetected = TRUE;
		}
}

//****************************************************************************************
// Function sets the WCET for the specified component subscribe port data struct.
//****************************************************************************************
void aif_set_wcet(SUBSCRIBE_PORT_LIST_ITEM &subscribe_port_item)
{
	BOOL bComponentInstanceFound = FALSE;
	BOOL bSubscribePortFound = FALSE;

	string spn = subscribe_port_item.sp.name();
    if(spn == "ROUTES")
	{
		_sleep(1);
	}

	set<Network> znetworks = aif_system.Network_kind_children();
	set<Network>::iterator znetworks_i = znetworks.begin();
	for(int nxx=0;znetworks_i != znetworks.end();nxx++)
	{
		AIF::Network this_network;

		this_network = *znetworks_i;

		set<Processor> zprocessors = this_network.Processor_kind_children();
		set<Processor>::iterator zprocessors_i = zprocessors.begin();
		for(int xx=0;zprocessors_i != zprocessors.end();xx++)
		{
			Processor zprocessor = *zprocessors_i;

			set <ComponentInstance> zcompinsts = zprocessor.ComponentInstance_children();
			set <ComponentInstance>::iterator zcompinsts_i = zcompinsts.begin();
			for(int xx=0; zcompinsts_i != zcompinsts.end();xx++)
			{
				ComponentInstance zcompinst = *zcompinsts_i;
				strCompInstName = zcompinst.name();

				// If this is the component instance we are looking for.
				if ((zcompinst.groupID() == subscribe_port_item.group) && (zcompinst.itemID() == subscribe_port_item.item)) 
				{

					bComponentInstanceFound = TRUE;

					set <AIF::SubscribePort> subscribepprops = zcompinst.SubscribePort_kind_children();
					set <AIF::SubscribePort>::iterator subscribepprops_i = subscribepprops.begin();
					for(int sxx=0; subscribepprops_i != subscribepprops.end();sxx++)
					{
						AIF::SubscribePort sp = *subscribepprops_i;
						string spn = sp.name();
						string spin = subscribe_port_item.sp.name();
						if(spn == spin)
						{
							//set <Action> as = sp.action(); tbd baa for medium debug
							_sleep(0);
							
							//TriggerQoSProperties tqosp = sp.trigger_end();
                        }
						subscribepprops_i++;
					}

					set <TriggerQoSProperties> trigqosprops = zcompinst.TriggerQoSProperties_kind_children();
					set <TriggerQoSProperties>::iterator trigqosprops_i = trigqosprops.begin();
					for(int xx=0; trigqosprops_i != trigqosprops.end();xx++)
					{
						TriggerQoSProperties trigqosprop = *trigqosprops_i;
						SubscribePort subscribeport = trigqosprop.trigger_end();
						string strSubscribePortName = subscribeport.name();

						// If this is the subscribe port we are looking for.
						string strSubscribePortItemName = subscribe_port_item.sp.name();
						if (strSubscribePortName == strSubscribePortItemName) {

							bSubscribePortFound = TRUE;

							// Read the current worst case execution times from the AIF.
							Action action = trigqosprop.action_end();

							// tbd baa -- for modes fix set<ComponentMode> modes = action.modes();
							set<ComponentMode> modes = zcompinst.ComponentMode_children();
							set<ComponentMode>::iterator modes_i;
							ComponentMode this_mode;
							string        sthis_mode;


#ifndef __INTEGRATED_INTO_OTIF__
							cout << "Modes for " << strCompInstName;
							for(modes_i = modes.begin();modes_i!=modes.end();modes_i++)
							{
								this_mode = *modes_i;
								sthis_mode = this_mode.name();
								const char *s = sthis_mode.c_str();

								cout << " " << sthis_mode;

							}
							cout << endl;
#endif


							strCurWcet = action.WCET();
							strCurAggregateWcet = trigqosprop.aggregateWCET();
							llCurWcet = _atoi64(strCurWcet.c_str());
							llCurAggregateWcet = _atoi64(strCurAggregateWcet.c_str());


                            {
							 string check_mode;
							 int zx;
							 int msz;
							 int iif_cnt;

							    iif_cnt = 0;
								for(zx=0;zx<MAX_MODES;zx++)
								{
							        string check_mode = iif_get_component_mode_name( subscribe_port_item.group, subscribe_port_item.item, zx);
									if(check_mode=="default") continue;
									if(check_mode=="") break;
									iif_cnt++;
								}
								msz = modes.size();
								if((msz < iif_cnt)  && (iif_cnt > 0))
								{
								  outStream << "Error: Subscribe Port " 
									                 << " " << strCompInstName 
  									                    << "{" << subscribe_port_item.group << "," <<    subscribe_port_item.item << "}."
													    <<  strSubscribePortItemName
									                 << " mode count " << iif_cnt << " in IIF exceeds " << msz << " in AIF" << endl;
								  cerr << "Error: Subscribe Port " 
									                 << " " << strCompInstName 
  									                    << "{" << subscribe_port_item.group << "," <<    subscribe_port_item.item << "}."
													    <<  strSubscribePortItemName
									                 << " mode count " << iif_cnt << " in IIF exceeds " << msz << " in AIF" << endl;
								  if(!bIgnoreFatalErrors)
								  {
									_sleep(3000);
									_exit(__LINE__);
								  }
								}
							}

							for(int zx=0;zx<MAX_MODES;zx++)
							{		
								sprintf(strNewWcet, "%I64d", subscribe_port_item.llWCExecutionTime[zx]);  // tbd baa -- must change for mode support
								sprintf(strNewAggregateWcet, "%I64d", subscribe_port_item.llWCWallClockTime[zx]);  // tbd baa -- must change for mode support

								string check_mode = iif_get_component_mode_name( subscribe_port_item.group, subscribe_port_item.item, zx);
								const char *cm = check_mode.c_str();
								const char *sp = strSubscribePortName.c_str();

								if(check_mode=="") break;
								int msz = modes.size();


								//**** first deal with components with NO modes *******
								if(msz == 0)
								{
									aif_update_wcet_helper(action, subscribe_port_item, trigqosprop, zx);
									break;
								}

								//**** now deal with components with MODES *******
								for(modes_i = modes.begin();modes_i!=modes.end();modes_i++)
								{
									this_mode = *modes_i;
									sthis_mode = this_mode.name();
									const char *s = sthis_mode.c_str();

									_sleep(1);


									if(
										((this_mode.initialMode()) && (check_mode=="default"))
										  ||
										(sthis_mode == check_mode)

									  )
									{

                                        // tbd baa -- for fix of mode stuff -- find the matching trigqosprop and action
							            set<Action> these_actions = this_mode.actions();
							            set<Action>::iterator these_actions_i;

										for(these_actions_i = these_actions.begin();these_actions_i!=these_actions.end();these_actions_i++)
										{
											 Action an_action = *these_actions_i;
											 set<TriggerQoSProperties> triggerqos_s = zcompinst.TriggerQoSProperties_kind_children();
											 set<TriggerQoSProperties>::iterator triggerqos_s_i;

											 for(triggerqos_s_i = triggerqos_s.begin();triggerqos_s_i!=triggerqos_s.end();triggerqos_s_i++)
											 {
											   TriggerQoSProperties a_trigqosprop = *triggerqos_s_i;
											   Action ta = a_trigqosprop.action_end();

											   if(ta == an_action)
											   {
												  aif_update_wcet_helper(an_action, subscribe_port_item, a_trigqosprop, zx);
											   }
											 }
										}
									}
								}
							}

							
						}
						trigqosprops_i++;
					}
				}
				zcompinsts_i++;
			}
			zprocessors_i++;
		}
		znetworks_i++;
	}


	// Check for component instance not found in AIF file.
	if (!bComponentInstanceFound) 
	{
	  outStream << "Error: Component instance group/item " << subscribe_port_item.group << "/"
		<< subscribe_port_item.item << " not found in AIF file." << endl;
	  cerr << "Error: Component instance group/item " << subscribe_port_item.group << "/"
		<< subscribe_port_item.item << " not found in AIF file." << endl;
	  if(!bIgnoreFatalErrors)
	  {
		_sleep(3000);
		_exit(__LINE__);
	  }
	}

	// Check for subscribe port not found in AIF file.
	if (bComponentInstanceFound && ! bSubscribePortFound) 
	{
		string strSubscribePortName = subscribe_port_item.sp.name();
		string strComponentName = subscribe_port_item.cp.name();
        bool got_wcet;


		got_wcet = FALSE;
		for(int zx=0;zx<MAX_MODES;zx++)
		{		
			if(subscribe_port_item.llWCExecutionTime[zx]) got_wcet = TRUE;
		}

		if(!got_wcet)
			{
#ifndef __INTEGRATED_INTO_OTIF__
				outStream << "Warning: Subscribe port defined, but not used " << strSubscribePortName.c_str()
					<< " for component " << strComponentName.c_str() << " not found in AIF file." << endl;
				cerr << "Warning: Subscribe port defined, but not used " << strSubscribePortName.c_str()
					<< " for component " << strComponentName.c_str() << " not found in AIF file." << endl;
#endif
			}
		   else
			{
				outStream << "Error: Subscribe port " << strSubscribePortName.c_str()
					<< " for component " << strComponentName.c_str() << " not found in AIF file." << endl;
				cerr << "Error: Subscribe port " << strSubscribePortName.c_str()
					<< " for component " << strComponentName.c_str() << " not found in AIF file." << endl;
 				if(!bIgnoreFatalErrors)
				{
				 _sleep(3000);
				 _exit(__LINE__);
				}
		   }
	}

	return;
}




//****************************************************************************************
// Function sets the WCET for the specified invocation
//****************************************************************************************
void aif_set_invocation_wcet(ACTION_LIST_ITEM &action_item, string mode_name, int mode_index)  // tbd baa -- use mode name
{
	BOOL bComponentInstanceFound = FALSE;
	BOOL bActionFound = FALSE;
	string ActionItemName = action_item.a.name();


	if(ActionItemName == "ChangeMode")
	{
		_sleep(1);  // tbd baa for debug
	}

	set<Network> znetworks = aif_system.Network_kind_children();
	set<Network>::iterator znetworks_i = znetworks.begin();
	for(int nxx=0;znetworks_i != znetworks.end();nxx++)
	{
		AIF::Network this_network;

		this_network = *znetworks_i;

		set<Processor> zprocessors = this_network.Processor_kind_children();
		set<Processor>::iterator zprocessors_i = zprocessors.begin();
		for(int xx=0;zprocessors_i != zprocessors.end();xx++)
		{
			Processor zprocessor = *zprocessors_i;

			set <ComponentInstance> zcompinsts = zprocessor.ComponentInstance_children();
			set <ComponentInstance>::iterator zcompinsts_i = zcompinsts.begin();
			for(int xx=0; zcompinsts_i != zcompinsts.end();xx++)
			{
				ComponentInstance zcompinst = *zcompinsts_i;
				strCompInstName = zcompinst.name();

				// If this is the component instance we are looking for.
				if ((zcompinst.groupID() == action_item.group) && (zcompinst.itemID() == action_item.item)) 
				{

					bComponentInstanceFound = TRUE;

					set <InvocationQoSProperties> invocqosprops = zcompinst.InvocationQoSProperties_kind_children();
					set <InvocationQoSProperties>::iterator invocqosprops_i = invocqosprops.begin();
					for(int xx=0; invocqosprops_i != invocqosprops.end();xx++)
					{
						InvocationQoSProperties invocqosprop = *invocqosprops_i;
//tbd baa for AIF 1.5						Action action = invocqosprop.actions_end();  
// tbd baa but must check for no connection!!						if(invocqosprop.action() == 0) continue;
						Action action = invocqosprop.action();  
						string this_name = action.name();

						// If this is the subscribe port we are looking for.
						string name_of_interest = action_item.a.name();

						if (this_name == name_of_interest) 
						{

							bActionFound = TRUE;

	                        string this_wcet_string = invocqosprop.aggregateWCET();
							unsigned __int64 this_wcet = _atoi64(this_wcet_string.c_str());

							if (!invocqosprop.locked()) 
								{
									  if(action_item.wcet[mode_index] > this_wcet)  // tbd baa -- deal with modes here
										{ 
											invocqosprop.aggregateWCET() = __i2s(action_item.wcet[mode_index]); // tbd baa -- deal with modes here
											invocqosprop.lastModDate() = cur_date_and_time();
											invocqosprop.lastModifier() = "IIF2AIF";
										} 
									  else 
										{
											if(action_item.wcet[mode_index] != 0)
											{
												// Indicate that no change was made.
#ifndef __INTEGRATED_INTO_OTIF__
												outStream << "Warning: " << strCompInstName.c_str() << " Current Invocation WCET = "
													<< __i2s(this_wcet) << " is greater than (or equal) the new Invocation time = "
													<< __i2s(action_item.wcet[mode_index]) << ". The time was not updated." << endl;  // tbd baa -- deal with modes here
#endif
												bWarningDetected = TRUE;
											}
										}
								} 
  							  else 
								{
									// Indicate that no change was made.
#ifndef __INTEGRATED_INTO_OTIF__
									outStream << "Warning: " << strCompInstName.c_str() << " invocgerQoSProperties is locked."<< " No WCET invocation update was done." << endl;
#endif
									bWarningDetected = TRUE;
								}
						}
						invocqosprops_i++;
					}
				}
				zcompinsts_i++;
			}
			zprocessors_i++;
		}
		znetworks_i++;
	}


	// Check for component instance not found in AIF file.
	if (!bComponentInstanceFound) 
	{
	  outStream << "Error: Component instance group/item " << action_item.group << "/"
		<< action_item.item << " not found in AIF file." << endl;
	  cerr << "Error: Component instance group/item " << action_item.group << "/"
		<< action_item.item << " not found in AIF file." << endl;
	  if(!bIgnoreFatalErrors)
	  {
		_sleep(3000);
		_exit(__LINE__);
	  }
	}

	// Check for subscribe port not found in AIF file.
	if (bComponentInstanceFound && ! bActionFound) 
	{
		string strActionName = action_item.a.name();
		string strComponentName = action_item.cp.name();
		outStream << "Error: Action " << strActionName.c_str()
			<< " for component " << strComponentName.c_str() << " not found in AIF file." << endl;
		cerr << "Error: Action " << strActionName.c_str()
			<< " for component " << strComponentName.c_str() << " not found in AIF file." << endl;
 	    if(!bIgnoreFatalErrors)
		{
		 _sleep(3000);
		 _exit(__LINE__);
		}
	}

	return;
}

#endif