#ifndef IIF_SUPPORT_H
#define	IIF_SUPPORT_H


// Labels used for the various timestamps.
#define	CE_TIMESTAMP		"ce_Timestamp"
#define	PE_TIMESTAMP		"pe_Timestamp"
#define	SE_TIMESTAMP		"se_Timestamp"
#define	RM_TIMESTAMP		"rm_Timestamp"
#define	FR_TIMESTAMP		"fr_Timestamp"
#define	MD_TIMESTAMP		"md_Timestamp"

// Labels for actions associated with timestamps.
#define	LEAVE_THREAD		"leave"
#define	RETURN_TO_THREAD	"return"
#define	BEGIN_EVENT			"begin"
#define	END_EVENT			"end"
#define	BEGIN_RM			"rbegin"
#define	END_RM			    "rend"

// Label for a null data field.
#define	NO_DATA_FIELD		"***"

// The types of timestamp data available.
typedef enum timestamp_type_enum
{
  eConsumedEvent = 0,
  eSuppliedEvent,
  ePreemption,
  eRemoteMethod,
  eFrame,
  eMode,
  eNone
} TIMESTAMP_TYPE;

// The states that a subscribe port can assume.
typedef enum SubscribePortState {
	Idle = 1,
	Active = 2,
	EventPreempted = 3,
	ThreadPreempted = 4,
	EventEnd = 5
} SUBSCRIBE_PORT_STATE;

// Data contained in each line of the timestamp output file.
typedef struct TimestampInfo {
	string  strType;
	unsigned __int64 llTimestamp;
	string  strAction;
	string  strThreadID;
	string  strSubscribePort;
	bool    bFrameController;
	int     iIndex;
	IIF::Process pr;
	IIF::RemoteMethodTimeStamp rm;
	IIF::Action action;
} TIMESTAMP_INFO;

#define MAX_MODES 200

// Subscribe port status data structure.
typedef struct subscribe_port_list_item_struct
{
	int task_id;
	__int64 item;
	__int64 group;
	IIF::Component       cp;
	IIF::FrameController fc;
	IIF::SubscribePort   sp;
	IIF::PublishPort     pp;
	IIF::Process         pr;
	string  strThreadID;
	string  strPreemptor;
	unsigned __int64 llTimeStarted;
	unsigned __int64 llTimeWentActive;
	unsigned __int64 llTimePreempted;
	unsigned __int64 WCBT;
	unsigned __int64 llExecutionTime;
	unsigned __int64 llWallClockTime;
	unsigned __int64 llWCExecutionTime[MAX_MODES];
	unsigned __int64 llWCWallClockTime[MAX_MODES];
	SUBSCRIBE_PORT_STATE nCurState;
	int recursion_level;
} SUBSCRIBE_PORT_LIST_ITEM;



typedef struct action_list_item_struct
{
	int task_id;
	__int64 item;
	__int64 group;
	IIF::Component       cp;
	IIF::Process         pr;
	IIF::Action           a;
	string  strThreadID;
	string  strPreemptor;
	unsigned __int64 time_went_active;
	unsigned __int64 time_consumed;
	unsigned __int64 wcet[MAX_MODES];
	SUBSCRIBE_PORT_STATE nCurState;
} ACTION_LIST_ITEM;


typedef struct component_instance_list_item_struct
{
	__int64 item;
	__int64 group;
	IIF::Component       cp;
	IIF::FrameController fc;
	string           mode_name[MAX_MODES];
	int      current_mode;
} COMPONENT_INSTANCE_LIST_ITEM;



// References to external data structures.
extern IIF::System iif_system;
extern Udm::SmartDataNetwork* iif;
extern bool bFrameTimestamps;

// External function prototypes.
void iif_close(void);
void iif_compute_wcet(void);
void iif_init(char* iif_file);
void iif_init_timestamps(void);
typedef enum { ROLL_IN_INFRASTRUCTURE=1, CREATE_FAKE_PREEMPTIONS, SORT_STRUCTURE} IIF_PROCESS_TIMESTAMP_PASS_OPTION;
void iif_process_timestamps(IIF_PROCESS_TIMESTAMP_PASS_OPTION pass_option);
string iif_get_component_mode_name(__int64 groupid, __int64 itemid, int mode_index );
void iif_show_component_instance_list(void);



//******************* inlines *****************************************************************
static inline unsigned __int64 __s2u(unsigned __int64 old) {    return 0x0fffffffffffffff & old; }
static inline unsigned __int64 __s2u(__int64 old) {    return 0x0fffffffffffffff & old; }

static inline string __i2s(int in)
{
    char buffer [33];
	itoa (in,buffer,10);
	return buffer;
}


static inline string __i2s(unsigned __int64 in)
{
    char buffer [200];
	_ui64toa (in,buffer,10);
	return buffer;
}


static inline string __i2s(__int64 in)
{
    char buffer [200];
	_i64toa (in,buffer,10);
	return buffer;
}


extern int output_timestamps;

#endif // ifndef IIF_SUPPORT_H





