#ifdef _WIN32
#include <UdmGme.h>			// these must be included to make SmartDataNetwork work
#endif

#ifdef __INTEGRATED_INTO_OTIF__

	#include "UdmCORBA.h"
	#include "UdmCORBA_Logger.h"
	Logger logger = Logger("Translator_IIF2AIF");

#endif


#include <UdmDom.h>
#include <Uml.h>
#include <UdmStatic.h>
#include <UmlExt.h>
#include <UdmBase.h>
#include <UdmInt.h>
#include <UdmCint.h>

#include "config.h"
#include "iif_support.h"
#include "aif_support.h"

#ifndef __INTEGRATED_INTO_OTIF__
//#include "windview.h"
#endif


#define	DEFAULT_WINDVIEW_FILE	"IIF2AIFOutput.wvr"
#define	DEFAULT_OUTPUT_FILE		"IIF2AIFOutput.txt"


BOOL bWarningDetected;

int windview_process_number = 0;

//****************************************************************************************
// Local function displays program usage.
//****************************************************************************************
void Usage(char *strAppName)
{
	cout << strAppName << " [-r] [-i] <iif.xml> <aif.xml> [-w <windview.wvr>]" << endl;
	_exit(__LINE__);
}

bool bRollInInfrastructure  = FALSE;
bool bCreateFakePreemptions = FALSE;



#ifndef __INTEGRATED_INTO_OTIF__

//****************************************************************************************
// Program entry point.  Program processes the specified IIF file to determine WCETs
// then inserts the computed WCETs into the specified AIF file.
//****************************************************************************************
int main(int argc, char* argv[])
{
	char *strAIFFile = NULL;
	char *strIIFFile = NULL;
	char *strWindviewFile = DEFAULT_WINDVIEW_FILE;
	int i;

	// Reset warning flag.
	bWarningDetected = FALSE;

	// Process command line args.
	if (argc >= 3) {
		for (i=1; i<argc; i++) 
		{


			if(!strcmp(argv[i], "-w")) 
			{
				if ((++i) < argc)
					strWindviewFile = argv[i];
				 else 
					Usage(argv[0]);
				continue;
			}

			if(!strcmp(argv[i], "-r")) 
			{
				bRollInInfrastructure = TRUE;
				continue;
			}


			// undocumented
			if(!strcmp(argv[i], "-p")) 
			{
				bCreateFakePreemptions = TRUE;
				continue;
			}
 
			// undocumented
			if(!strcmp(argv[i], "-D")) 
			{
				output_timestamps = 1;
				continue;
			}


			// undocumented
			if( 
				(!strcmp(argv[i], "-0")) || (!strcmp(argv[i], "-1")) || (!strcmp(argv[i], "-2")) || (!strcmp(argv[i], "-3")) || 
				(!strcmp(argv[i], "-4")) || (!strcmp(argv[i], "-5")) || (!strcmp(argv[i], "-6")) || (!strcmp(argv[i], "-7")) || 
				(!strcmp(argv[i], "-8")) || (!strcmp(argv[i], "-9"))
			  ) 
			{
//				windview_process_number = (int)(argv[i][1] - '0');
				continue;
			}

			if(!strcmp(argv[i], "-i")) 
			{
				bIgnoreFatalErrors = TRUE;
				continue;
			}

			// undocumented (ie not in Usage()) frame timestamp debug stuff added to windview
			if(!strcmp(argv[i], "-f")) 
			{
				bFrameTimestamps = TRUE;
				continue;
			}

			if (NULL == strIIFFile) 
			{
				strIIFFile = argv[i];
                continue;
			} 
			if (NULL == strAIFFile) 
			{
				strAIFFile = argv[i];
				continue;
			} 
			printf("\n");
			Usage(argv[0]);
		}
	} else {
		Usage(argv[0]);
	}

	if ((NULL == strIIFFile) || (NULL == strAIFFile)) {
		Usage(argv[0]);
	}


	if(bFrameTimestamps && bRollInInfrastructure)
	{
		cerr << "Only -f or -r" << endl;
		cout << "Only -f or -r" << endl;
		Usage(argv[0]);
	}

	// Initialize Windview (used for debug/analysis).
//	windview_init(windview_process_number, strWindviewFile);


	cout << "IIF2AIF (built on " << __DATE__ << " at " << __TIME__ << ")" << endl;
	cout << "   invoked with: ";
	for(i=0;i<argc;i++)
	{
		cout << argv[i];
		cout << " ";
	}
    cout << endl;
    cout << endl;

	cerr << "IIF2AIF (built on " << __DATE__ << " at " << __TIME__ << ")" << endl;
	cerr << "   invoked with: ";
	for(i=0;i<argc;i++)
	{
		cerr << argv[i];
		cerr << " ";
	}
    cerr << endl;
    cerr << endl;

	// Open the IIF and AIF files and init the data objects.
	cout       << "Loading IIF file... " << endl;
	cerr << "Loading IIF file... " << endl;
	iif_init(strIIFFile);

	cout       << "Loading AIF file..." << endl;
	cerr << "Loading AIF file..." << endl;
	aif_init(strAIFFile);

	// Build a list of sorted timestamps.
	cout       << "Sorting IIF..." << endl;
	cerr << "Sorting IIF..." << endl;
	iif_init_timestamps();

    if(bRollInInfrastructure)
	{
		// overhead added
		cout << "Rolling in infrastructure overhead..." << endl;
		cerr << "Rolling in infrastructure overhead ..." << endl;
		iif_process_timestamps(ROLL_IN_INFRASTRUCTURE);
	}

    if(bCreateFakePreemptions)
	{
		// overhead added
		cout << "Creating fake preemption time stamps..." << endl;
		cerr << "Creating fake preemption time stamps ..." << endl;
		iif_process_timestamps(CREATE_FAKE_PREEMPTIONS);
	}

	cout << "Evaluating structure..." << endl;
	cerr << "Evaluating structure..." << endl;
	iif_process_timestamps(SORT_STRUCTURE);
    iif_show_component_instance_list();

	// Compute the WCETs.
	cout << "Computing WCET..." << endl;
	cerr << "Computing WCET..." << endl;
	iif_compute_wcet();

	// Check for any subscribe ports that did not get WCETs assigned.
	aif_check_for_null_wcets();

	// Close the files; update the AIF file.
	iif_close();
	aif_close();


	// Shutdown Windview.
//	windview_close();

	// Exit the program.
	if (!bWarningDetected) {
		cout << "Done." << endl;
		cerr << "Done." << endl;
	} else {
		cout << "Finished with warning(s)"<< endl;
		cerr << "Finished with warning(s)"<< endl;
	}
//    windview_close();
	_sleep(2000);
	_exit(0);

	return 0;
}

#endif


#ifdef __INTEGRATED_INTO_OTIF__

extern IIF::System iif_system;
extern AIF::System aif_system;
extern void collect_component_instances(void);
extern void collect_framecontrollers(void);
extern void collect_threads(void);


	class myTranslator : public UdmCORBA::Translator {

	public:
		myTranslator(const Udm::UdmDiagram &from_metainfo, const char *from_name, const char *from_version, const Uml::Class &from_rootclass,
				const Udm::UdmDiagram &to_metainfo, char *to_name, char *to_version, const Uml::Class &to_rootclass,
				int argc, char **argv, Logger *logger)
				: Translator(from_metainfo, from_name, from_version, from_rootclass, 
							to_metainfo, to_name, to_version, to_rootclass, 
							argc, argv, logger) {};

		void notify(const char *name, const char *version, const char *remarks, const unsigned long numOfDocs) {
		
			try {

				// Open the IIF and AIF datanetworks and init the data objects.
				iif_system = IIF::System::Cast(get_fromDN("IIF", "2.3")->GetRootObject());
				collect_component_instances();
				collect_framecontrollers();
				collect_threads();
//				iif_show_component_instance_list();
				
				aif_system = AIF::System::Cast(get_fromDN("AIF", "1.5")->GetRootObject());

				// Build a list of sorted timestamps.
				iif_init_timestamps();


				iif_process_timestamps(SORT_STRUCTURE);
//    			iif_show_component_instance_list();

				// Compute the WCETs.
				iif_compute_wcet();

				// Check for any subscribe ports that did not get WCETs assigned.
				aif_check_for_null_wcets();


				// publish the updated document
				publish(get_fromDN("AIF", "1.5"), name, version, "After the IIF->AIF update translation");

			} catch (udm_exception &e) {
				cout << "Exception: " << e.what();
			}
		};
	};

	int main(int argc, char **argv) {

		try {
			myTranslator tr(IIF_V2_3::diagram, "IIF", "2.3", IIF_V2_3::System::meta, 
							AIF::diagram, "AIF", "1.5", AIF::System::meta, 
							argc, argv, &logger);


			tr.addFromDN(AIF::diagram, "AIF", "1.5", AIF::System::meta);
			tr.run();
		} catch (udm_exception &e) {
			cout << e.what() << endl;
			logger.log(1,  "Error", e.what());
		} catch (udmcorba_exception &e) {
			cout << e.what() << endl;
			logger.log(1,  "Error", e.what());
		} catch (...) {
			cerr << "Fatal exception" << endl;
			logger.log(1,  "Error", "Fatal exception");
		}	
		return 0;
	}

#endif
