#include "UdmDom.h"
#include "UdmGme.h"
#include "UmlExt.h"

#ifdef __INTEGRATED_INTO_OTIF__

	#include "UdmCORBA.h"
	#include "UdmCORBA_Logger.h"
	Logger logger = Logger("Translator_Matlab2ECSL");

#endif

#include "ecsl.h"
#include "matlab.h"

#include <fstream>
#include <map>

#define GME_X_SCALE_FACTOR 3
#define GME_Y_SCALE_FACTOR 3
#define GME_X_OFFSET 225
#define GME_Y_OFFSET 100

string mangle(string theVal);

#define Matlab matlab

void SubSystemPass1(const matlab::Block &blk, ecsl::System &t_sys);
void SystemPass1(matlab::System &sys, ecsl::System &t_sys);
void BlockPass1(const matlab::Block &blk, ecsl::Block &t_blk);
void SystemPass2(ecsl::System &t_sys);
void ProcessNestedBranch(const matlab::NestedBranch &nb, ecsl::System &t_sys, ecsl::BranchPoint &previousBP);

void GetExtendedmatlabBlockName(matlab::Block &blk, string& longName);

void MachinePass1(matlab::machine &mach, ecsl::State &t_machState);
void ChartPass1(const matlab::chart &cht, ecsl::State &t_chtState);
void MachinePass2(ecsl::State &t_machState);
void ChartPass2(const ecsl::State &t_chtState);
void StatePass1(const matlab::state &st, ecsl::State &t_st);
void StatePass2(const ecsl::State &t_st);
void TranslateEvent(const matlab::event &ev, ecsl::Event &t_ev);
void TranslateData(const matlab::data &data_item, ecsl::Data &t_data_item);

void SetSystemPosition(const matlab::Block &blk, ecsl::System &t_sys);
void SetBlockPosition(const matlab::Block &blk, ecsl::Block &t_blk);
void SetAnnotationPosition(const matlab::Annotation &ann, ecsl::Annotation &t_ann);
void SetStatePosition(const matlab::state &st, ecsl::State &t_st);
void SetJunctionPosition(const matlab::junction &junct, ecsl::Junction &t_junct);
void SetHistoryPosition(const matlab::junction &hist, ecsl::History &t_hist);
ecsl::System SystemExplore(ecsl::System &system, string &path);

list<string> ParsePointList(string pointStr);
void ParsePointPair(const string &pointPair, int *x, int *y);
string MakeGMEPosString(int x, int y);

map<ecsl::System,matlab::System> system_map;
map<ecsl::Block,matlab::Block> block_map;
map<ecsl::Port,matlab::Port> forward_port_map;
map<matlab::Port,ecsl::Port> reverse_port_map;
map<ecsl::State,matlab::state> forward_state_map;
map<matlab::state,ecsl::State> reverse_state_map;
map<ecsl::Transition,matlab::transition> transition_map;
map<matlab::junction,ecsl::Junction> reverse_junction_map;
map<matlab::junction,ecsl::History> reverse_history_map;
map<string, string> chart_name_map;

ecsl::System ecslRootSystem;
list<ecsl::Block> block_list;

string mangle(string theVal)
{
	size_t ip;


	ip = theVal.find("\\n");
	while(ip != string::npos)
	{
		theVal.replace(ip,2,"__");
		ip = theVal.find("\\n",++ip);
	}

	ip = theVal.find("\\t");
	while(ip != string::npos)
	{
		theVal.replace(ip,2,"__");
		ip = theVal.find("\\t",++ip);
	}

	ip = theVal.find("\\r");
	while(ip != string::npos)
	{
		theVal.replace(ip,2,"__");
		ip = theVal.find("\\r",++ip);
	}

	ip = theVal.find("\\f");
	while(ip != string::npos)
	{
		theVal.replace(ip,2,"__");
		ip = theVal.find("\\f",++ip);
	}

	ip = theVal.find_first_of(" !@#$%^&*-+=\\/\'\"{}[];:.,?|()");
	while(ip != string::npos)
	{
		theVal.replace(ip,1,"_");
		ip = theVal.find_first_of(" !@#$%^&*-+=\\/\'\"{}[];:.,?|()",++ip);
	}

	return(theVal);
}

void SetSystemPosition(const matlab::Block &blk, ecsl::System &t_sys) {

	string loc = blk.Position();

	int x, y;
	string x_str;
	string y_str;
	char xbuf[5];
	char ybuf[5];
	string finalPos;

	size_t lBracketPos = loc.find("[");
	size_t firstCommaPos = loc.find(",");
	size_t secondCommaPos = loc.find(",",firstCommaPos+1);
	x_str = loc.substr(lBracketPos+1,firstCommaPos - (lBracketPos+1));
	y_str = loc.substr(firstCommaPos+1,secondCommaPos - (firstCommaPos + 1));
	x = atoi(x_str.c_str());
	y = atoi(y_str.c_str());
	x = (int) (x * GME_X_SCALE_FACTOR);
	y = (int) (y * GME_Y_SCALE_FACTOR);
	_itoa(x,xbuf,10);
	_itoa(y,ybuf,10);
	finalPos = "(";
	finalPos += xbuf;
	finalPos += ",";
	finalPos += ybuf;
	finalPos += ")";

	t_sys.position() = finalPos;
}


void SetBlockPosition(const matlab::Block &blk, ecsl::Block &t_blk)
{
	string loc = blk.Position();

	int x, y;
	string x_str;
	string y_str;
	char xbuf[5];
	char ybuf[5];
	string finalPos;

	size_t lBracketPos = loc.find("[");
	size_t firstCommaPos = loc.find(",");
	size_t secondCommaPos = loc.find(",",firstCommaPos+1);
	x_str = loc.substr(lBracketPos+1,firstCommaPos - (lBracketPos+1));
	y_str = loc.substr(firstCommaPos+1,secondCommaPos - (firstCommaPos + 1));
	x = atoi(x_str.c_str());
	y = atoi(y_str.c_str());
	x = (int) (x * GME_X_SCALE_FACTOR);
	y = (int) (y * GME_Y_SCALE_FACTOR);
	_itoa(x,xbuf,10);
	_itoa(y,ybuf,10);
	finalPos = "(";
	finalPos += xbuf;
	finalPos += ",";
	finalPos += ybuf;
	finalPos += ")";

	t_blk.position() = finalPos;
}

void SetAnnotationPosition(const matlab::Annotation &ann, ecsl::Annotation &t_ann)
{
	int x, y;
	string x_str;
	string y_str;
	char xbuf[5];
	char ybuf[5];
	string finalPos;

	string loc = ann.Position();
	size_t lBracketPos = loc.find("[");
	size_t commaPos = loc.find(",");
	size_t rBracketPos = loc.find("]");
	x_str = loc.substr(lBracketPos+1,commaPos - (lBracketPos+1));
	y_str = loc.substr(commaPos+1,rBracketPos - (commaPos + 1));
	x = atoi(x_str.c_str());
	y = atoi(y_str.c_str());
	x = (int) (x * GME_X_SCALE_FACTOR);
	y = (int) (y * GME_Y_SCALE_FACTOR);
	_itoa(x,xbuf,10);
	_itoa(y,ybuf,10);
	finalPos = "(";
	finalPos += xbuf;
	finalPos += ",";
	finalPos += ybuf;
	finalPos += ")";

	t_ann.position() = finalPos;

}

void SetStatePosition(const matlab::state &st, ecsl::State &t_st)
{
	string stateLoc = st.position();
	size_t firstSpacePos = stateLoc.find(" ");
	string stateLocX = stateLoc.substr(1,firstSpacePos - 1);
	size_t secondSpacePos = stateLoc.find(" ",firstSpacePos + 1);
	string stateLocY = stateLoc.substr(firstSpacePos + 1,secondSpacePos - 1);
	int newX = (GME_X_SCALE_FACTOR / 2) * (int) (atof(stateLocX.c_str())) + GME_X_OFFSET;
	int newY = (GME_Y_SCALE_FACTOR / 2) * (int) (atof(stateLocY.c_str())) + GME_Y_OFFSET;
	char newXbuf[5];
	char newYbuf[5];
	_itoa(newX,newXbuf,10);
	_itoa(newY,newYbuf,10);
	string newStatePos = "(";
	newStatePos += newXbuf;
	newStatePos += ",";
	newStatePos += newYbuf;
	newStatePos += ")";
	t_st.position() = newStatePos;
}

void SetJunctionPosition(const matlab::junction &junct, ecsl::Junction &t_junct)
{
	string junctLoc = junct.position();
	size_t firstSpacePos = junctLoc.find(" ");
	string junctLocX = junctLoc.substr(1,firstSpacePos - 1);
	size_t secondSpacePos = junctLoc.find(" ",firstSpacePos + 1);
	string junctLocY = junctLoc.substr(firstSpacePos + 1,secondSpacePos - 1);
	int newX = (GME_X_SCALE_FACTOR / 2) * (int) (atof(junctLocX.c_str())) + GME_X_OFFSET;
	int newY = (GME_Y_SCALE_FACTOR / 2) * (int) (atof(junctLocY.c_str())) + GME_Y_OFFSET;
	char newXbuf[5];
	char newYbuf[5];
	_itoa(newX,newXbuf,10);
	_itoa(newY,newYbuf,10);
	string newjunctPos = "(";
	newjunctPos += newXbuf;
	newjunctPos += ",";
	newjunctPos += newYbuf;
	newjunctPos += ")";
	t_junct.position() = newjunctPos;

}

void SetHistoryPosition(const matlab::junction &hist, ecsl::History &t_hist)
{
	string histLoc = hist.position();
	size_t firstSpacePos = histLoc.find(" ");
	string histLocX = histLoc.substr(1,firstSpacePos - 1);
	size_t secondSpacePos = histLoc.find(" ",firstSpacePos + 1);
	string histLocY = histLoc.substr(firstSpacePos + 1,secondSpacePos - 1);
	int newX = (GME_X_SCALE_FACTOR / 2) * (int) (atof(histLocX.c_str())) + GME_X_OFFSET;
	int newY = (GME_Y_SCALE_FACTOR / 2) * (int) (atof(histLocY.c_str())) + GME_Y_OFFSET;
	char newXbuf[5];
	char newYbuf[5];
	_itoa(newX,newXbuf,10);
	_itoa(newY,newYbuf,10);
	string newHistPos = "(";
	newHistPos += newXbuf;
	newHistPos += ",";
	newHistPos += newYbuf;
	newHistPos += ")";
	t_hist.position() = newHistPos;

}

void ParsePoints(string &pointStr, list<string> *resultList)
{
	ASSERT(resultList);		// a non-null resultList must be passed in
	size_t startPos = pointStr.find("[");
	size_t commaPos = startPos;
	size_t semiPos = startPos;
	size_t stopPos;
	string xVal;
	string yVal;
	string pairVal;
	bool done = false;

	do
	{
		commaPos = pointStr.find(",", commaPos + 1);
		semiPos = pointStr.find(";", semiPos + 1);
		if(semiPos == basic_string<char>::npos)
		{
			stopPos = pointStr.find("]");
			done = true;
			if(stopPos == basic_string<char>::npos)
			{
				// no points to parse, so return
				break;
			}
		}
		xVal = pointStr.substr(startPos+1,commaPos-(startPos+1));
		startPos = semiPos;
		if(!done)
		{
			yVal = pointStr.substr(commaPos+1,semiPos-(commaPos+1));
		}
		else
		{
			yVal = pointStr.substr(commaPos+1,stopPos-(commaPos+1));
		}

		resultList->push_back("(" + xVal + "," + yVal + ")");

	} while(!done);
	
}

void ParsePointPair(const string &pointPair, int *x, int *y)
{
	size_t commaPos;
	size_t closeParenPos;
	string xStr;
	string yStr;

	commaPos = pointPair.find(",");
	closeParenPos = pointPair.find(")");
	xStr = pointPair.substr(1,commaPos - 1);
	yStr = pointPair.substr(commaPos + 1, closeParenPos - (commaPos + 1));
	*x = atoi(xStr.c_str());
	*y = atoi(yStr.c_str());
}

string MakeGMEPosString(int x, int y)
{
	char xbuf[5];
	char ybuf[5];
	string position;

	_itoa(x,xbuf,10);
	_itoa(y,ybuf,10);

	position = "(";
	position += xbuf;
	position += ",";
	position += ybuf;
	position += ")";

	return(position);

}

void GetExtendedmatlabBlockName(matlab::Block &blk, string& longName)
{
	string currName = blk.Name();

	matlab::System parentSystem = blk.parent();
	matlab::Block parentBlockOfSystem;
	matlab::SimContainer parentSimContainer;

	try
	{
		parentBlockOfSystem = parentSystem.parentSubsystem();
	}
	catch(const udm_exception &)
	{
		parentSimContainer = parentSystem.parentContainer();
	}

	if(parentBlockOfSystem)
	{
		
		GetExtendedmatlabBlockName(parentBlockOfSystem,longName);
		longName += "/";
	}

	longName += currName;
}

void CreatePorts(const matlab::Block &blk, Udm::Object &t_obj) {
	ecsl::System t_subSys;
	ecsl::Block t_blk;

	try {
		t_blk = ecsl::Block::Cast(t_obj);
	} catch(udm_exception) {
		t_subSys = ecsl::System::Cast(t_obj);
	}
	
	// port positioning
	string inportPos = "(25,25)";
	int yIn = 25;
	char yInBuf[5];
	int yOut = 25;
	char yOutBuf[5];
	string outportPos = "(600,25)";

	// create ports
	set<matlab::Port> port_set = blk.ports();
	for(set<matlab::Port>::iterator j = port_set.begin(); j != port_set.end(); j++) {

		if(Uml::IsDerivedFrom(matlab::InputPort.meta,j->type())) {
			ecsl::InputPort t_input;
			if (t_blk) t_input = ecsl::InputPort::Create(t_blk);
			else t_input = ecsl::InputPort::Create(t_subSys);

			matlab::InputPort inpt = matlab::InputPort::Cast(*j);
			__int64 port_num = inpt.Number();
			string t_port_num;
			char port_num_buf[8];
			_i64toa(port_num,port_num_buf,10);
			t_port_num = port_num_buf;
			t_input.name() = "In" + t_port_num;
			t_input.Number() = atoi(t_port_num.c_str());
			string inpt_name = blk.Name();
			inpt_name += t_input.name();
			reverse_port_map[inpt] = t_input;
			forward_port_map[t_input] = inpt;

			t_input.position() = inportPos;
			yIn += 100;
			_itoa(yIn,yInBuf,10);
			inportPos = "(25,";
			inportPos += yInBuf;
			inportPos += ")";

		} else if(Uml::IsDerivedFrom(matlab::TriggerPort.meta,j->type()))	{
			ecsl::TriggerPort t_trigger;
			if (t_blk) t_trigger = ecsl::TriggerPort::Create(t_blk);
			else t_trigger = ecsl::TriggerPort::Create(t_subSys);
		
			matlab::TriggerPort trigt = matlab::TriggerPort::Cast(*j);
			t_trigger.name() = trigt.Name();
			string trigt_name = blk.Name();
			trigt_name += t_trigger.name();
			try{
				t_trigger.TriggerType() = trigt.TriggerType();
			}
			catch(...){}
			reverse_port_map[trigt] = t_trigger;
			forward_port_map[t_trigger] = trigt;

			t_trigger.position() = inportPos;
			yIn += 100;
			_itoa(yIn,yInBuf,10);
			inportPos = "(25,";
			inportPos += yInBuf;
			inportPos += ")";

		} else if(Uml::IsDerivedFrom(matlab::EnablePort.meta,j->type())) {
			ecsl::EnablePort t_enable;
			if (t_blk) t_enable = ecsl::EnablePort::Create(t_blk);
			else t_enable = ecsl::EnablePort::Create(t_subSys);
			
			matlab::EnablePort ent = matlab::EnablePort::Cast(*j);
			t_enable.name() = ent.Name();
			string ent_name = blk.Name();
			ent_name += t_enable.name();
			try{
				t_enable.StatesWhenEnabling() = ent.StatesWhenEnabling();
			}
			catch(...){}
			reverse_port_map[ent] = t_enable;
			forward_port_map[t_enable] = ent;

			t_enable.position() = inportPos;
			yIn += 100;
			_itoa(yIn,yInBuf,10);
			inportPos = "(25,";
			inportPos += yInBuf;
			inportPos += ")";

		} else if(Uml::IsDerivedFrom(matlab::OutputPort.meta,j->type())) {
			ecsl::OutputPort t_output;
			if (t_blk) t_output = ecsl::OutputPort::Create(t_blk);
			else t_output = ecsl::OutputPort::Create(t_subSys);

			matlab::OutputPort outpt = matlab::OutputPort::Cast(*j);
			__int64 port_num = outpt.Number();
			string t_port_num;
			char port_num_buf[8];
			_i64toa(port_num,port_num_buf,10);
			t_port_num = port_num_buf;
			t_output.name() = "Out" + t_port_num;
			t_output.Number() = atoi(t_port_num.c_str());
			string outpt_name = blk.Name();
			outpt_name += t_output.name();
			reverse_port_map[outpt] = t_output;
			forward_port_map[t_output] = outpt;

			t_output.position() = outportPos;
			yOut += 100;
			_itoa(yOut,yOutBuf,10);
			outportPos = "(600,";
			outportPos += yOutBuf;
			outportPos += ")";
		} else {
			cerr << "Port cast error - SubSystemPass1";
		}
	}
}

void SubSystemPass1(const matlab::Block &blk, ecsl::System &t_sys) {
	// ECSL without subsystem!
	matlab::System subsys_sys;
	set<matlab::Parameter> pars;

	if ( Uml::IsDerivedFrom(matlab::Normal.meta, blk.type()) ) {
		matlab::Normal norm_subsys = matlab::Normal::Cast(blk);
		subsys_sys = norm_subsys.system();
		pars = norm_subsys.parameters();
	} else if ( Uml::IsDerivedFrom(matlab::Masked.meta, blk.type()) ) {
		matlab::Masked mask_subsys = matlab::Masked::Cast(blk);
		subsys_sys = mask_subsys.system();
		pars = mask_subsys.parameters();
	} else {
		cerr << "Subsystem cast error!";
	}
			
	ecsl::System t_subSys = ecsl::System::Create(t_sys);
	t_subSys.Name() = blk.Name();
	t_subSys.name() = mangle(blk.Name());
	t_subSys.position() = "(400,400)";
			
	string paramPos = "(100,100)";
	int x = 100;
	int y = 100;
	char xbuf[5];
	char ybuf[5];

	for(set<matlab::Parameter>::iterator i = pars.begin(); i != pars.end(); i++) {
		matlab::Text text_i = matlab::Text::Cast(*i);
		ecsl::Parameter t_par = ecsl::Parameter::Create(t_subSys);
		t_par.name() = text_i.Name();
		t_par.Value() = text_i.Value();

		// positioning
		t_par.position() = paramPos;
		x += 100;
		if(x >= 500) {
			x = 100;
			y += 100;
		}
		_itoa(x,xbuf,10);
		_itoa(y,ybuf,10);
		paramPos = "(";
		paramPos += xbuf;
		paramPos += ",";
		paramPos += ybuf;
		paramPos += ")";
	}		

//  MODIFIED, we don`t want to create the ports of the subsystem, because subsystem stuff doesn`t exist :)
//	CreatePorts(blk, t_subSys);
//  ********

	SetSystemPosition(blk, t_subSys);


	SystemPass1(subsys_sys, t_subSys);
}

void SystemPass1(matlab::System &sys, ecsl::System &t_sys)
{
	system_map[t_sys] = sys;

//	cout << "Actual system: " << (string) sys.Name() << endl;
	
	// create child annotations
	int notecounter = 1;
	string numstr;
	char converter[10];
	_itoa(notecounter,converter,10);
	numstr = converter;

	set<matlab::Annotation> notes = sys.annotations();
	for(set<matlab::Annotation>::iterator i = notes.begin(); i != notes.end(); i++)
	{
		ecsl::Annotation t_note = ecsl::Annotation::Create(t_sys);
		t_note.name() = "Note" + numstr;
		notecounter++;
		_itoa(notecounter,converter,10);
		numstr = converter;
		t_note.Text() = i->Text();

		SetAnnotationPosition(*i,t_note);
	}

	// create child blocks
	int ip_num = 1;
	int op_num = 1;

	set<matlab::Block> child_blocks = sys.blocks();
	for(set<matlab::Block>::iterator j = child_blocks.begin(); j != child_blocks.end(); j++)
	{
		string blockType = j->BlockType();

		if(blockType.compare("SubSystem") == 0)
		{

			SubSystemPass1(*j, t_sys);

/*
			ecsl::Subsystem t_subsys_block = ecsl::Subsystem::Create(t_sys);
			t_subsys_block.Name() = j->Name();
			t_subsys_block.name() = mangle(j->Name());

			BlockPass1(*j, t_subsys_block);
*/
		}
		else if(blockType.compare("Reference") == 0)
		{
			ecsl::Reference t_ref_block = ecsl::Reference::Create(t_sys);
			t_ref_block.Name() = j->Name();
			t_ref_block.name() = mangle(j->Name());

			BlockPass1(*j, t_ref_block);
		}
		else
		{
			// *** if block is a port, don't want to create explicitly but associate its name
			// *** with the appropriate transin/out port
			matlab::Primitive prim_block = matlab::Primitive::Cast(*j);
			
			if ((blockType.compare("Inport") == 0))
			{
				ecsl::InputPort t_inp = ecsl::InputPort::Create(t_sys);
				t_inp.name() = mangle(prim_block.Name());


				// <exp>
				// get all matlab port children of this primitive
				// make a map entry for each one
				set<matlab::Port> prim_ports = prim_block.ports();
				for(set<matlab::Port>::iterator pi = prim_ports.begin(); pi != prim_ports.end(); pi++)
				{
					reverse_port_map[*pi] = t_inp;
				}
				// </exp>

				// now find the port it refers to in the parent block and rename it to match
				matlab::Port parent_port = prim_block.refPort();

				// LA
				if (parent_port != Udm::null)
				{
					reverse_port_map[parent_port] = t_inp;

					matlab::InputPort input_port = matlab::InputPort::Cast(parent_port);
					t_inp.Number() = input_port.Number();
				}
				else
				{
					// TBD SKN -- this case happens when the "InputPort" is present in a root-system
					// we need to determine the actual port-numbers, which is probably present as an attribute
					t_inp.Number() = ip_num++;		// automatically assign port numbers
				}

/*				
				map<matlab::Port,ecsl::Port>::iterator portPos = reverse_port_map.find(parent_port);
				if(portPos != reverse_port_map.end())
				{
					portPos->second.name() = t_inp.name();
					// 8/27/01 kwc
					// copy over the port number from the found port
					matlab::InputPort inpt;
					try{
						inpt = matlab::InputPort::Cast(parent_port);
						t_inp.Number() = inpt.Number();
					}
					catch (...){}
					// 8/27/01 kwc
					// connect the ports
					ecsl::System parent_subsystem;

					try
					{
						parent_subsystem = t_sys.parentSubSystem();
					}
					catch(const udm_exception &)
					{
//						cerr << "Caught: " << e1.what() << endl;
						ecsl::Simulink parent_simulink = t_sys.parentSimulink();
					}

					if(parent_subsystem)
					{
						ecsl::Line bridge = ecsl::Line::Create(parent_subsystem);
						bridge.src_end() = portPos->second;
						bridge.dst_end() = t_inp;
					}
				}
				else
				{
					// no matching port or no refport
					// can occur if nothing connected to a block input
//					cerr << "Input port mapping error!";
//					ASSERT(false);
				}
*/

			}
			else if(blockType.compare("TriggerPort") == 0)
			{
				ecsl::TriggerPort t_inp = ecsl::TriggerPort::Create(t_sys);
				t_inp.name() = mangle(prim_block.Name());

				// <exp>
				// get all matlab port children of this primitive
				// make a map entry for each one
				set<matlab::Port> prim_ports = prim_block.ports();
				for(set<matlab::Port>::iterator pi = prim_ports.begin(); pi != prim_ports.end(); pi++)
				{
					reverse_port_map[*pi] = t_inp;
				}
				// </exp>

				// now find the port it refers to in the parent block and rename it to match
				matlab::Port parent_port = prim_block.refPort();

				// LA
				if (parent_port != Udm::null)
				{
					reverse_port_map[parent_port] = t_inp;

					matlab::TriggerPort triggerpt = matlab::TriggerPort::Cast(parent_port);
					t_inp.TriggerType() = triggerpt.TriggerType();
				}


/*			
				map<matlab::Port,ecsl::Port>::iterator portPos = reverse_port_map.find(parent_port);
				if(portPos != reverse_port_map.end())
				{
					portPos->second.name() = t_inp.name();
					// copy over the TriggerType from the found port
					matlab::TriggerPort inpt;
					try{
						inpt = matlab::TriggerPort::Cast(parent_port);
						t_inp.TriggerType() = inpt.TriggerType();
					}
					catch (...){}
					// connect the ports
					ecsl::System parent_subsystem;

					try
					{
						parent_subsystem = t_sys.parentSubSystem();
					}
					catch(const udm_exception &)
					{
//						cerr << "Caught: " << e1.what() << endl;
						ecsl::Simulink parent_simulink = t_sys.parentSimulink();
					}

					if(parent_subsystem)
					{
						ecsl::Line bridge = ecsl::Line::Create(parent_subsystem);
						bridge.src_end() = portPos->second;
						bridge.dst_end() = t_inp;
					}
				}
				else
				{
					// no matching port or no refport
					// can occur if nothing connected to a block input
//					cerr << "Trigger port mapping error!";
//					ASSERT(false);
				}
*/
  
			}
			else if(blockType.compare("EnablePort") == 0)
			{
				ecsl::EnablePort t_inp = ecsl::EnablePort::Create(t_sys);
				t_inp.name() = mangle(prim_block.Name());

				// <exp>
				// get all matlab port children of this primitive
				// make a map entry for each one
				set<matlab::Port> prim_ports = prim_block.ports();
				for(set<matlab::Port>::iterator pi = prim_ports.begin(); pi != prim_ports.end(); pi++)
				{
					reverse_port_map[*pi] = t_inp;
				}
				// </exp>

				// now find the port it refers to in the parent block and rename it to match
				matlab::Port parent_port = prim_block.refPort();

				// LA
				if (parent_port != Udm::null)
				{
					reverse_port_map[parent_port] = t_inp;
					matlab::EnablePort inpt = matlab::EnablePort::Cast(parent_port);
					t_inp.StatesWhenEnabling() = inpt.StatesWhenEnabling();
				}


/*
				map<matlab::Port,ecsl::Port>::iterator portPos = reverse_port_map.find(parent_port);
				if(portPos != reverse_port_map.end())
				{
					portPos->second.name() = t_inp.name();
					// copy over the StatesWhenEnabling from the found port
					matlab::EnablePort inpt;
					try{
						inpt = matlab::EnablePort::Cast(parent_port);
						t_inp.StatesWhenEnabling() = inpt.StatesWhenEnabling();
					}
					catch (...){}
					// connect the ports
					ecsl::System parent_subsystem;

					try
					{
						parent_subsystem = t_sys.parentSubSystem();
					}
					catch(const udm_exception &)
					{
//						cerr << "Caught: " << e1.what() << endl;
						ecsl::Simulink parent_simulink = t_sys.parentSimulink();
					}

					if(parent_subsystem)
					{
						ecsl::Line bridge = ecsl::Line::Create(parent_subsystem);
						bridge.src_end() = portPos->second;
						bridge.dst_end() = t_inp;
					}
				}
				else
				{
					// no matching port or no refport
					// can occur if nothing connected to a block input
//					cerr << "Input port mapping error!";
//					ASSERT(false);
				}
*/

			}
			else if(blockType.compare("Outport") == 0)
			{
				ecsl::OutputPort t_outp = ecsl::OutputPort::Create(t_sys);
				t_outp.name() = mangle(prim_block.Name());

				// <exp>
				// get all matlab port children of this primitive
				// make a map entry for each one
				set<matlab::Port> prim_ports = prim_block.ports();
				for(set<matlab::Port>::iterator pi = prim_ports.begin(); pi != prim_ports.end(); pi++)
				{
					reverse_port_map[*pi] = t_outp;
				}
				// </exp>

				// now find the port it refers to in the parent block and rename it to match
				matlab::Port parent_port = prim_block.refPort();

				// LA
				if (parent_port != Udm::null)
				{
					reverse_port_map[parent_port] = t_outp;

					matlab::OutputPort output_port = matlab::OutputPort::Cast(parent_port);
					t_outp.Number() = output_port.Number();
				}
				else
				{
					t_outp.Number() = op_num++;	// see inport section above
				}
				
/*
				map<matlab::Port,ecsl::Port>::iterator portPos = reverse_port_map.find(parent_port);
				if(portPos != reverse_port_map.end())
				{
					portPos->second.name() = t_outp.name();
					// copy over the Number from the found port
					matlab::OutputPort outpt;
					try{
						outpt = matlab::OutputPort::Cast(parent_port);
						t_outp.Number() = outpt.Number();
					}
					catch (...){}
					// connect the ports
					ecsl::System parent_subsystem;

					try
					{
						parent_subsystem = t_sys.parentSubSystem();
					}
					catch(const udm_exception &)
					{
//						cerr << "Caught: " << e1.what() << endl;
						ecsl::Simulink parent_simulink = t_sys.parentSimulink();
					}

					if(parent_subsystem)
					{
						ecsl::Line bridge = ecsl::Line::Create(parent_subsystem);
						bridge.src_end() = t_outp;
						bridge.dst_end() = portPos->second;
					}
				}
				else
				{
					// no matching port or no refport
					// can occur if nothing connected to a block output
//					cerr << "Output port mapping error!";
//					ASSERT(false);
				}
*/
			}
			else
			{
				ecsl::Primitive t_prim_block = ecsl::Primitive::Create(t_sys);
				t_prim_block.Name() = j->Name();
				t_prim_block.name() = mangle(j->Name());

				BlockPass1(*j, t_prim_block);
			}
		}
	}
}

void SystemPass2(ecsl::System &t_sys)
{
	// port positioning
	string inportPos = "(25,25)";
	int y = 25;
	char ybuf[5];

//	string s_name = t_sys.Name();
//	cout << "Actual system: " << s_name << endl;

	set<ecsl::InputPort> inport_kids = t_sys.InputPort_kind_children();
	for(set<ecsl::InputPort>::iterator ip = inport_kids.begin(); ip != inport_kids.end(); ip++)
	{
			// positioning
			ip->position() = inportPos;
			y += 100;
			_itoa(y,ybuf,10);
			inportPos = "(";
			inportPos += "25";
			inportPos += ",";
			inportPos += ybuf;
			inportPos += ")";
		
	}

	set<ecsl::TriggerPort> trigport_kids = t_sys.TriggerPort_kind_children();
	for(set<ecsl::TriggerPort>::iterator tp = trigport_kids.begin(); tp != trigport_kids.end(); tp++)
	{
			// positioning
			tp->position() = inportPos;
			y += 100;
			_itoa(y,ybuf,10);
			inportPos = "(";
			inportPos += "25";
			inportPos += ",";
			inportPos += ybuf;
			inportPos += ")";
		
	}

	set<ecsl::EnablePort> enport_kids = t_sys.EnablePort_kind_children();
	for(set<ecsl::EnablePort>::iterator ep = enport_kids.begin(); ep != enport_kids.end(); ep++)
	{
			// positioning
			ep->position() = inportPos;
			y += 100;
			_itoa(y,ybuf,10);
			inportPos = "(";
			inportPos += "25";
			inportPos += ",";
			inportPos += ybuf;
			inportPos += ")";
		
	}

	
	int largestX = 600;
	y = 25;

	set<ecsl::Block> block_kids = t_sys.Block_kind_children();
	for(set<ecsl::Block>::iterator bc = block_kids.begin(); bc != block_kids.end(); bc++)
	{
		string loc = bc->position();
		size_t commaPos = loc.find(",");
		string xPos_str = loc.substr(1,commaPos - 1);
		int xPos = atoi(xPos_str.c_str());
		if(xPos > largestX)
		{
			largestX = xPos;
		}
	}

	int x = largestX + 200;
	char xbuf[5];

	_itoa(x,xbuf,10);
	string outportPos = "(";
	outportPos += xbuf;
	outportPos += ",25)";

	set<ecsl::OutputPort> outport_kids = t_sys.OutputPort_kind_children();
	for(set<ecsl::OutputPort>::iterator op = outport_kids.begin(); op != outport_kids.end(); op++)
	{
		// position output ports
		op->position() = outportPos;
		y += 100;
		_itoa(y,ybuf,10);
		outportPos = "(";
		outportPos += xbuf;
		outportPos += ",";
		outportPos += ybuf;
		outportPos += ")";

	}

	map<ecsl::System,matlab::System>::iterator im = system_map.find(t_sys);
	if(im != system_map.end())
	{
		matlab::System sys = im->second;
		set<matlab::Line> line_set = sys.lines();
		for(set<matlab::Line>::iterator li = line_set.begin(); li != line_set.end(); li++)
		{
			if(Uml::IsDerivedFrom(matlab::DirectLine.meta,li->type()))
			{
				// easier case: get the matlab ports, map to ecsl ports, and connect
				matlab::DirectLine directL = matlab::DirectLine::Cast(*li);
				matlab::Port matlabSrc = directL.srcPort();
				matlab::Port matlabDst = directL.dstPort();

//				cout << (string) matlabSrc.Name() << ":" << (string) matlabDst.Name() << endl;
				
				ecsl::Port ecslSrc;
				ecsl::Port ecslDst;

				map<matlab::Port,ecsl::Port>::iterator pm = reverse_port_map.find(matlabSrc);
				if(pm != reverse_port_map.end())
				{
					ecslSrc = pm->second;
				}
				else
				{
//					cerr << "Port map error - SystemPass2";
				}

				pm = reverse_port_map.find(matlabDst);
				if(pm != reverse_port_map.end())
				{
					ecslDst = pm->second;
				}
				else
				{
//					cerr << "Port map error - SystemPass2";
				}

//				ASSERT(ecslDst && ecslSrc);
				if((ecslSrc && ecslDst))
				{
					ecsl::Line connectLine = ecsl::Line::Create(t_sys);
					connectLine.srcLine_end() = ecslSrc;
					connectLine.dstLine_end() = ecslDst;
				}

			}
			else if(Uml::IsDerivedFrom(matlab::BranchedLine.meta,li->type()))
			{
				// connect from source to a branchpoint
				matlab::BranchedLine branchedL = matlab::BranchedLine::Cast(*li);
				matlab::Port matlabSrc = branchedL.srcPort();
				string matlabsrcportname;
				ecsl::Port ecslSrc;
				string ecslSrcPortName;

				try{
				matlabsrcportname  = matlabSrc.Name();
				}
				catch(...){}

				map<matlab::Port,ecsl::Port>::iterator pm = reverse_port_map.find(matlabSrc);
				if(pm != reverse_port_map.end()) {
					ecslSrc = pm->second;
					ecslSrcPortName = ecslSrc.name();
				} else {
					cerr << "Port map error - SystemPass2" << endl;
				}
	
				ecsl::Line toSplit = ecsl::Line::Create(t_sys);
				ecsl::BranchPoint firstBP = ecsl::BranchPoint::Create(t_sys);

				string sysName = t_sys.name();

//				if (!sysName.compare("servo_control_fast")) {
//					cout << "Here" << endl;
//				}
				
				toSplit.srcLine_end() = ecslSrc;
				toSplit.dstLine_end() = firstBP;

//#if(0)
				// position the branchpoints
				string srcLoc;
				ecsl::System blSrc;
				try
				{
					ecslSrcPortName = ecslSrc.name();
					blSrc = ecslSrc.System_parent();
					srcLoc = blSrc.position();
				}
				catch(const udm_exception &)
				{
					// port's parent must be a system, so use the port's position
//					ecsl::System sysSrc = ecslSrc.parentSystem();
//					srcLoc = sysSrc.position();
					srcLoc = ecslSrc.position();
				}

				int srcLocX = 0;
				int srcLocY = 0;
				if (!srcLoc.empty()) {
					size_t commaPos = srcLoc.find(",");
					size_t closeParenPos = srcLoc.find(")");
					string srcLocX_str = srcLoc.substr(1,commaPos - 1);
					string srcLocY_str = srcLoc.substr(commaPos+1,closeParenPos - (commaPos + 1));
					srcLocX = atoi(srcLocX_str.c_str());
					srcLocY = atoi(srcLocY_str.c_str());
				}

				// offset from upper left of the block
				if(blSrc)
				{
					// offset for a block model
					srcLocX += 120;
					srcLocY += 40;
				}
				else
				{
					// offset for a port atom
					srcLocX += 37;
					srcLocY += 14;
				}
				string points_str = branchedL.Points();
				list<string> points_list;
				ParsePoints(points_str,&points_list);

				ecsl::BranchPoint prevBP = &Udm::_null;
				ecsl::BranchPoint currBP = firstBP;
				bool firstTime = true;

				for(list<string>::iterator lit = points_list.begin(); lit != points_list.end(); lit++)
				{
					if(!firstTime)
					{
						currBP = ecsl::BranchPoint::Create(t_sys);
					}
					else
					{
						firstTime = false;
					}

					int currX, currY;
					ParsePointPair(*lit,&currX,&currY);

					// now calculate the absolute coordinates
					currX = (GME_X_SCALE_FACTOR * currX) + srcLocX;
					currY = (GME_Y_SCALE_FACTOR * currY) + srcLocY;

					currBP.position() = MakeGMEPosString(currX,currY);
					srcLocX = currX;
					srcLocY = currY;

					if(prevBP)
					{
						ecsl::Line connBPs = ecsl::Line::Create(t_sys);
						connBPs.srcLine_end() = prevBP;
						connBPs.dstLine_end() = currBP;
					}

					prevBP = currBP;
				}
//#endif
				ecsl::BranchPoint branchedLineEndBP = currBP;

				// then for each directbranch, connect from branchpoint to dst
				set<matlab::DirectBranch> db_set = branchedL.DirectBranch_kind_children();
				for(set<matlab::DirectBranch>::iterator dbi = db_set.begin(); dbi != db_set.end(); dbi++)
				{
					// for each DirectBranch, start from the end of the BranchedLine
					currBP = branchedLineEndBP;

					matlab::DirectBranch currDb = *dbi;
					matlab::Port currDb_dst = currDb.dstPort();
					ecsl::Port ecslDst;

					map<matlab::Port,ecsl::Port>::iterator pm = reverse_port_map.find(currDb_dst);
					if(pm != reverse_port_map.end())
					{
						ecslDst = pm->second;
					}
					else
					{
//						cerr << "Port map error - SystemPass2";
					}

					if(ecslDst)
					{
						ecsl::Line fromSplit = ecsl::Line::Create(t_sys);
						// originally firstBP, should be currBP for positioning
						fromSplit.srcLine_end() = currBP;

						// <inserted position code>
						ecsl::Line currConnLine = fromSplit;
						string branchedPoints_str = currDb.Points();
						list<string> db_points_list;
						ParsePoints(branchedPoints_str,&db_points_list);

						prevBP = currBP;
						firstTime = false;

						for(list<string>::iterator lit2 = db_points_list.begin(); lit2 != db_points_list.end(); lit2++)
						{
							if(!firstTime)
							{
								currBP = ecsl::BranchPoint::Create(t_sys);
							}
							else
							{
								firstTime = false;
							}

							int currX, currY;
							ParsePointPair(*lit2,&currX,&currY);

							// now calculate the absolute coordinates
							currX = (GME_X_SCALE_FACTOR * currX) + srcLocX;
							currY = (GME_Y_SCALE_FACTOR * currY) + srcLocY;

							currBP.position() = MakeGMEPosString(currX,currY);
							srcLocX = currX;
							srcLocY = currY;

//							ecsl::Line currConnLine = ecsl::Line::Create(t_sys);
//							currConnLine.src_end() = prevBP;
							currConnLine.dstLine_end() = currBP;

							currConnLine = ecsl::Line::Create(t_sys);
							prevBP = currBP;
							currConnLine.srcLine_end() = prevBP;
						}
						

						// </inserted position code>
						
						currConnLine.dstLine_end() = ecslDst;
					}
				}

				set<matlab::NestedBranch> nb_set = branchedL.NestedBranch_kind_children();
				for(set<matlab::NestedBranch>::iterator nbi = nb_set.begin(); nbi != nb_set.end(); nbi++)
				{
					ProcessNestedBranch(*nbi,t_sys,branchedLineEndBP);
				}
			}
		}
	}

/*
	ecsl::System parent_subsystem;

  try
	{
		parent_subsystem = t_sys.parentSubSystem();
	}
	catch(const udm_exception &)
	{
//		cerr << "Caught: " << e1.what() << endl;
		ecsl::Simulink parent_simulink = t_sys.parentSimulink();
	}
*/

	set<ecsl::System> subSystems = t_sys.System_kind_children();
	for(set<ecsl::System>::iterator j = subSystems.begin(); j!=subSystems.end(); j++) {
		ecsl::System sys = *j;
		SystemPass2(sys);
	}
}

void ProcessNestedBranch(const matlab::NestedBranch &nb, ecsl::System &t_sys, ecsl::BranchPoint &previousBP)
{
	if(previousBP)
	{
		ecsl::BranchPoint nestedBP;
		ecsl::BranchPoint lastBP;

		int srcLocX, srcLocY;

		ParsePointPair(previousBP.position(),&srcLocX,&srcLocY);

		string points_str = nb.Points();
		list<string> points_list;
		ParsePoints(points_str,&points_list);

		lastBP = &Udm::_null;
		nestedBP = previousBP;
		bool firstTime = true;

		for(list<string>::iterator lit = points_list.begin(); lit != points_list.end(); lit++)
		{
			if(!firstTime)
			{
				nestedBP = ecsl::BranchPoint::Create(t_sys);
			}
			else
			{
				firstTime = false;
			}

			int currX, currY;
			ParsePointPair(*lit,&currX,&currY);

			currX = (GME_X_SCALE_FACTOR * currX) + srcLocX;
			currY = (GME_Y_SCALE_FACTOR * currY) + srcLocY;

			nestedBP.position() = MakeGMEPosString(currX,currY);
			srcLocX = currX;
			srcLocY = currY;

			if(lastBP)
			{
				ecsl::Line bpConnLine = ecsl::Line::Create(t_sys);
				bpConnLine.srcLine_end() = lastBP;
				bpConnLine.dstLine_end() = nestedBP;
			}

			lastBP = nestedBP;
		}

		ecsl::BranchPoint nestedBranchEndBP = nestedBP;

		ecsl::BranchPoint currDirBP;
		ecsl::BranchPoint prevDirBP;

		set<matlab::DirectBranch> db_set = nb.DirectBranch_kind_children();
		for(set<matlab::DirectBranch>::iterator nbdbi = db_set.begin(); nbdbi != db_set.end(); nbdbi++)
		{
			// for each DirectBranch, start from the end of the NestedBranch
			currDirBP = nestedBranchEndBP;
			
			matlab::DirectBranch currDb = *nbdbi;
			matlab::Port currDb_dst = currDb.dstPort();
			ecsl::Port ecslDst;

			map<matlab::Port,ecsl::Port>::iterator pm = reverse_port_map.find(currDb_dst);
			if(pm != reverse_port_map.end())
			{
				ecslDst = pm->second;
			}
			else
			{
	//			cerr << "Port map error - SystemPass2";
			}

			if(ecslDst)
			{
				ecsl::Line fromSplit = ecsl::Line::Create(t_sys);
				fromSplit.srcLine_end() = currDirBP;

				ecsl::Line currConnLine = fromSplit;
				string branchedPoints_str = currDb.Points();
				list<string> db_points_list;
				ParsePoints(branchedPoints_str,&db_points_list);

				prevDirBP = currDirBP;
				
				for(list<string>::iterator lit2 = db_points_list.begin(); lit2 != db_points_list.end(); lit2++)
				{
					currDirBP = ecsl::BranchPoint::Create(t_sys);

					int currX, currY;
					ParsePointPair(*lit2,&currX,&currY);

					// now calculate the absolute coordinates
					currX = (GME_X_SCALE_FACTOR * currX) + srcLocX;
					currY = (GME_Y_SCALE_FACTOR * currY) + srcLocY;

					currDirBP.position() = MakeGMEPosString(currX,currY);
					srcLocX = currX;
					srcLocY = currY;
					currConnLine.dstLine_end() = currDirBP;

					currConnLine = ecsl::Line::Create(t_sys);
					prevDirBP = currDirBP;
					currConnLine.srcLine_end() = prevDirBP;

				}

				currConnLine.dstLine_end() = ecslDst;
			}
		}

		set<matlab::NestedBranch> nb_set = nb.NestedBranch_kind_children();
		for(set<matlab::NestedBranch>::iterator nbi = nb_set.begin(); nbi != nb_set.end(); nbi++)
		{
			ProcessNestedBranch(*nbi,t_sys,nestedBranchEndBP);
		}
	}
}


void BlockPass1(const matlab::Block &blk, ecsl::Block &t_blk)
{
	block_map[t_blk] = blk;
	string blockType = blk.BlockType();

	CreatePorts(blk, t_blk);

	set<matlab::Parameter> pars;

	if(blockType.compare("Reference") == 0) {
		matlab::Reference refBlk = matlab::Reference::Cast(blk);
		pars = refBlk.parameters();
	} else {
		matlab::Primitive primBlk = matlab::Primitive::Cast(blk);
		pars = primBlk.parameters();
	}

	string paramPos = "(100,100)";
	int x = 100;
	int y = 100;
	char xbuf[5];
	char ybuf[5];

	// create child parameters - currently only text handled
	for(set<matlab::Parameter>::iterator i = pars.begin(); i != pars.end(); i++) {
		matlab::Text text_i = matlab::Text::Cast(*i);
		ecsl::Parameter t_par = ecsl::Parameter::Create(t_blk);
		t_par.name() = text_i.Name();
		t_par.Value() = text_i.Value();

		// positioning
		t_par.position() = paramPos;
		x += 100;
		if(x >= 500) {
			x = 100;
			y += 100;
		}
		_itoa(x,xbuf,10);
		_itoa(y,ybuf,10);
		paramPos = "(";
		paramPos += xbuf;
		paramPos += ",";
		paramPos += ybuf;
		paramPos += ")";
	}

	// set attributes
	t_blk.BlockType() = blk.BlockType();
	string priority_str = blk.Priority();
	if (priority_str.length() > 0)
		t_blk.Priority() = atoi(priority_str.c_str());
	t_blk.Tag() = blk.Tag();
	t_blk.Description() = blk.Description();

	// set position
	SetBlockPosition(blk,t_blk);
}

void MachinePass1(matlab::machine &mach, ecsl::State &t_machState)
{
	string eventDataPos = "(75,100)";
	int y = 100;
	char ybuf[5];

	// get events
	set<matlab::event> evs = mach.events();
	for(set<matlab::event>::iterator i = evs.begin(); i != evs.end(); i++)
	{
		ecsl::Event new_event = ecsl::Event::Create(t_machState);
		TranslateEvent(*i,new_event);

		// positioning
		new_event.position() = eventDataPos;
		y += 100;
		_itoa(y,ybuf,10);
		eventDataPos = "(";
		eventDataPos += "75";
		eventDataPos += ",";
		eventDataPos += ybuf;
		eventDataPos += ")";
	
	}

	// get data
	set<matlab::data> data_set = mach.data();
	for(set<matlab::data>::iterator j = data_set.begin(); j != data_set.end(); j++)
	{
		ecsl::Data new_data = ecsl::Data::Create(t_machState);
		TranslateData(*j,new_data);

		// positioning
		new_data.position() = eventDataPos;
		y += 100;
		_itoa(y,ybuf,10);
		eventDataPos = "(";
		eventDataPos += "175";
		eventDataPos += ",";
		eventDataPos += ybuf;
		eventDataPos += ")";

	}

	string chartPos = "(150,100)";
	int chartX = 100;
	int chartY = 100;
	char chartXbuf[5];
	char chartYbuf[5];

	// get charts
	set<matlab::chart> chart_set = mach.charts();
	for(set<matlab::chart>::iterator k = chart_set.begin(); k != chart_set.end(); k++)
	{

		// Changed, check if it is valid!!!
		matlab::instance inst = (*k).instance();
		string instName = inst.name(), restofpath;
		string systemname;
		int pos=0;
		// get the system name from the front of path
		pos = instName.find_first_of("/");
		if (pos == -1) {
			systemname = instName;
			restofpath.erase();
		} else {
			systemname = instName.substr(0, pos);
			restofpath = instName.substr(pos + 1, instName.length() - pos);
		}

		// see if the system name matches the system passed in
		if (systemname.compare(ecslRootSystem.name()) == 0) {
			ecsl::System targetSystem = SystemExplore(ecslRootSystem, restofpath);
			if (!targetSystem) continue;
		}
		// *******
		
		ecsl::State new_chartState = ecsl::State::Create(t_machState);
		ChartPass1(*k,new_chartState);

		// positioning
		new_chartState.position() = chartPos;
		chartX += 200;
		if(chartX >= 800)
		{
			chartX = 100;
			chartY += 200;
		}
		_itoa(chartX,chartXbuf,10);
		_itoa(chartY,chartYbuf,10);
		chartPos = "(";
		chartPos += chartXbuf;
		chartPos += ",";
		chartPos += chartYbuf;
		chartPos += ")";

	}
}

void MachinePass2(ecsl::State &t_machState)
{
	t_machState.Decomposition() = "AND_STATE";
	
	set<ecsl::State> chart_states = t_machState.subState();
	for(set<ecsl::State>::iterator si = chart_states.begin(); si != chart_states.end(); si++)
	{
		ChartPass2(*si);
	}
}

/*
ecsl::System SystemExplore(ecsl::System &system, string &path);
ecsl::System SubsystemExplore(ecsl::System &subsystem, string &path){
	string systemname, restofpath;
	int pos=0;

	ecsl::System stateflowSys = subsystem;
	if (!stateflowSys)
		return NULL;

	// extract name of system from front of path
	pos = path.find_first_of("/");
	if (pos == -1)
		systemname = path;
	else{
		systemname = path.substr(0, pos);
//		Updated!
//		restofpath = path.substr(pos + 1, path.length() - pos);
		restofpath = path.substr(2*pos + 2, path.length() - (2*pos+1));
	}

	if (systemname.compare(stateflowSys.Name()) == 0)
		return SystemExplore(stateflowSys, restofpath);
	else
		return NULL;
}
*/

ecsl::System SystemExplore(ecsl::System &system, string &path) {
	string subsystemname, tmp_restofpath, restofpath;
	int pos=0, nextpos=0;

	// if the path is empty, then we reached the system we want,
	// so return it.
	if (path.empty())
		return system;
	// get the subsystem name from the front of path
	nextpos = path.find_first_of("/");
	if (nextpos == -1){
		// this means the path ends in a subsystem, which would be
		//	wrong because only systems contain the _SFunction_ block
		//	we are looking for.
		return NULL;
	} else {
		subsystemname = path.substr(pos, nextpos - pos);
		restofpath = path.substr(nextpos + 1, path.length() - nextpos);

		nextpos = restofpath.find_first_of("/");
		if (nextpos == -1) restofpath = "";
		else restofpath = restofpath.substr(nextpos + 1, restofpath.length() - nextpos);
		
//		Updated!
//		restofpath = path.substr(nextpos + 1, path.length() - nextpos);

		// now get the subsystem(s) in this system and look for the
		//	one specified by name
		ecsl::System subsys;
		string subsysname;

		set<ecsl::System> subsyschildren = system.System_kind_children();
		for (set<ecsl::System>::iterator ssi = subsyschildren.begin(); ssi != subsyschildren.end(); ssi++){
			subsys = *ssi;
			subsysname = subsys.Name();
			if (subsystemname.compare(subsys.Name()) == 0){
				subsystemname.erase();
				return SystemExplore(subsys, restofpath);
			}
		}
		return NULL;
	}
}

string ExtractChartName(const string &extendedchartname){
	string chartname;
	int pos;

	pos = extendedchartname.find_last_of("/");
	if (pos == -1)
		chartname = extendedchartname;
	else
		chartname = extendedchartname.substr(pos + 1, extendedchartname.length() - pos);

	return chartname;
}

void ChartPass1(const matlab::chart &cht, ecsl::State &t_chtState)
{
	t_chtState.name() = mangle(ExtractChartName(cht.name()));

	string eventDataPos = "(75,150)";
	int y = 100;
	char ybuf[5];

	// get events
	set<matlab::event> evs = cht.events();
	for(set<matlab::event>::iterator i = evs.begin(); i != evs.end(); i++)
	{
		ecsl::Event new_event = ecsl::Event::Create(t_chtState);
		TranslateEvent(*i,new_event);

		// positioning
		new_event.position() = eventDataPos;
		y += 100;
		_itoa(y,ybuf,10);
		eventDataPos = "(";
		eventDataPos += "75";
		eventDataPos += ",";
		eventDataPos += ybuf;
		eventDataPos += ")";
	
	}

	eventDataPos = "(175,150)";

	// get data
	set<matlab::data> data_set = cht.data();
	for(set<matlab::data>::iterator j = data_set.begin(); j != data_set.end(); j++)
	{
		ecsl::Data new_data = ecsl::Data::Create(t_chtState);
		TranslateData(*j,new_data);

		// positioning
		new_data.position() = eventDataPos;
		y += 100;
		_itoa(y,ybuf,10);
		eventDataPos = "(";
		eventDataPos += "175";
		eventDataPos += ",";
		eventDataPos += ybuf;
		eventDataPos += ")";

	}

	// get connective junctions
	set<matlab::junction> junct_set = cht.junctions();
	for(set<matlab::junction>::iterator m = junct_set.begin(); m != junct_set.end(); m++)
	{
		string junct_type = m->type();
		
		if(junct_type.compare("HISTORY_JUNCTION") == 0)
		{
			string chartName = cht.name();
			cerr << "Error - History found in chart: " << chartName << endl;
			ASSERT(false);
		}
		else if(junct_type.compare("CONNECTIVE_JUNCTION") == 0)
		{
			ecsl::Junction new_junct = ecsl::Junction::Create(t_chtState);
			reverse_junction_map[*m] = new_junct;
			SetJunctionPosition(*m,new_junct);
		}
		else
		{
			string chartName = cht.name();
			cerr << "Error - Unrecognized junction type: " << junct_type << "in chart: " << chartName;
			ASSERT(false);
		}
	}

	// get transitions
	set<matlab::transition> trans_set = cht.transitions();
	for(set<matlab::transition>::iterator ti = trans_set.begin(); ti != trans_set.end(); ti++)
	{
		ecsl::Transition new_trans = ecsl::Transition::Create(t_chtState);
		transition_map[new_trans] = *ti;
		// get trigger, guard, action here
		string trigger;
		string guard;
		string guardAction;
		string action;

		trigger = ti->trigger();
		guard = ti->condition();
		if(guard.length() > 0)
		{
			guard = "[";
			guard += ti->condition();
			guard += "]";
		}
		guardAction = ti->conditionAction();
		if(guardAction.length() > 0)
		{
			guard += "{";
			guard += ti->conditionAction();
			guard += "}";
		}

		action = ti->action();

		new_trans.Trigger() = trigger;
		new_trans.Guard() = guard;
		new_trans.Action() = action;
	}


	// get substates
	set<matlab::state> state_set = cht.states();
	for(set<matlab::state>::iterator k = state_set.begin(); k != state_set.end(); k++)
	{
		ecsl::State new_state = ecsl::State::Create(t_chtState);
		StatePass1(*k,new_state);

		SetStatePosition(*k,new_state);
	}

	// set up blockref
	matlab::instance inst = cht.instance();
	string instName = inst.name(), restofpath;
	string systemname;
	int pos=0;
	// get the system name from the front of path
	pos = instName.find_first_of("/");
	if (pos == -1) {
		systemname = instName;
		restofpath.erase();
	} else {
		systemname = instName.substr(0, pos);
		restofpath = instName.substr(pos + 1, instName.length() - pos);
	}
	// see if the system name matches the system passed in
	if (systemname.compare(ecslRootSystem.name()) == 0){
		ecsl::System targetSystem = SystemExplore(ecslRootSystem, restofpath);
		if (targetSystem){
			string sysname = targetSystem.name();
			set<ecsl::Block> stateflowGuts = targetSystem.Block_kind_children();
			for(set<ecsl::Block>::iterator bli = stateflowGuts.begin(); bli != stateflowGuts.end(); bli++)
			{
				string bliName = bli->Name();
				if(bliName.compare(" SFunction ") == 0)
				{
					ecsl::BlockRef blockLink = ecsl::BlockRef::Create(t_chtState);
					blockLink.ref() = *bli;
					// for Kar-Wing
					blockLink.name() = mangle(sysname + bliName);
					bli->name() = blockLink.name();
					blockLink.position() = "(10,10)";
				}

			}
		} else {
			cout << "The system specified by '" << instName << "' could not be located in the source file.  Therefore no block reference can be made from the Stateflow chart object." << endl;
		}
	} else {
		string ecslrootname = ecslRootSystem.name();
		cout << "The root system name, " << systemname << ", from the XML file does not match the name of the root system created for the ecsl network (" << ecslrootname << "). Therefore, the block reference to the respective simulink block cannot be made." << endl;
	}

}

void ChartPass2(const ecsl::State &t_chtState)
{
	set<ecsl::State> substates = t_chtState.subState();

	bool and_states = false;
	bool or_states = false;

	string decomp;

	for(set<ecsl::State>::iterator si = substates.begin(); si != substates.end(); si++)
	{
		ecsl::State currState = *si;
		matlab::state matlabState;

		map<ecsl::State,matlab::state>::iterator pos = forward_state_map.find(currState);
		if(pos != forward_state_map.end())
		{
			matlabState = pos->second;
			decomp = matlabState.type();
			if(decomp.compare("AND_STATE") == 0)
			{
				and_states = true;
			}
			else if(decomp.compare("OR_STATE") == 0)
			{
				or_states = true;
			}
			else if(decomp.compare("FUNC_STATE") == 0)
			{
				or_states = true;
			}
			else if(decomp.compare("GROUP_STATE") == 0)
			{
				or_states = true;
			}
			else
			{
				cerr << "Unknown state decomposition:" << decomp << endl;
				ASSERT(false);
			}
		}

		StatePass2(*si);
	}

	if(substates.size() == 0)
	{
		// leaf state
		decomp = "OR_STATE";
		or_states = true;
	}

	if(and_states && or_states)
	{
		string chartName = t_chtState.name();
		cerr << "Can't decide chart decomposition of " << chartName << endl;
		ASSERT(false);
	}
	else if(and_states || or_states)
	{
		t_chtState.Decomposition() = decomp;	
	}
	else
	{
		string chartName = t_chtState.name();
		cerr << "Can't decide chart decomposition of " << chartName << endl;
		ASSERT(false);
	}

	// hook up transitions, junctions, initial transition, history

	int refXpos = 275;
	char refXbuf[5];
	string refPos = "(275,50)";
	
	set<ecsl::Transition> trans_set = t_chtState.Transition_kind_children();
	for(set<ecsl::Transition>::iterator ti = trans_set.begin(); ti != trans_set.end(); ti++)
	{
		matlab::transition origTrans;
		map<ecsl::Transition,matlab::transition>::iterator tPos = transition_map.find(*ti);
		if(tPos != transition_map.end())
		{
			ecsl::TransConnector dstConn;
			ecsl::TransConnector srcConn;
			ecsl::State dstConn_parentState;
			ecsl::State srcConn_parentState;

			origTrans = tPos->second;
			// now get the src and determine if state or junction
			matlab::transitionTerminal origDst = origTrans.dst();

			matlab::state dstState;
			matlab::junction dstJunction;
			try
			{
				dstState = matlab::state::Cast(origDst);
			}
			catch(const udm_exception &)
			{
				dstJunction = matlab::junction::Cast(origDst);
			}
			
			if(dstJunction)
			{
				// look up matching ecsl junction
				map<matlab::junction,ecsl::Junction>::iterator jPos = reverse_junction_map.find(dstJunction);
				if(jPos != reverse_junction_map.end())
				{
					ecsl::Junction ecslDst = jPos->second;
					dstConn = ecslDst;

// Changed
//					dstConn_parentState = ecslDst.parent();
					dstConn_parentState = ecslDst.Junction_State_parent();
				}
				else
				{
					map<matlab::junction,ecsl::History>::iterator hPos = reverse_history_map.find(dstJunction);
					if(hPos != reverse_history_map.end())
					{
						ecsl::History ecslDst = hPos->second;
						dstConn = ecslDst;

// Changed
//						dstConn_parentState = ecslDst.parent();
						dstConn_parentState = ecslDst.History_State_parent();
					}
					else
					{
						cerr << "Destination junction not found!";
						ASSERT(false);
					}
				}
			}
			else
			{
				ASSERT(dstState);
				// look up matching ecsl state
				map<matlab::state,ecsl::State>::iterator stPos = reverse_state_map.find(dstState);
				if(stPos != reverse_state_map.end())
				{
					ecsl::State ecslDst = stPos->second;
					ecsl::TransInPort dstPort;
					// create a transinport for the state if needed and then connect to it
					set<ecsl::TransInPort> inports = ecslDst.TransInPort_kind_children();
					if(inports.size() == 0)
					{
						dstPort = ecsl::TransInPort::Create(ecslDst);
						dstPort.position() = "(50,50)";
					}
					else
					{
						for(set<ecsl::TransInPort>::iterator pi = inports.begin(); pi != inports.end(); pi++)
						{
							// there should only be one, so just grab the first
							dstPort = *pi;
							break;
						}
					}
					dstConn = dstPort;
					
// Changed
//					dstConn_parentState = dstPort.parent();
					dstConn_parentState = dstPort.TransInPort_State_parent();
					dstConn_parentState = dstConn_parentState.parentState();

				}
				else
				{
					cerr << "Destination state not found!";
					ASSERT(false);
				}
			}

			matlab::transitionTerminal origSrc = origTrans.src();
			if(!origSrc)
			{
				// must be default transition
				ecsl::TransStart defaultSrc = ecsl::TransStart::Create(t_chtState);
				srcConn = defaultSrc;

// Changed				
				srcConn_parentState = defaultSrc.TransStart_State_parent();

				defaultSrc.position() = "(175,50)";
			}
			else
			{
				matlab::state srcState;
				matlab::junction srcJunction;
				try
				{
					srcState = matlab::state::Cast(origSrc);
				}
				catch(const udm_exception &)
				{
					srcJunction = matlab::junction::Cast(origSrc);
				}
				
				if(srcJunction)
				{
					// look up matching ecsl junction
					map<matlab::junction,ecsl::Junction>::iterator jPos = reverse_junction_map.find(srcJunction);
					if(jPos != reverse_junction_map.end())
					{
						ecsl::Junction ecslSrc = jPos->second;
						srcConn = ecslSrc;

// Changed						
						srcConn_parentState = ecslSrc.Junction_State_parent();
					}
					else
					{
						map<matlab::junction,ecsl::History>::iterator hPos = reverse_history_map.find(srcJunction);
						if(hPos != reverse_history_map.end())
						{
							ecsl::History ecslSrc = hPos->second;
							srcConn = ecslSrc;

// Changed
							srcConn_parentState = ecslSrc.History_State_parent();
						}
						else
						{
							cerr << "Source junction not found!";
							ASSERT(false);
						}
					}
				}
				else
				{
					ASSERT(srcState);
					// look up matching ecsl state
					map<matlab::state,ecsl::State>::iterator stPos = reverse_state_map.find(srcState);
					if(stPos != reverse_state_map.end())
					{
						ecsl::State ecslSrc = stPos->second;
						ecsl::TransOutPort srcPort;
						// create a transoutport for the state if needed and then connect to it
						set<ecsl::TransOutPort> outports = ecslSrc.TransOutPort_kind_children();
						if(outports.size() == 0)
						{
							srcPort = ecsl::TransOutPort::Create(ecslSrc);
							srcPort.position() = "(700,50)";
						}
						else
						{
							for(set<ecsl::TransOutPort>::iterator pi = outports.begin(); pi != outports.end(); pi++)
							{
								// there should only be one, so just grab the first
								srcPort = *pi;
								break;
							}
						}
						srcConn = srcPort;

// Changed						
						srcConn_parentState = srcPort.TransOutPort_State_parent();
						srcConn_parentState = srcConn_parentState.parentState();
					}
					else
					{
						cerr << "Source state not found!";
						ASSERT(false);
					}
				}

			}

			if(srcConn_parentState == dstConn_parentState)
			{
				ti->dstTransition_end() = dstConn;
				ti->srcTransition_end() = srcConn;
			}
			else
			{
				// cross hierarchy transition; reference needed
				ecsl::Junction junctDst;
				ecsl::TransInPort transInDst;
				ecsl::History histDst;

				try
				{
					junctDst = ecsl::Junction::Cast(dstConn);
				}
				catch(const udm_exception &)
				{
					try
					{
						transInDst = ecsl::TransInPort::Cast(dstConn);
					}
					catch(const udm_exception &)
					{
						histDst = ecsl::History::Cast(dstConn);
					}

				}
				if(srcConn_parentState == t_chtState)
				{
					// create ref to dst
					ecsl::ConnectorRef dstRef = ecsl::ConnectorRef::Create(srcConn_parentState);
					dstRef.ref() = dstConn;
					ti->dstTransition_end() = dstRef;
					ti->srcTransition_end() = srcConn;

					// position ref
					dstRef.position() = refPos;
					refXpos += 100;
					_itoa(refXpos,refXbuf,10);
					refPos = "(";
					refPos += refXbuf;
					refPos += ",";
					refPos += "50";
					refPos += ")";

				}
				else if(dstConn_parentState = t_chtState)
				{
					// create ref to source 
					ecsl::ConnectorRef srcRef = ecsl::ConnectorRef::Create(dstConn_parentState);
					srcRef.ref() = srcConn;
					ti->srcTransition_end() = srcRef;
					ti->dstTransition_end() = dstConn;

					// position ref
					srcRef.position() = refPos;
					refXpos += 100;
					_itoa(refXpos,refXbuf,10);
					refPos = "(";
					refPos += refXbuf;
					refPos += ",";
					refPos += "50";
					refPos += ")";
				}
				else
				{
					// create refs to both
					ecsl::ConnectorRef dstRef = ecsl::ConnectorRef::Create(srcConn_parentState);
					ecsl::ConnectorRef srcRef = ecsl::ConnectorRef::Create(dstConn_parentState);
					dstRef.ref() = dstConn;
					srcRef.ref() = srcConn;
					ti->dstTransition_end() = dstRef;
					ti->srcTransition_end() = srcRef;

					// position srcRef
					srcRef.position() = refPos;
					refXpos += 100;
					_itoa(refXpos,refXbuf,10);
					refPos = "(";
					refPos += refXbuf;
					refPos += ",";
					refPos += "50";
					refPos += ")";

					// position dstRef
					dstRef.position() = refPos;
					refXpos += 100;
					_itoa(refXpos,refXbuf,10);
					refPos = "(";
					refPos += refXbuf;
					refPos += ",";
					refPos += "50";
					refPos += ")";

				}
			}
		}
		else
		{
			cerr << "Transition mapping failed!";
			ASSERT(false);
		}

	}

}

void StatePass1(const matlab::state &st, ecsl::State &t_st)
{
	forward_state_map[t_st] = st;
	reverse_state_map[st] = t_st;
	
	// state name now a simple attribute
	t_st.name() = st.name();

	string label;
	string tester;

	label += st.name();
	label += "/\n";

	tester = st.entryAction();
	if(tester.length() > 0)
	{
		label += "entry: ";
		label += tester;
		label += "\r\n";
	}

	tester = st.duringAction();
	if(tester.length() > 0)
	{
		label += "during: ";
		label += tester;
		label += "\r\n";
	}

	tester = st.exitAction();
	if(tester.length() > 0)
	{
		label += "exit: ";
		label += tester;
		label += "\r\n";
	}
	
	tester = st.eventActions();
	if(tester.length() > 0)
	{
		label += tester;
		label += "\r\n";
	}

	t_st.Label() = label;


	string eventDataPos = "(75,150)";
	int y = 100;
	char ybuf[5];

	// get events
	set<matlab::event> evs = st.events();
	for(set<matlab::event>::iterator i = evs.begin(); i != evs.end(); i++)
	{
		ecsl::Event new_event = ecsl::Event::Create(t_st);
		TranslateEvent(*i,new_event);

		// positioning
		new_event.position() = eventDataPos;
		y += 100;
		_itoa(y,ybuf,10);
		eventDataPos = "(";
		eventDataPos += "75";
		eventDataPos += ",";
		eventDataPos += ybuf;
		eventDataPos += ")";
	}

	eventDataPos = "(175,150)";

	// get data
	set<matlab::data> data_set = st.data();
	for(set<matlab::data>::iterator j = data_set.begin(); j != data_set.end(); j++)
	{
		ecsl::Data new_data = ecsl::Data::Create(t_st);
		TranslateData(*j,new_data);

		// positioning
		new_data.position() = eventDataPos;
		y += 100;
		_itoa(y,ybuf,10);
		eventDataPos = "(";
		eventDataPos += "175";
		eventDataPos += ",";
		eventDataPos += ybuf;
		eventDataPos += ")";

	}

	// get history and connective junctions
	set<matlab::junction> junct_set = st.junctions();
	for(set<matlab::junction>::iterator m = junct_set.begin(); m != junct_set.end(); m++)
	{
		string junct_type = m->type();
		
		if(junct_type.compare("HISTORY_JUNCTION") == 0)
		{
			ecsl::History new_history = ecsl::History::Create(t_st);
			new_history.name() = "History";
			reverse_history_map[*m] = new_history;
			SetHistoryPosition(*m,new_history);
		}
		else if(junct_type.compare("CONNECTIVE_JUNCTION") == 0)
		{
			ecsl::Junction new_junct = ecsl::Junction::Create(t_st);
			reverse_junction_map[*m] = new_junct;
			SetJunctionPosition(*m,new_junct);
		}
		else
		{
			string stateName = t_st.name();
			cerr << "Unrecognized junction type: " << junct_type << "in state " << stateName << endl;
			ASSERT(false);
		}
	}

	// get transitions
	set<matlab::transition> trans_set = st.transitions();
	for(set<matlab::transition>::iterator ti = trans_set.begin(); ti != trans_set.end(); ti++)
	{
		ecsl::Transition new_trans = ecsl::Transition::Create(t_st);
		transition_map[new_trans] = *ti;
		// get trigger, guard, action here
		string trigger;
		string guard;
		string guardAction;
		string action;

		trigger = ti->trigger();
		guard = ti->condition();
		if(guard.length() > 0)
		{
			guard = "[";
			guard += ti->condition();
			guard += "]";
		}
		guardAction = ti->conditionAction();
		if(guardAction.length() > 0)
		{
			guard += "{";
			guard += ti->conditionAction();
			guard += "}";
		}

		action = ti->action();

		new_trans.Trigger() = trigger;
		new_trans.Guard() = guard;
		new_trans.Action() = action;
	}

	// get substates
	set<matlab::state> state_set = st.subStates();
	for(set<matlab::state>::iterator k = state_set.begin(); k != state_set.end(); k++)
	{
		ecsl::State new_state = ecsl::State::Create(t_st);
		StatePass1(*k,new_state);

		SetStatePosition(*k,new_state);
	}

}

void StatePass2(const ecsl::State &t_st)
{
	set<ecsl::State> state_set = t_st.subState();
	string type = "";

	map<ecsl::State,matlab::state>::iterator pos = forward_state_map.find(t_st);
	if (pos != forward_state_map.end()) type = pos->second.type();
//	else return;
	
	if (!type.compare("AND_STATE") || !type.compare("OR_STATE")) {

		bool all_children_and_state = true;

		for(set<ecsl::State>::iterator si = state_set.begin(); si != state_set.end(); si++) {
			map<ecsl::State,matlab::state>::iterator pos = forward_state_map.find(*si);
			if(pos != forward_state_map.end()) {
				
				string children_type = pos->second.type();

				if (children_type.compare("AND_STATE")) all_children_and_state = false;
			}
		}

		if (all_children_and_state) t_st.Decomposition() = "AND_STATE";
		else t_st.Decomposition() = "OR_STATE";

	} else {
		t_st.Decomposition() = type;
	}


	// walk through the children
	for(set<ecsl::State>::iterator si = state_set.begin(); si != state_set.end(); si++) {
		// process substates
		StatePass2(*si);
	}


/*
	// Changed: it seems this stuff is totally bogus!
  
	set<ecsl::State> state_set = t_st.subState();	
	bool and_states = false;
	bool or_states = false;
	string decomp;

	for(set<ecsl::State>::iterator si = state_set.begin(); si != state_set.end(); si++)
	{
		// set decomposition
		ecsl::State currState = *si;
		matlab::state matlabState;

		map<ecsl::State,matlab::state>::iterator pos = forward_state_map.find(currState);
		if(pos != forward_state_map.end())
		{
			matlabState = pos->second;
			decomp = matlabState.type();

			if(decomp.compare("AND_STATE") == 0)
			{
				and_states = true;
			}
			else if(decomp.compare("OR_STATE") == 0)
			{
				or_states = true;
			}
			else if(decomp.compare("FUNC_STATE") == 0)
			{
				or_states = true;

				// because of FUNC_STATE leafs, the parent will be FUNC_STATE, we should restore the former value
				map<ecsl::State,matlab::state>::iterator pos = forward_state_map.find(t_st);
				if(pos != forward_state_map.end()) decomp = pos->second.type();
			}
			else if(decomp.compare("GROUP_STATE") == 0)
			{
				or_states = true;
			}
			else
			{
				string stateName = t_st.name();
				cerr << "Unknown state decomposition: " << decomp << "in state " << stateName << endl;
				ASSERT(false);
			}
		}

		// process substates
		StatePass2(*si);
	}


	// leaf state
	if(state_set.size() == 0)
	{
		map<ecsl::State,matlab::state>::iterator pos = forward_state_map.find(t_st);
		if(pos != forward_state_map.end()) decomp = pos->second.type();

		// leaf state
//		decomp = "OR_STATE";
		or_states = true;
	}

	
	if(and_states && or_states)
	{
		cerr << "Can't decide state decomposition!";
		ASSERT(false);
	}
	else if(and_states || or_states)
	{
		t_st.Decomposition() = decomp;	
	}
	else
	{
		cerr << "Can't decide state decomposition!";
		ASSERT(false);
	}
*/

	// hook up transitions, junctions, initial transition, history

	int refXpos = 275;
	char refXbuf[5];
	string refPos = "(275,50)";
	
	set<ecsl::Transition> trans_set = t_st.Transition_kind_children();
	for(set<ecsl::Transition>::iterator ti = trans_set.begin(); ti != trans_set.end(); ti++)
	{
		matlab::transition origTrans;
		map<ecsl::Transition,matlab::transition>::iterator tPos = transition_map.find(*ti);
		if(tPos != transition_map.end())
		{
			ecsl::TransConnector dstConn;
			ecsl::TransConnector srcConn;
			ecsl::State dstConn_parentState;
			ecsl::State srcConn_parentState;

			origTrans = tPos->second;
			// now get the src and determine if state or junction
			matlab::transitionTerminal origDst = origTrans.dst();

			matlab::state dstState;
			matlab::junction dstJunction;
			try
			{
				dstState = matlab::state::Cast(origDst);
			}
			catch(const udm_exception &)
			{
				dstJunction = matlab::junction::Cast(origDst);
			}
			
			if(dstJunction)
			{
				// look up matching ecsl junction
				map<matlab::junction,ecsl::Junction>::iterator jPos = reverse_junction_map.find(dstJunction);
				if(jPos != reverse_junction_map.end())
				{
					ecsl::Junction ecslDst = jPos->second;
					dstConn = ecslDst;
// Changed
					
					dstConn_parentState = ecslDst.Junction_State_parent();
				}
				else
				{
					map<matlab::junction,ecsl::History>::iterator hPos = reverse_history_map.find(dstJunction);
					if(hPos != reverse_history_map.end())
					{
						ecsl::History ecslDst = hPos->second;
						dstConn = ecslDst;

// Changed						
						dstConn_parentState = ecslDst.History_State_parent();
					}
					else
					{
						cerr << "Destination junction not found!";
						ASSERT(false);
					}
				}
			}
			else
			{
				ASSERT(dstState);
				// look up matching ecsl state
				map<matlab::state,ecsl::State>::iterator stPos = reverse_state_map.find(dstState);
				if(stPos != reverse_state_map.end())
				{
					ecsl::State ecslDst = stPos->second;
					ecsl::TransInPort dstPort;
					// create a transinport for the state if needed and then connect to it
					set<ecsl::TransInPort> inports = ecslDst.TransInPort_kind_children();
					if(inports.size() == 0)
					{
						dstPort = ecsl::TransInPort::Create(ecslDst);
						dstPort.position() = "(50,50)";
					}
					else
					{
						for(set<ecsl::TransInPort>::iterator pi = inports.begin(); pi != inports.end(); pi++)
						{
							// there should only be one, so just grab the first
							dstPort = *pi;
							break;
						}
					}
					dstConn = dstPort;

// Changed						
//					dstConn_parentState = dstPort.parent();
					dstConn_parentState = dstPort.TransInPort_State_parent();
					dstConn_parentState = dstConn_parentState.parentState();

				}
				else
				{
					cerr << "Destination state not found!";
					ASSERT(false);
				}
			}

			matlab::transitionTerminal origSrc = origTrans.src();
			if(!origSrc)
			{
				// must be default transition
				ecsl::TransStart defaultSrc = ecsl::TransStart::Create(t_st);
				srcConn = defaultSrc;

// Changed				
				srcConn_parentState = defaultSrc.TransStart_State_parent();

				defaultSrc.position() = "(175,50)";
			}
			else
			{
				matlab::state srcState;
				matlab::junction srcJunction;
				try
				{
					srcState = matlab::state::Cast(origSrc);
				}
				catch(const udm_exception &)
				{
					srcJunction = matlab::junction::Cast(origSrc);
				}
				
				if(srcJunction)
				{
					// look up matching ecsl junction
					map<matlab::junction,ecsl::Junction>::iterator jPos = reverse_junction_map.find(srcJunction);
					if(jPos != reverse_junction_map.end())
					{
						ecsl::Junction ecslSrc = jPos->second;
						srcConn = ecslSrc;

// Changed						
						srcConn_parentState = ecslSrc.Junction_State_parent();
					}
					else
					{
						map<matlab::junction,ecsl::History>::iterator hPos = reverse_history_map.find(srcJunction);
						if(hPos != reverse_history_map.end())
						{
							ecsl::History ecslSrc = hPos->second;
							srcConn = ecslSrc;

// Changed							
							srcConn_parentState = ecslSrc.History_State_parent();
						}
						else
						{
							cerr << "Source junction not found!";
							ASSERT(false);
						}
					}
				}
				else
				{
					ASSERT(srcState);
					// look up matching ecsl state
					map<matlab::state,ecsl::State>::iterator stPos = reverse_state_map.find(srcState);
					if(stPos != reverse_state_map.end())
					{
						ecsl::State ecslSrc = stPos->second;
						ecsl::TransOutPort srcPort;
						// create a transoutport for the state if needed and then connect to it
						set<ecsl::TransOutPort> outports = ecslSrc.TransOutPort_kind_children();
						if(outports.size() == 0)
						{
							srcPort = ecsl::TransOutPort::Create(ecslSrc);
							srcPort.position() = "(700,50)";
						}
						else
						{
							for(set<ecsl::TransOutPort>::iterator pi = outports.begin(); pi != outports.end(); pi++)
							{
								// there should only be one, so just grab the first
								srcPort = *pi;
								break;
							}
						}
						srcConn = srcPort;

// Changed						
						srcConn_parentState = srcPort.TransOutPort_State_parent();
						srcConn_parentState = srcConn_parentState.parentState();
					}
					else
					{
						cerr << "Source state not found!";
						ASSERT(false);
					}
				}

			}

			if(srcConn_parentState == dstConn_parentState)
			{
				ti->dstTransition_end() = dstConn;
				ti->srcTransition_end() = srcConn;
			}
			else
			{
				// cross hierarchy transition; reference needed
				ecsl::Junction junctDst;
				ecsl::TransInPort transInDst;
				ecsl::History histDst;

				try
				{
					junctDst = ecsl::Junction::Cast(dstConn);
				}
				catch(const udm_exception &)
				{
					try
					{
						transInDst = ecsl::TransInPort::Cast(dstConn);
					}
					catch(const udm_exception &)
					{
						histDst = ecsl::History::Cast(dstConn);
					}

				}
				if(srcConn_parentState == t_st)
				{
					// create ref to dst
					ecsl::ConnectorRef dstRef = ecsl::ConnectorRef::Create(srcConn_parentState);
					dstRef.ref() = dstConn;
					ti->dstTransition_end() = dstRef;
					ti->srcTransition_end() = srcConn;

					// position ref
					dstRef.position() = refPos;
					refXpos += 100;
					_itoa(refXpos,refXbuf,10);
					refPos = "(";
					refPos += refXbuf;
					refPos += ",";
					refPos += "50";
					refPos += ")";

				}
				else if(dstConn_parentState == t_st)
				{
					// create ref to source 
					ecsl::ConnectorRef srcRef = ecsl::ConnectorRef::Create(dstConn_parentState);
					srcRef.ref() = srcConn;
					ti->srcTransition_end() = srcRef;
					ti->dstTransition_end() = dstConn;

					// position ref
					srcRef.position() = refPos;
					refXpos += 100;
					_itoa(refXpos,refXbuf,10);
					refPos = "(";
					refPos += refXbuf;
					refPos += ",";
					refPos += "50";
					refPos += ")";

				}
				else
				{
					// create refs to both
					ecsl::ConnectorRef dstRef = ecsl::ConnectorRef::Create(t_st);
					ecsl::ConnectorRef srcRef = ecsl::ConnectorRef::Create(t_st);
					dstRef.ref() = dstConn;
					srcRef.ref() = srcConn;
					ti->dstTransition_end() = dstRef;
					ti->srcTransition_end() = srcRef;

					// position srcRef
					srcRef.position() = refPos;
					refXpos += 100;
					_itoa(refXpos,refXbuf,10);
					refPos = "(";
					refPos += refXbuf;
					refPos += ",";
					refPos += "50";
					refPos += ")";

					// position dstRef
					dstRef.position() = refPos;
					refXpos += 100;
					_itoa(refXpos,refXbuf,10);
					refPos = "(";
					refPos += refXbuf;
					refPos += ",";
					refPos += "50";
					refPos += ")";

				}
			}
		}
		else
		{
			cerr << "Transition mapping failed!";
			ASSERT(false);
		}

	}

}

void TranslateEvent(const matlab::event &ev, ecsl::Event &t_ev)
{
	t_ev.Name() = ev.name();
	t_ev.name() = mangle(ev.name());
	t_ev.Description() = ev.description();
	t_ev.Scope() = ev.scope();
	t_ev.Trigger() = ev.trigger();

}

void TranslateData(const matlab::data &data_item, ecsl::Data &t_data_item)
{
	t_data_item.Name() = data_item.name();
	t_data_item.name() = mangle(data_item.name());
	t_data_item.Description() = data_item.description();
	t_data_item.Scope() = data_item.scope();
	t_data_item.DataType() = data_item.dataType();
	t_data_item.Units() = data_item.units();

	matlab::props data_props = data_item.props();	
	if(data_props)
	{
		t_data_item.InitialValue() = data_props.initialValue();
	
		matlab::range data_range = data_props.range();
		if(data_range)
		{
			t_data_item.Min() = data_range.minimum();
			t_data_item.Max() = data_range.maximum();
		}
		
		matlab::array data_array = data_props.array();
		if(data_array)
		{
			t_data_item.ArraySize() = data_array.size();
			t_data_item.ArrayFirstIndex() = data_array.firstIndex();
		}
	}
}


#ifndef __INTEGRATED_INTO_OTIF__

	int main(int argc, char **argv)
	{
		if(argc < 2)
		{
			cerr << "Usage: xml2ecsl <xmlfilename> {gmeprojectname} \n";
			return 1;
		}

		cerr << "Converting " << argv[1] << " ...";

		try
		{
			// load source xml file
			UdmDom::DomDataNetwork  matlab_net(matlab::diagram);
			UdmGme::GmeDataNetwork ecsl_net(ecsl::diagram);
			{		
				matlab_net.OpenExisting(argv[1],"", Udm::CHANGES_LOST_DEFAULT);
				
				Udm::Object object = matlab_net.GetRootObject();

				matlab::Simulink top = matlab::Simulink::Cast(object);
				matlab::SimContainer rootSC = top.simContainer();
				matlab::System rootSys = rootSC.system();

				matlab::machine mach;
				matlab::Stateflow rootSF = top.stateflow();
				
				if(rootSF)
				{
					mach = rootSF.machine();
				}
				
				string projName = "MGA=";
				if(argc == 3)
				{
					projName += argv[2];
					projName += ".mga";
				}
				else
				{
					projName += argv[1];
					size_t extPos = projName.find(".xml");
					projName.replace(extPos,4,".mga");
				}

				ecsl_net.CreateNew(projName,"ecsl",ecsl::RootFolder::meta,Udm::CHANGES_PERSIST_ALWAYS);
				ecsl::RootFolder rootFdr = ecsl::RootFolder::Cast(ecsl_net.GetRootObject());
				set<ecsl::Simulink> simFdr_set = rootFdr.Simulink_kind_children();
				
				ecsl::Simulink rootSimulink = ecsl::Simulink::Create(rootFdr);
				rootSimulink.name() = "Simulink";

	//			ecsl::System t_rootSys = ecsl::System::Create(rootSimulink);
				// 8/1/01 kwc
				ecslRootSystem = ecsl::System::Create(rootSimulink);
				// 8/1/01/kwc
				ecslRootSystem.Name() = rootSys.Name();
				ecslRootSystem.name() = mangle(rootSys.Name());

				SystemPass1(rootSys,ecslRootSystem);
				SystemPass2(ecslRootSystem);

				if(mach)
				{
					ecsl::Stateflow t_rootSF = ecsl::Stateflow::Create(rootFdr);
					t_rootSF.name() = "Stateflow";

					ecsl::State t_machineState = ecsl::State::Create(t_rootSF);
					t_machineState.name() = mangle(mach.name());

					MachinePass1(mach,t_machineState);
					MachinePass2(t_machineState);
				}

				ecslRootSystem = NULL;
			}
			// 8/2/01 kwc
			chart_name_map.clear();
			// 8/2/01 kwc

			forward_port_map.clear();
			reverse_port_map.clear();
			block_map.clear();
			system_map.clear();
			forward_state_map.clear();
			reverse_state_map.clear();
			transition_map.clear();
			reverse_junction_map.clear();
			reverse_history_map.clear();

			ecsl_net.CloseWithUpdate();
		}

		catch(const udm_exception &e)
		{
			cerr << "Exception: " << e.what();
			return 1;
		}

		cerr << "Done" << endl;

		return 0;
	}

#endif


#ifdef __INTEGRATED_INTO_OTIF__

	class myTranslator : public UdmCORBA::Translator {

	public:
		myTranslator(const Udm::UdmDiagram &from_metainfo, const char *from_name, const char *from_version, const Uml::Class &from_rootclass,
				const Udm::UdmDiagram &to_metainfo, char *to_name, char *to_version, const Uml::Class &to_rootclass,
				int argc, char **argv, Logger *logger)
				: Translator(from_metainfo, from_name, from_version, from_rootclass, 
							to_metainfo, to_name, to_version, to_rootclass, 
							argc, argv, logger) {};


		void notify(const char *name, const char *version, const char *remarks, const unsigned long numOfDocs) {

			matlab::Simulink top = matlab::Simulink::Cast(fromDN->GetRootObject());
			matlab::SimContainer rootSC = top.simContainer();
			matlab::System rootSys = rootSC.system();

			matlab::machine mach;
			matlab::Stateflow rootSF = top.stateflow();

			if(rootSF) {
				mach = rootSF.machine();
			}
		
			ecsl::RootFolder rootFdr = ecsl::RootFolder::Cast(toDN->GetRootObject());
			set<ecsl::Simulink> simFdr_set = rootFdr.Simulink_kind_children();
			
			ecsl::Simulink rootSimulink = ecsl::Simulink::Create(rootFdr);
			rootSimulink.name() = "Simulink";

			ecslRootSystem = ecsl::System::Create(rootSimulink);
			ecslRootSystem.Name() = rootSys.Name();
			ecslRootSystem.name() = mangle(rootSys.Name());

			SystemPass1(rootSys,ecslRootSystem);
			SystemPass2(ecslRootSystem);

			if(mach) {
				ecsl::Stateflow t_rootSF = ecsl::Stateflow::Create(rootFdr);
				t_rootSF.name() = "Stateflow";

				ecsl::State t_machineState = ecsl::State::Create(t_rootSF);
				t_machineState.name() = mangle(mach.name());

				MachinePass1(mach,t_machineState);
				MachinePass2(t_machineState);
			}

			ecslRootSystem = NULL;

			chart_name_map.clear();
			forward_port_map.clear();
			reverse_port_map.clear();
			block_map.clear();
			system_map.clear();
			forward_state_map.clear();
			reverse_state_map.clear();
			transition_map.clear();
			reverse_junction_map.clear();
			reverse_history_map.clear();

			publish(name, version, "After the Matlab->ECSL translation");
		}
	};


	int main(int argc, char **argv) {

		try {
			myTranslator tr(matlab::diagram, "MATLAB", "1.0", matlab::Simulink::meta, 
							ecsl::diagram, "ECSL", "1.0", ecsl::RootFolder::meta, 
							argc, argv, &logger);

			tr.run();
		} catch (udm_exception &e) {
			cout << e.what() << endl;
			logger.log(1,  "Error", e.what());
		} catch (udmcorba_exception &e) {
			cout << e.what() << endl;
			logger.log(1,  "Error", e.what());
		} catch (...) {
			cerr << "Fatal exception" << endl;
			logger.log(1,  "Error", "Fatal exception");
		}	
		return 0;
	}

#endif
