#include "stdafx.h"

#include "Component.h"
/* 
// This method is usually no longer in use and does not need to be implemented,
// except when DEPRECATED_BON_INVOKE_IMPLEMENTED is defined in Component.h.
// See the top section of ComponentObj.cpp for details.
void CComponent::Invoke(CBuilder &builder,CBuilderObjectList &selected, long param)
{
	AfxMessageBox("GME Component --- Sample", MB_OK | MB_ICONSTOP);
}
*/
#pragma warning( disable : 4666 )

#include "UdmCORBA.h"
#include "NameService_dlg.h"

#include <string>
#include <map>
#include <set>

int p_id;
int t_id;

using namespace std;
using namespace UdmCORBA;

struct PARADIGM {
	string name;
	string version;
};

map<long, PARADIGM> paradigm_map;
map<long, string> tooladaptor_map;
map<long, PARADIGM> translator_map;

string backplane;
string paradigms;
string translators;
string tooladaptors;
string links;
string startup_xml;
string password = "";

bool error;

void processDiagram(const CBuilderModel *diagram);
CString getAttribute(const CBuilderObject *model, CString attribute);
void createXML();


bool send_config(int argc, char **argv, const char *password, const CBuilderModel *diagram) {
	
	ManagerObject* manager;

	try {
		manager = new ManagerObject(argc, argv);
		manager->login(password);

		processDiagram(diagram);
		createXML();
		if (error) {
			AfxMessageBox("Error: A link is not in a specific Paradigm set, so the link is not defined!", MB_OK + MB_ICONERROR);
			return true;
		}
//		AfxMessageBox(startup_xml.c_str(), MB_ICONINFORMATION);

		manager->uploadStartup(startup_xml);
		AfxMessageBox("Configuration file is sent to the Backplane server!", MB_ICONINFORMATION);

	} catch (udmcorba_exception &e) {
		MessageBox(NULL, e.what(), "Error", MB_OK + MB_ICONEXCLAMATION);
	}

	return true;
}

void release_clear() {
	error = false;
	paradigm_map.clear();
	tooladaptor_map.clear();
	translator_map.clear();
	p_id = 1;
	t_id = 1;
	backplane = "";
	paradigms = "";
	translators = "";
	tooladaptors = "";
	links = "";
	startup_xml = "";
}

void CComponent::InvokeEx(CBuilder &builder,CBuilderObject *focus, CBuilderObjectList &selected, long param) 
{
	const CBuilderModelList *diagrams = builder.GetRootFolder()->GetRootModels();
	CBuilderModel *diagram = diagrams->GetHead();

	int argc = 0;
	char **argv = new char*[argc];
	
	bool sent_or_cancel = false;

	while (!sent_or_cancel) {
		NameService_dlg ns_dlg;
		ns_dlg.m_password = password.c_str();
		int result = ns_dlg.DoModal();
		password = (LPCSTR)ns_dlg.m_password;

		if (result==IDOK) {
			release_clear();
			sent_or_cancel = send_config(argc, argv, password.c_str(), diagram);
		} else {
			sent_or_cancel = true;
		}
	}
}


// ***********************************************************************************************
//	Process GME model
// ***********************************************************************************************
long paradigmID(string name, string version) {
	for(map<long, PARADIGM>::iterator i=paradigm_map.begin(); i!=paradigm_map.end(); i++) {
		if ( !name.compare((*i).second.name) && !version.compare((*i).second.version) ) return (*i).first;
	}	
	
	error = true;
	return -1;
}

long paradigmIDlong(const CBuilderConnection *con) {

	string name = "";
	string version = "";

	CBuilderObjectList objs;
	con->GetMemberOfSets(objs);
	
	if (objs.GetCount() != 1) AfxMessageBox("Error: A paradigm should be part of exactly one set!", MB_OK + MB_ICONERROR);

	CBuilderSet *set = (CBuilderSet*) objs.GetHead();
	name = (string)getAttribute(set, "paradigmName");
	version = (string)getAttribute(set, "paradigmVersion");

	return paradigmID(name, version);
}

CString paradigmID(const CBuilderConnection *con) {

	string name = "";
	string version = "";

	CBuilderObjectList objs;
	con->GetMemberOfSets(objs);
	
	if (objs.GetCount() != 1) AfxMessageBox("Error: A paradigm should be part of exactly one set!", MB_OK + MB_ICONERROR);

	CBuilderSet *set = (CBuilderSet*) objs.GetHead();
	name = (string)getAttribute(set, "paradigmName");
	version = (string)getAttribute(set, "paradigmVersion");

	char id[16];
	itoa(paradigmID(name, version), id, 10);
	return id;
}

CString translatorID(CString name, CString version) {
	char id[16];
	for(map<long, PARADIGM>::iterator i=translator_map.begin(); i!=translator_map.end(); i++)
		if ( !name.Compare((*i).second.name.c_str()) && !version.Compare((*i).second.version.c_str()) ) {
			ltoa((*i).first, id, 10);
			return id;
		}
	return "";
}

CString tooladaptorID(CString type) {
	char id[16];
	for(map<long, string>::iterator i=tooladaptor_map.begin(); i!=tooladaptor_map.end(); i++)
		if ( !type.Compare((*i).second.c_str()) ) {
			ltoa((*i).first, id, 10);
			return id;
		}
	return "";
}

CString getAttribute(const CBuilderObject *model, CString attribute) {
	CString value;
	model->GetAttribute(attribute, value);
	return value;
}

CString getAttribute(const CBuilderSet *CBset, CString attribute) {
	CString value;
	CBset->GetAttribute(attribute, value);
	return value;
}

void processDiagram(const CBuilderModel *diagram) {

	// process paradigms
	const CBuilderSetList *sets = diagram->GetSets("Paradigm");
	POSITION pos = sets->GetHeadPosition();
	while (pos) {
		CBuilderSet *set = sets->GetNext(pos);
		PARADIGM par;
		par.name = getAttribute(set, "paradigmName");
		par.version = getAttribute(set, "paradigmVersion");
		char id[16];
		itoa(p_id, id, 10);

		paradigms += "<paradigm id=\"" + (string)id + "\" name=\"" + par.name + "\" version=\"" + par.version + \
			"\" locked=\"false\" filename=\"" + (string)getAttribute(set, "paradigmFile") + "\"/>\n";
		paradigm_map.insert(pair<const long, PARADIGM>(p_id++, par));
	}
	
	const CBuilderModelList *models = diagram->GetModels();
	pos = models->GetHeadPosition();
	while (pos) {
		CBuilderModel *model = models->GetNext(pos);

		// process tooladaptors
		if (!model->GetKindName().Compare("ToolAdaptor")) {
			CString ins;
			CString outs;
			set<long> ins_list;
			set<long> outs_list;

			const CBuilderConnectionList *in_conns = model->GetInConnections("Link");
			if (in_conns->GetCount()!=0) {
				POSITION pos2 = in_conns->GetHeadPosition();
				bool first = true;
				while (pos2) {
					CBuilderConnection *cn = in_conns->GetNext(pos2);
					long pid = paradigmIDlong(cn);
					if (ins_list.find(pid)==ins_list.end()) {
						ins_list.insert(pid);
						ins += ((!first) ? "," : "") + ((cn) ? paradigmID(cn) : "");
						if (cn) first = false;
					}
				}
			}

			map<long, CString> temp_outs;
			const CBuilderConnectionList *out_conns = model->GetOutConnections("Link");
			if (out_conns->GetCount()!=0) {
				POSITION pos2 = out_conns->GetHeadPosition();
				while (pos2) {
					CBuilderConnection *cn = out_conns->GetNext(pos2);

					map<long, CString>::iterator i = temp_outs.find(cn->GetDestination()->GetObjId());
					if (i!=temp_outs.end()) {
						(*i).second += "|" + paradigmID(cn);
					} else {
						CString pid = paradigmID(cn);
						bool found = false;
						for(map<long, CString>::iterator j=temp_outs.begin(); j!=temp_outs.end(); j++)
							if (!pid.Compare((*j).second)) found = true;

						if (!found) temp_outs.insert(pair<const long, CString>(cn->GetDestination()->GetObjId(), paradigmID(cn)));
					}
				}
			}

			bool first = true;
			for(map<long, CString>::iterator i=temp_outs.begin(); i!=temp_outs.end(); i++) {
				outs += ((!first) ? "," : "") + (*i).second;
				first = false;
			}

			char id[16];
			itoa(t_id, id, 10);
			tooladaptor_map.insert(pair<const long, string>(t_id++, (LPCSTR)getAttribute(model, "type")));

			tooladaptors += "<tooladaptor id=\"" + (CString)id + "\" type=\"" + getAttribute(model, "type") + \
				"\" inputParadigms=\"" + ins + "\" outputParadigms=\"" + outs + "\"/>\n";

		// process translators
		} else if (!model->GetKindName().Compare("Translator")) {
			CString ins;
			CString outs;
			set<long> ins_list;
			set<long> outs_list;

			const CBuilderConnectionList *in_conns = model->GetInConnections("Link");
			const CBuilderConnectionList *out_conns = model->GetOutConnections("Link");
			if (in_conns->GetCount()!=0) {
				POSITION pos2 = in_conns->GetHeadPosition();
				bool first = true;
				while (pos2) {
					CBuilderConnection *cn = in_conns->GetNext(pos2);
					long pid = paradigmIDlong(cn);
					if (ins_list.find(pid)==ins_list.end()) {
						ins_list.insert(pid);
						ins += ((!first) ? "," : "") + ((cn) ? paradigmID(cn) : "");
						if (cn) first = false;
					}
				}
			}
			if (out_conns->GetCount()!=0) {
				POSITION pos2 = out_conns->GetHeadPosition();
				bool first = true;
				while (pos2) {
					CBuilderConnection *cn = out_conns->GetNext(pos2);
					long pid = paradigmIDlong(cn);
					if (outs_list.find(pid)==outs_list.end()) {
						outs_list.insert(pid);
						outs += ((!first) ? "," : "") + ((cn) ? paradigmID(cn) : "");
						if (cn) first = false;
					}
				}
			}

			char id[16];
			itoa(t_id, id, 10);
			PARADIGM tr;
			tr.name = getAttribute(model, "name");
			tr.version = getAttribute(model, "version");
			translator_map.insert(pair<const long, PARADIGM>(t_id++, tr));

			translators += "<translator id=\"" + (CString)id + "\" name=\"" + getAttribute(model, "name") + \
				"\" version=\"" + getAttribute(model, "version") + "\" command=\"" + getAttribute(model, "translatorAction") + \
				"\" inputParadigms=\"" + ins + "\" outputParadigms=\"" + outs + "\"/>\n";

		// process backplane
		} else if (!model->GetKindName().Compare("Backplane")) {
			bool onlyonemanager;
			model->GetAttribute("onlyOneManager", onlyonemanager);

			backplane += "<backplane name=\"" + getAttribute(model, "name") + "\" version=\"" + \
				getAttribute(model, "version") + "\">\n";

			backplane += "<options password=\"" + (CString)password.c_str() + \
				"\" log_level=\"" + getAttribute(model, "logLevel") + \
				"\" uml_paradigm=\"" + getAttribute(model, "umlParadigm") + \
				"\" onlyOneManager=\"" + ((onlyonemanager) ? "true" : "false") + \
				"\" translator_documents=\"" + getAttribute(model, "translatorDocuments") + "\"/>\n";
		}
	}

	// process links
	const CBuilderConnectionList *conns = diagram->GetConnections("Link");
	pos = conns->GetHeadPosition();
	while (pos) {
		CBuilderConnection *conn = conns->GetNext(pos);
		
		CString source_id;
		CString dest_id;
		if (!(conn->GetSource()->GetKindName()).Compare("ToolAdaptor"))
			source_id = tooladaptorID(getAttribute((CBuilderModel*)conn->GetSource(), "type"));
		if (!(conn->GetSource()->GetKindName()).Compare("Translator")) 
			source_id = translatorID(getAttribute((CBuilderModel*)conn->GetSource(), "name"), getAttribute((CBuilderModel*)conn->GetSource(), "version"));
		if (!(conn->GetDestination()->GetKindName()).Compare("ToolAdaptor")) 
			dest_id = tooladaptorID(getAttribute((CBuilderModel*)conn->GetDestination(), "type"));
		if (!(conn->GetDestination()->GetKindName()).Compare("Translator")) 
			dest_id = translatorID(getAttribute((CBuilderModel*)conn->GetDestination(), "name"), getAttribute((CBuilderModel*)conn->GetDestination(), "version"));
		
		links += "<link source=\"" + source_id + "\" destination=\"" + dest_id + "\" outputParadigm=\"" + paradigmID(conn) + "\"/>\n";
	}
}


// ***********************************************************************************************
//	Create XML startup file
// ***********************************************************************************************
void createXML() {
	startup_xml += backplane;

	startup_xml += "<startup>\n";

	startup_xml += "<paradigms>\n";
	startup_xml += paradigms;
	startup_xml += "</paradigms>\n";

	startup_xml += "<translators>\n";
	startup_xml += translators;
	startup_xml += "</translators>\n";

	startup_xml += "<tooladaptors>\n";
	startup_xml += tooladaptors;
	startup_xml += "</tooladaptors>\n";

	startup_xml += "<workflow>\n";
	startup_xml += links;
	startup_xml += "</workflow>\n";

	startup_xml += "</startup>\n";
	startup_xml += "</backplane>\n";
}

